Attribute VB_Name = "Commands"
Option Explicit
'
' combo box types
'
Global Const CBODROPDOWN = 0
Global Const CBOSIMPLE = 1
Global Const CBODROPLIST = 2
'
' envelope field gap
'
Const ENV_RIGHTGAP = 400
Const ENV_LEFTGAP = 140
Const MAXSCALE = 1238


Sub ArrangeWindow(frm As Form)
'
' this routine arranges the elements of the window
' for aesthetic layout, handling toolbar, formatbar,
' and statusbar visible/invisible, and the CanvasParent and
' envelope.
'
Dim prvyRunning As Long    ' holds the running vertical position of each control
Dim prvyEnvRun As Long
Dim fRefresh As Boolean
Dim twipRefTop As Long
Dim twipRefLeft As Long
Dim twipRefHeight As Long
Dim twipRefWidth As Long
Static fInResize As Boolean

If fInResize Then Exit Sub

On Error Resume Next
        
    With frm
        prvyRunning = 0

        ' items are arranged from top to bottom.  So we
        ' need a running vertical position.  If there is
        ' no toolbar, then skip this section.

        If .menViewToolbar.CHECKED Then
            ' the toolbar is visible, so it goes first
            prvyRunning = prvyRunning + .pnlToolbar.height
        End If

        If .menViewFormatBar.CHECKED Then
            twipRefWidth = .ScaleWidth
            
            If twipRefWidth > MAXSCALE * Screen.TwipsPerPixelY Then twipRefWidth = MAXSCALE * Screen.TwipsPerPixelY

            If .FormatBar.top <> prvyRunning Or .FormatBar.left <> 0 Or .FormatBar.width <> twipRefWidth Then
                .FormatBar.Move 0, prvyRunning, twipRefWidth, .FormatBar.height
            End If
            prvyRunning = prvyRunning + .FormatBar.height
        End If

        ' size/move the envelope frame
        If .fEnvFrame_vis Then
        
            twipRefTop = .EnvFrame.top
            If twipRefTop <> prvyRunning Then
                twipRefTop = prvyRunning
                fRefresh = True
            End If
            
            twipRefLeft = .EnvFrame.left
            If Not .fLoaded And twipRefLeft <> 0 Then
                twipRefLeft = 0
                fRefresh = True
            End If
            ' resize envelope fields


            twipRefWidth = .EnvFrame.width
            
            If twipRefWidth <> .ScaleWidth Then
                twipRefWidth = .ScaleWidth
                If twipRefWidth > MAXSCALE * Screen.TwipsPerPixelY Then twipRefWidth = MAXSCALE * Screen.TwipsPerPixelY
                fRefresh = True
            End If
            
            twipRefHeight = ArrangeEnv(frm, twipRefWidth, twipRefTop)
            
            prvyRunning = prvyRunning + twipRefHeight


            If fRefresh Then
                .EnvFrame.Move twipRefLeft, twipRefTop, twipRefWidth, twipRefHeight
                fRefresh = False
            End If
        
            ' .EnvFrame.ZOrder 0
        End If

        ' handle the CanvasParent, sizing it vertically.
        twipRefTop = .CanvasParent.top
        
        If .CanvasParent.top <> prvyRunning Then 'to avoid VB4 paint bug that causes heavyweight controls on a panel to become transparent
            twipRefTop = prvyRunning
            fRefresh = True
        End If
    
        Dim twipNewSizeV As Long, twipNewSizeH As Long, fHsbCanvas As Integer, fVsbCanvas As Integer
        Dim twipWHeight As Long
        
        ' calculate new size assuming scroll bars are visible
        fHsbCanvas = False
        fVsbCanvas = False

        twipNewSizeH = .ScaleWidth
        
        If .FormLoaded Then ' not the first time we do this
        
            twipNewSizeV = .ScaleHeight - twipRefTop
            
            If .menViewStatusBar.CHECKED Then
                twipNewSizeV = twipNewSizeV - .pnlStatusBar.height
            End If
             
            
        Else ' set the canvas to the initial height
        
            twipNewSizeV = .twipInitCanvV
            
            twipWHeight = twipNewSizeV + twipRefTop

            If .menViewStatusBar.CHECKED Then
                twipWHeight = twipWHeight + .pnlStatusBar.height
            End If

             fInResize = True ' prevent loop
             
            .Move .left, .top, .width, twipWHeight + (.height - .ScaleHeight) ' resize window to init size
            
             fInResize = False
             
        End If
        
    
    
        If twipNewSizeH < .CanvMaxH Then
            ' If there's a need for a horizontal scroll bar, adjust the size Vertically
            twipNewSizeV = twipNewSizeV - .hsbCanvas.height
            fHsbCanvas = True
        End If
    
        If twipNewSizeV < .CanvMaxV And Not (fVsbCanvas) Then
            ' If there's a need for a vertical scroll bar, adjust the size horizontally
            twipNewSizeH = twipNewSizeH - .vsbCanvas.width
            fVsbCanvas = True

            ' we have to recheck the horizontal size
            If Not (fHsbCanvas) And twipNewSizeH < .CanvMaxH Then
                twipNewSizeV = twipNewSizeV - .hsbCanvas.height
                fHsbCanvas = True
            End If
        End If

        
        twipRefHeight = .CanvasParent.height
        
        ' if the window isn't resizable then refresh it anyway
        
        If .CanvasParent.height <> twipNewSizeV Or Not .fLoaded Then
            twipRefHeight = twipNewSizeV
            fRefresh = True
        End If
        
        twipRefWidth = .CanvasParent.width
         
        If .CanvasParent.width <> twipNewSizeH Or Not .fLoaded Then
            twipRefWidth = twipNewSizeH
            fRefresh = True
        End If
        
        If fRefresh Then ' do all the canvas resizing in one call
            .CanvasParent.Move 0, twipRefTop, twipRefWidth, twipRefHeight
        End If
        
        
        ' if the form has an RTF field that stretches to fit to the bottom of the canvas
        ' then re-size that rtf
        If Not .rtfStretch Is Nothing Then
            ' make sure the rtf control isn't too short,
            ' if it is expand it to fit the virtual canvas instead
            prvyRunning = IIf(.CanvMaxV < twipNewSizeV, twipNewSizeV, .CanvMaxV) - .rtfStretch.top

            .rtfStretch.height = prvyRunning

            ' make sure the rtf control isn't too skinny,
            ' if it is expand it to fit the virtual canvas instead
            prvyRunning = IIf(.CanvMaxH < twipNewSizeH, twipNewSizeH, .CanvMaxH) - .rtfStretch.left
        
            .rtfStretch.width = prvyRunning
        End If

        ' frm.CanvasParent.ZOrder 0

        ' now do the scroll bars
        .vsbCanvas.top = .CanvasParent.top
        .vsbCanvas.left = .ScaleWidth - .vsbCanvas.width
        .vsbCanvas.height = twipNewSizeV
        .vsbCanvas.LargeChange = twipNewSizeV / 2
        .vsbCanvas.max = .CanvMaxV - twipNewSizeV
        .vsbCanvas.Visible = fVsbCanvas
    
        .hsbCanvas.top = .CanvasParent.top + twipNewSizeV
        .hsbCanvas.left = 0
        .hsbCanvas.width = twipNewSizeH
        .hsbCanvas.LargeChange = twipNewSizeH / 2
        .hsbCanvas.max = .CanvMaxH - twipNewSizeH
        .hsbCanvas.Visible = fHsbCanvas

        ' Update the canvas for any changes to the scroll bars
        If .Canvas_Ctrl.left <> -.hsbCanvas.Value Or .Canvas_Ctrl.top <> -.vsbCanvas.Value Then
            .Canvas_Ctrl.Move -.hsbCanvas.Value, -.vsbCanvas.Value
        End If
    
        ' finally, the status bar

        fRefresh = Not .pnlStatusBar.Visible And .Visible

        .pnlStatusBar.Align = 0
        
        If fRefresh Then
            .pnlStatusBar.Visible = True
        End If

        .pnlStatusBar.Align = 2
        
        If fRefresh Then
            .pnlStatusBar.Visible = False
        End If
        
    
        .FormLoaded = True
        
    End With
    
End Sub



Sub ComposeCustomResponse(frm As Form, Index As Integer)
'
' launch the appropriate form
'

On Error Resume Next

    With frm
        If .fDirty Then
            '
            ' the form is dirty, so save changes, but
            ' don't close the window, then launch
            ' the new one.
            '
            Dim RetVal As Variant

            RetVal = .Window_Store(.mfobj.message, False)

            .mfobj.message.Update False, True

            '
            ' init the launch (the data gets pushed over
            ' to the new message in the InitMessage event.
            '
            Call DoVerb(.mfobj, VERB_CUSTOM1 + Index)
        
        Else
            '
            ' init the launch (the data gets pushed over
            ' to the new message in the InitMessage event.
            '
            Call DoVerb(.mfobj, VERB_CUSTOM1 + Index)
        '
        ' the form is clean, so close the
        ' window, then launch the new one.
        '
        Call FileClose(frm)
        End If
    End With
End Sub

Sub ComposeNewForm(frm As Form)
'
' This command will bring up a form
' picker for the user to pick a form.
' It will then launch the
' window for the user.
'
    ' if the user cancels then we resume next
    '
    On Error GoTo CompNewErr
    
    'Are the two cases supposed to be different? CRC

    fInDialog = True
    
    'If frm.fModal Then
        frm.mfobj.LaunchFormUI VERB_OPEN, frm.hWnd
    'Else
    '    frm.mfobj.LaunchFormUI VERB_OPEN, frm.hWnd
    'End If
    
    fInDialog = False
    Exit Sub

CompNewErr:
    fInDialog = False
    If Err <> MAPIF_LAUNCH_CANCEL Then
        Dim strErr As String

        
        strErr = GetErrorForNum(Err.Number, Err.Description)
        frmAdmin.MapifVbx1.NoEvents = True
        MsgBox LAUNCHNEWERROR & strErr
        
        frmAdmin.MapifVbx1.NoEvents = False
    End If
End Sub

Sub ComposeNewNote(frm As Form)
'
' this command bring up a standard
' Exchange 4.0 compose note.
'
    Call FormLaunch(frm.mfobj, "IPM.Note", VERB_OPEN, frm.hWnd)
            
End Sub

Sub ComposeNewPost(frm As Form)
'
' this command bring up a standard
' Exchange 4.0 post note.
'
    Call FormLaunch(frm.mfobj, "IPM.POST", VERB_OPEN, frm.hWnd)
            
End Sub

Sub ComposeNewCustom(frm As Form, Class As String)
'
' this command bring up a custom
' Compose note.
'
    Call FormLaunch(frm.mfobj, Class, VERB_OPEN, frm.hWnd)
        
End Sub

Sub ComposeGeneral(frm As Form, ComposeType As Integer)
'
' This command will bring up a new compose window.  Any main body
' text fields will be marked and indented in the standard
' method used by Capone. The new compose window will be
' addressed only to the person who sent the message.
'
On Error Resume Next
    With frm
        If .fDirty Then
            ' save changes, but don't close the
            ' window, then launch the new one.
            Dim RetVal As Variant

            '
            ' Store the current properties of the form
            '
            RetVal = .Window_Store(.mfobj.message, False)

            '
            ' flush & refresh the property cache
            '
            
            .mfobj.message.Update False, True
            
            ' init the launch (the data gets pushed over
            ' to the new message in the InitMessage event.
            Call DoVerb(.mfobj, ComposeType)
        
        Else
            ' init the launch (the data gets pushed over
            ' to the new message in the InitMessage event.
            Call DoVerb(.mfobj, ComposeType)

            ' the form is clean, so close the
            ' window, then launch the new one.
            Call FileClose(frm)
        End If
    End With
End Sub

Sub EditCopy(frm As Form)
'
' this routine copies the selected text to the clipboard.
'
Dim ctlCurrent As Control             ' the currently selected control
    
    On Error Goto EditCopyError
    Set ctlCurrent = frm.ActiveControl

    '
    ' now branch based on the type of control
    '
    If TypeOf ctlCurrent Is SSRibbon Then
        Set ctlCurrent = frm.ctrlLastActive
        
        If ctlCurrent Is Nothing Then Exit Sub
    End If
    
    If TypeOf ctlCurrent Is TextBox Then
        '
        ' it's a text box
        '
        Clipboard.Clear
        Clipboard.SetText ctlCurrent.SelText, CF_TEXT
    ElseIf TypeName(ctlCurrent) = stReCtrlClass Then
        '
        ' it's a rich text field
        '
        ctlCurrent.Action = RE_ACTION_COPY
    ElseIf TypeOf ctlCurrent Is ComboBox Then
        '
        ' it's a combo box
        '
        If (ctlCurrent.style = CBOSIMPLE) Or (ctlCurrent.style = CBODROPDOWN) Then
            Clipboard.Clear
            Clipboard.SetText ctlCurrent.SelText, CF_TEXT
        End If
    End If
    '
    ' update the edit menu
    '
    Call UpdateMenus(frm)
    Exit Sub

EditCopyError:
    MsgBox LOWMEMCLIPBOARDERR, vbExclamation
    Exit Sub
    
End Sub

Sub EditCut(frm As Form)
'
' this routine copies the selected text to the clipboard.
'
Dim ctlCurrent As Control             ' the currently selected control

    On Error Goto EditCutError
    Set ctlCurrent = frm.ActiveControl

    '
    ' now branch based on the type of control
    '
    
    If TypeOf ctlCurrent Is SSRibbon Then
        Set ctlCurrent = frm.ctrlLastActive
        If ctlCurrent Is Nothing Then Exit Sub
        
    End If
    
    If TypeOf ctlCurrent Is TextBox Then
        '
        ' it's a text box
        '
        Clipboard.Clear
        Clipboard.SetText ctlCurrent.SelText, CF_TEXT
        ctlCurrent.SelText = ""
    ElseIf TypeName(ctlCurrent) = stReCtrlClass Then
        '
        ' it's a rich text field
        '
        ctlCurrent.Action = RE_ACTION_CUT
    ElseIf TypeOf ctlCurrent Is ComboBox Then
        '
        ' it's a combo box
        '
        If (ctlCurrent.style = CBOSIMPLE) Or (ctlCurrent.style = CBODROPDOWN) Then
            Clipboard.Clear
            Clipboard.SetText ctlCurrent.SelText, CF_TEXT
            ctlCurrent.SelText = ""
            If Not frm.fSubmitted Then
                frm.grpToolBar(US_TBCUT).Enabled = False
                frm.grpToolBar(US_TBCOPY).Enabled = False
            End If
        End If
    End If
    Exit Sub

EditCutError:
    MsgBox LOWMEMCLIPBOARDERR, vbExclamation
    Exit Sub

End Sub

Sub EditPaste(frm As Form)
'
' this routine copies the selected text to the clipboard.
'
Dim ctlCurrent As Control               ' the currently selected control
    
    On Error Goto EditPasteError
    Set ctlCurrent = frm.ActiveControl

    '
    ' now branch based on the type of control
    '
    If TypeOf ctlCurrent Is SSRibbon Then
        Set ctlCurrent = frm.ctrlLastActive
        If ctlCurrent Is Nothing Then Exit Sub
    End If
    If TypeOf ctlCurrent Is TextBox Then
        '
        ' it's a text box
        '
        ctlCurrent.SelText = Clipboard.GetText(CF_TEXT)
    ElseIf TypeName(ctlCurrent) = stReCtrlClass Then
        '
        ' it's a rich text field
        '
        ctlCurrent.Action = RE_ACTION_PASTE
    ElseIf TypeOf ctlCurrent Is ComboBox Then
        '
        ' it's a combo box
        '
        If (ctlCurrent.style = CBOSIMPLE) Or (ctlCurrent.style = CBODROPDOWN) Then
            ctlCurrent.SelText = Clipboard.GetText(CF_TEXT)
        End If

    End If
    '
    ' update the edit menu
    '
    Call UpdateMenus(frm)
    Exit Sub

EditPasteError:
    MsgBox LOWMEMCLIPBOARDERR, vbExclamation
    Exit Sub

End Sub

Sub EditSelectAll(frm As Form)
'
' enabled only for the rich edit control
'
    If TypeName(frm.ActiveControl) = stReCtrlClass Then
        frm.ActiveControl.Action = RE_ACTION_SELECTALL
    ElseIf TypeOf frm.ActiveControl Is TextBox Then
        frm.ActiveControl.SelStart = 0
        frm.ActiveControl.SelLength = Len(frm.ActiveControl.Text)
        frm.menEditCut.Enabled = Not frm.ActiveControl.Locked
        frm.menEditCopy.Enabled = True
    Else
        frm.menEditSelectAll.Enabled = False
    End If
End Sub

Sub EditUndo(frm As Form)
'
' this routine performs an undo on the control
' that is currently active (if possible).
'
Dim ctlCurrent As Control                       ' the currently selected control

    Set ctlCurrent = frm.ActiveControl

    '
    ' now branch based on the type of control
    '
    
    If TypeOf ctlCurrent Is SSRibbon Then
        ctlCurrent = frm.ctrlLastActive
        If ctlCurrent Is Nothing Then Exit Sub
    End If
    
    If TypeOf ctlCurrent Is TextBox Then
        '
        ' it's a text box
        '
        Call SendMessage(ctlCurrent.hWnd, EM_UNDO, ByVal 0, ByVal 0&)
    ElseIf TypeName(ctlCurrent) = stReCtrlClass Then
        '
        ' it's a rich text field
        '
        ctlCurrent.Action = RE_ACTION_UNDO
    ElseIf TypeOf ctlCurrent Is ComboBox Then
        '
        ' it's a combo box
        '
        Call SendMessage(GetComboEditBoxhWnd(ctlCurrent), EM_UNDO, ByVal 0, ByVal 0&)
    End If
    '
    ' update the menus
    '
    Call UpdateMenus(frm)
End Sub

Sub FormLaunch(mf As Object, Class As String, pVerb As Integer, hWnd As Integer)
'
' this routine handles launching of forms
' and puts up a msg box if the launch fails
'
On Error GoTo LAUNCHERROR

    msgDestClass = Class
    
    mf.LaunchForm Class, pVerb, hWnd
    
    msgDestClass = ""
    
    Exit Sub
    
LAUNCHERROR:
Dim strErr As String

strErr = GetErrorForNum(Err.Number, Err.Description)

    frmAdmin.MapifVbx1.NoEvents = True

    MsgBox LAUNCHNEWERROR & strErr
    
    frmAdmin.MapifVbx1.NoEvents = False
End Sub

Sub FileClose(frm As Form)
'
' this routine closes the form
'
    '
    ' unload the form
    '
    On Error Resume Next
    Unload frm
        
End Sub

Sub FileDelete(frm As Form)
Dim answer As Integer
Dim newMessage As Variant

'
' delete the current message and
' unload the form
'
    On Error GoTo FILEDELERR
    
    answer = vbYes
    ' if it's dirty or it's not new
    ' then verify deletion
    '
    newMessage = IsNull(GetMapiProp(Created_Prop, frm.mfobj.message, False))
    
    
    '
    ' Make call to update attachments
    '
    
    If Not frm.ctlRichMain Is Nothing Then
        frm.ctlRichMain.Action = RE_ACTION_CHECKDIRTYFILES
    End If
    
    frmAdmin.MapifVbx1.NoEvents = True
    
    If frm.fDirty Then
        answer = MsgBox(CHANGED & CONFIRMDELETE, vbYesNo, MSGBOXTITLE)
    ElseIf Not newMessage Then
        If Not frm.mfobj.DeleteIsMove Then answer = MsgBox(CONFIRMDELETE, vbYesNo, MSGBOXTITLE)
    End If
    
    frmAdmin.MapifVbx1.NoEvents = False
    
    If answer = vbYes Then
        frm.Enabled = False ' temporarily disable the form
        frm.mfobj.IsDirty = False
        frm.mfobj.DeleteMessage (frm.hWnd)
        frm.fDirty = False  'no message so not dirty
    End If

    Exit Sub
    
FILEDELERR:
Dim strErr As String

    frm.Enabled = True 'enable the form
     
    strErr = GetErrorForNum(Err.Number, Err.Description)
    
    frmAdmin.MapifVbx1.NoEvents = True
    MsgBox (FILEDELERR & strErr)
    frmAdmin.MapifVbx1.NoEvents = False
    
End Sub

Function DoDirtyForm(frm As Form, fForce As Boolean) As Boolean
'
' this routine should be called when a possibly dirty message is dismissed
' either implicitly or explicitly AND you want the user queried
' about saving/posting...
' the force flag is used to forego any UI and allways save if possible

  Dim msgAnswer As Integer
  
  DoDirtyForm = True
  
  If Not frm.ctlRichMain Is Nothing Then
    frm.ctlRichMain.Action = RE_ACTION_CHECKDIRTYFILES
  End If
  
  If frm.fDirty Then
    '
    ' if it's a send form we ask if they want to save changes
    '
    VBSetForegroundWindow32 frm.hWnd
    
    
    If frm.frmUnloadMode = vbAppWindows Then
       
        frmAdmin.MapifVbx1.NoEvents = True
        msgAnswer = MsgBox(CLOSEOK, vbYesNo, MSGBOXTITLE)
        frmAdmin.MapifVbx1.NoEvents = False
        If msgAnswer = vbNo Then
                msgAnswer = vbCancel ' no means cancel close
        Else
                msgAnswer = vbNo ' no means cancel close
        End If
    ElseIf frm.fFormSend Then
        If frm.menFileSave.Enabled Then ' only save if save enabled
            If Not (fForce) Then
                frmAdmin.MapifVbx1.NoEvents = True
                msgAnswer = MsgBox(SAVINGSAVE, vbYesNoCancel, MSGBOXTITLE)
                frmAdmin.MapifVbx1.NoEvents = False
            Else
                msgAnswer = vbYes
            End If
            If msgAnswer = vbYes Then
                DoDirtyForm = FileSave(frm)
            End If
        Else
            If Not (fForce) Then
                frmAdmin.MapifVbx1.NoEvents = True
                msgAnswer = MsgBox(CLOSEOK, vbYesNo, MSGBOXTITLE)
                frmAdmin.MapifVbx1.NoEvents = False
            Else
                msgAnswer = vbNo
            End If
            If msgAnswer = vbNo Then msgAnswer = vbCancel ' no means cancel close
        End If
        
    Else ' otherwise we ask if they want to Post or Save changes
        If Not (fForce) Then
            frmAdmin.MapifVbx1.NoEvents = True
            If frm.fSubmitted Then
                msgAnswer = MsgBox(SAVINGSAVE, vbYesNoCancel, MSGBOXTITLE)
            Else
                msgAnswer = MsgBox(SAVINGPOST, vbYesNoCancel, MSGBOXTITLE)
            End If
            frmAdmin.MapifVbx1.NoEvents = False
        Else
            msgAnswer = vbYes
        End If
        
        If msgAnswer = vbYes Then
            DoDirtyForm = FilePost(frm)
        End If
    End If
    
    If msgAnswer = vbNo Then
      frm.fDirty = False 'clear so other chance calls don't ask again :)
      frm.mfobj.IsDirty = False
    ElseIf msgAnswer = vbCancel Then
       DoDirtyForm = False
    End If
    
  End If

End Function


Function FileSave(frm As Form) As Boolean
'
' this routine saves the current message in its
' current state, including any edits the user
' has made so far.
'
    
    FileSave = DoVerb(frm.mfobj, VERB_SAVE)

    
End Function

Function FilePost(frm As Form) As Boolean
'
' this routine posts the current message in its
' current state, including any edits the user
' has made so far. And sets the message 'sent'
' flag so Mail knows it's been submitted
'

FilePost = False

On Error GoTo POSTERR

    With frm
        If Not (.fSubmitted) Then
            .mfobj.message.Sent = True
            .mfobj.message.Unread = True
        End If
    
        .mfobj.message.TimeSent = Now ' time stamp as "sent" now
        FilePost = DoVerb(.mfobj, VERB_SAVE_VALIDATE)
    End With
    Exit Function
    
POSTERR:
    Dim strErr As String

    strErr = GetErrorForNum(Err.Number, Err.Description)
    frmAdmin.MapifVbx1.NoEvents = True
    MsgBox (VERBERROR1 & stACT_SAVE & VERBERROR2 & strErr) ' error message
    frmAdmin.MapifVbx1.NoEvents = False

End Function

Function FileSend(frm As Form) As Boolean
'
' this routine saves the message,
' verifies the recipient list,
' then sends it,
' then closes the form
'
    '
    ' send the form
    '
    FileSend = DoVerb(frm.mfobj, VERB_SAVECOMMIT_VALIDATE)
    
End Function

Sub SetReadReceipt(frm As Form, fValue As Integer)
'
' set the read receipt property
' according to the fValue parameter.
'
    On Error Resume Next ' we'll catch the real error later
    
    frm.mfobj.message.ReadReceipt = fValue
    If Not (frm.fDirty ) Then frm.mfobj.IsDirty = True
    frm.fDirty = True
    
    
End Sub

Sub SetPriority(frm As Form, Imprt As Integer)
'
' set the return receipt property
' according to the fValue parameter.
'
    On Error Resume Next ' we'll catch the real error later
    frm.mfobj.message.importance = Imprt
    If Not (frm.fDirty ) Then frm.mfobj.IsDirty = True
    frm.fDirty = True
    

End Sub


Sub REFormat(frm As Form, RE_Action As Integer)
'
' this routine brings up the format font dialog box.
'
    
    
    With frm
        If .ActiveControl.CtrlType = RE_TYPE_RTFEDIT Or .ActiveControl.CtrlType = RE_TYPE_BODY Then
            Dim ctrlTemp As Control
            
            Set ctrlTemp = .ActiveControl
            
            .ActiveControl.Action = RE_Action ' focus is lost after this
            
            ctrlTemp.SetFocus
        Else
            .FormatBar.Enabled = False
            .menFormatFont.Enabled = False
            .menFormatParagraph.Enabled = False
        End If
    End With
End Sub

Function GetComboEditBoxhWnd(pctlControl As Control) As Integer
'
' This function returns the hWnd of the edit box portion of the
' combo box control passed in.  It will return 0 on failure.
'
Dim child As Integer
Dim ClassName As String
Dim cChar As Integer

    child = GetWindow(pctlControl.hWnd, GW_CHILD)
    ClassName = Space$(50)
    cChar = GetClassName(child, ClassName, Len(ClassName))
    If UCase$(left$(ClassName, cChar)) <> UCase$(COMBOEDITCLASS) Then
        '
        ' the style of the child had a vertical
        ' scroll style (the list window), so find its sibling that doesn't.
        '
        Do
            child = GetWindow(child, GW_HWNDNEXT)
            cChar = GetClassName(child, ClassName, Len(ClassName))
        Loop Until (UCase$(left$(ClassName, cChar)) <> UCase$(COMBOEDITCLASS)) Or (child = 0)
    End If

    GetComboEditBoxhWnd = child

End Function

Function GetLongClassName(pClass As String) As String
'
' this function returns the long OLE class name for the
' short name passed in.
'
Dim prvIniEntry As String                   ' holds the ini string (which contains the class name)
Dim prvcChar As Integer                     ' holds the number of characters in the ini string
Dim prvcCommaPos As Integer                 ' holds the string position of the first comma

    prvIniEntry = Space$(100)
    prvcChar = GetProfileString(INIEMBEDGROUP, pClass, "", prvIniEntry, Len(prvIniEntry))
    prvcCommaPos = InStr(1, prvIniEntry, CI_INISEP)
    GetLongClassName = left$(prvIniEntry, prvcCommaPos - 1)
End Function


Sub HelpAbout(frm As Form)
'
' This command brings up the Help About dialog
' box. In EFD this is Form Help
'
    If FormHelpID = 0 Then 'Try Quick Help
        If frmQHelp.QHelp(1) Then '1 is the constant value for form about
            Exit Sub
        Else
        
            frmAdmin.MapifVbx1.NoEvents = True
            
            MsgBox NOHELPERR
            
            frmAdmin.MapifVbx1.NoEvents = False
            
        End If
    Else
        If UserHelpFile = "" Then
            SetUserHelpFile
        End If
        WinHelp 0, UserHelpFile, HELP_CONTEXT, FormHelpID
    End If
End Sub

Sub SetUserHelpFile()
' Find where the exe lives and set UserHelpFile to
' point to formhelp.hlp in the same directory
    UserHelpFile = App.Path & IIf(right$(App.Path, 1) = "\", "", "\") & "formhelp.hlp"
End Sub

Function fShowHelp(ctl)
' Look for help for given control (which may be a form)
' If found, this function brings up one of the following:
'   * The user-created Windows help file
'   * The standard Exchange Windows help file
'   * Quick Help composed in the Forms Designer
' Returns: whether there is any help for the control
    If Not ctl Is Nothing Then
        Dim HelpID As Long, ResID As Integer
        On Error GoTo NoHelp 'if no HelpContextID then not interested
        HelpID = ctl.HelpContextID
        On Error GoTo 0
        If HelpID > 2000000000 Then 'Use standard form help
            WinHelp 0, "efdform.hlp", HELP_CONTEXT, HelpID - 2000000000
            fShowHelp = True
        ElseIf HelpID = 0 Then 'Try Quick Help
            fShowHelp = frmQHelp.QHelp(Val(ctl.Tag))
        Else 'Try WinHelp
            If UserHelpFile = "" Then
                SetUserHelpFile
            End If
            WinHelp 0, UserHelpFile, HELP_CONTEXT, HelpID
            fShowHelp = True
        End If
    End If
NoHelp:
    Exit Function
End Function

Sub HelpForCurrentField(frm As Form)
    Dim ctlCurrent ' could be control or form

    Set ctlCurrent = frm.ActiveControl
    Do Until fShowHelp(ctlCurrent) Or ctlCurrent Is Nothing
        Set ctlCurrent = CtrlParent(frm, ctlCurrent)
    Loop
End Sub


Sub HelpGeneral(frm As Form)
'
' This command brings up the Help system for the eform
'
    fShowHelp frm
End Sub

' InsertFile not enabled for beta 2
'Sub InsertFile(frm)
'
' This command brings up a dialog box
' allowing the user to insert a file into
' the RTF body field.
'
'    If Not frm.ctlRichMain Is Nothing Then
'        If frm.ActiveControl = frm.ctlRichMain Then
'            '
'            ' pop up the common file dialog
'            ' to get the text file name
'            '
'            Const ComDlgFlags = &H1804
'            frm.ComDlg.Flags = ComDlgFlags
'            frm.ComDlg.Filter = "Text (*.TXT)|*.TXT"
'            frm.ComDlg.FileName = ""
'            frm.ComDlg.ShowOpen
'            If frm.ComDlg.FileName <> "" Then
'                frm.ctlRichMain.ActionFileName = frm.ComDlg.FileName
'                frm.ctlRichMain.Action = RE_ACTION_INSERTFILEASTEXT
'            End If
'        Else
'            frm.FormatBar.Enabled = False
'            frm.menInsertFile.Enabled = False
'            frm.menInsertMessage.Enabled = False
'        End If
'    End If
'
'End Sub

Sub ToolsCheckNames(frm As Form)
'
' This command attempts to resolve all
' email names on the To, CC, and BCC lines.
' If no matches can be made, a dialog
' showing closest matches will appear,
' allowing the user to pick the appropriate
' name.
'

    ' Check the addresses
    If Not frm.ctlRichAddr Is Nothing Then
        frm.ctlRichAddr.Action = RE_ACTION_CHECKNAMES
    End If
    
End Sub


Sub ToolsSelectNames(frm As Form)
'
' this command brings up the personal address book
'
    If Not frm.ctlRichAddr Is Nothing Then
        frm.ctlRichAddr.Action = RE_ACTION_SELECTNAMES
    End If
End Sub

Sub UpdateMenus(frm As Form)
'
' this routine handles enabling and disabling the view menu,
' cut, copy, paste, and paste special menus and toolbars
' based on the current selection.
'
Dim prvctlCurrent As Control            ' the currently selected control
Dim fUpdateStatus As Boolean

    ' Update the check marks on the View menu.  Note that these items
    ' must be shown in order for this to work.  (i.e., the visible property
    ' of a panel is meaningless unless the form it is on is shown).

    With frm
        
        .menViewToolbar.CHECKED = .pnlToolbar.Visible
        .menViewFormatBar.CHECKED = .FormatBar.Visible
        .menViewStatusBar.CHECKED = .pnlStatusBar.Visible

        ' get the current control
        Set prvctlCurrent = .ActiveControl
        
        Dim fFormat As Boolean, fCutCopy As Boolean, fPaste As Boolean, _
            fUndo As Boolean, fLocked As Boolean, fSelAll As Boolean

        
        If Not prvctlCurrent Is Nothing Then
            
            If TypeOf prvctlCurrent Is SSRibbon Then
                Set prvctlCurrent = .ctrlLastActive
            Else
                '
                ' we only update the status bar when the control changes
                '
                
                fUpdateStatus = Not (.ctrlLastActive Is prvctlCurrent)
                
                Set .ctrlLastActive = prvctlCurrent
            End If
        
            ' now branch based on the type of control
            If TypeOf prvctlCurrent Is TextBox Then
                fCutCopy = prvctlCurrent.SelLength > 0
                fPaste = Clipboard.GetFormat(CF_TEXT)
                'fUndo = SendMessage(prvctlCurrent.hWnd, EM_CANUNDO, ByVal 0, ByVal 0&)
                fSelAll = True
                fLocked = prvctlCurrent.Locked
            ElseIf TypeName(prvctlCurrent) = stReCtrlClass Then
                fCutCopy = prvctlCurrent.SelLength > 0
                fUndo = prvctlCurrent.CanUndo
                fLocked = prvctlCurrent.ReadOnly
                fPaste = prvctlCurrent.CanPaste
                fFormat = prvctlCurrent.CtrlType = RE_TYPE_RTFEDIT Or prvctlCurrent.CtrlType = RE_TYPE_BODY
                fSelAll = True
            ElseIf TypeOf prvctlCurrent Is ComboBox Then
                If (prvctlCurrent.style = CBOSIMPLE) Or (prvctlCurrent.style = CBODROPDOWN) Then
                    fCutCopy = prvctlCurrent.SelLength > 0
                Else
                    fCutCopy = False
                End If
                fPaste = Clipboard.GetFormat(CF_TEXT) And (prvctlCurrent.style <> CBODROPLIST)
                fUndo = SendMessage(GetComboEditBoxhWnd(prvctlCurrent), EM_CANUNDO, ByVal 0, ByVal 0&)
            End If
        End If

        .menFormatFont.Enabled = fFormat And Not fLocked
        .menFormatParagraph.Enabled = fFormat And Not fLocked
        .FormatBar.Enabled = fFormat And Not fLocked
        .menEditCut.Enabled = fCutCopy And Not fLocked
        .menEditCopy.Enabled = fCutCopy
        .menEditPaste.Enabled = fPaste And Not fLocked
        .menEditSelectAll.Enabled = fSelAll
        
        If Not .fSubmitted Then
            .grpToolBar(US_TBCUT).Enabled = fCutCopy And Not fLocked
            .grpToolBar(US_TBCOPY).Enabled = fCutCopy
            .grpToolBar(US_TBPASTE).Enabled = fPaste
        End If
        
        .menEditUndo.Enabled = fUndo And Not fLocked
    End With
    
    If fUpdateStatus Then SetControlStatusMessage frm
    
End Sub

Sub ViewFormatBar(frm As Form)
'
' alternately show or hide the format bar
'
    With frm
        .menViewFormatBar.CHECKED = Not .menViewFormatBar.CHECKED
        .FormatBar.Visible = .menViewFormatBar.CHECKED
    End With
End Sub

Sub ViewPrevNext(frm As Form, PrevNext As Boolean)
'
' This command closes the current message
' and displays the Next message if PrevNext is True or
' the Previous Message if PrevNext is False.
' If no other messages are availale in the choosen direction
' in the current folder "frm" is unloaded
'
    If DoDirtyForm(frm, False) Then
      On Error GoTo ActNextError
      
      frm.Enabled = False ' temporarily disable the form
      
      frm.mfobj.ActivateNext PrevNext, frm.hWnd
    End If
    
    GoTo ExitPoint
    
ActNextError:
Dim strErr As String

    frm.Enabled = True 'enable the form
    
    strErr = GetErrorForNum(Err.Number, Err.Description)
    frmAdmin.MapifVbx1.NoEvents = True
    
    MsgBox (LAUNCHNEWERROR & strErr)
    
    frmAdmin.MapifVbx1.NoEvents = False

ExitPoint:
    
End Sub

Sub ViewStatusBar(frm As Form)
'
' alternately show or hide the status bar
'
    With frm
        .menViewStatusBar.CHECKED = Not .menViewStatusBar.CHECKED
        .pnlStatusBar.Visible = .menViewStatusBar.CHECKED
    End With
End Sub

Sub ViewToolBar(frm As Form)
'
' alternately show or hide the tool bar
'
    With frm
        .menViewToolbar.CHECKED = Not .menViewToolbar.CHECKED
        .pnlToolbar.Visible = .menViewToolbar.CHECKED
    End With
End Sub

Sub ImportanceHigh(frm As Form, fToolbar As Boolean)
'
' alternately set or clear High Importance
'
    With frm
        If Not fSettingProps Then
            fSettingProps = True
            .menToolsImportanceHigh.CHECKED = Not .menToolsImportanceHigh.CHECKED
            If Not fToolbar And Not .fSubmitted Then
                .grpToolBar(US_TBPRIOHIGH).Value = Not .grpToolBar(US_TBPRIOHIGH).Value
            End If
            .menToolsImportanceLow.CHECKED = False
            If Not .fSubmitted Then .grpToolBar(US_TBPRIOLOW).Value = False
            Call SetPriority(frm, IIf(.menToolsImportanceHigh.CHECKED, PRIO_HIGH, PRIO_NORMAL))
        fSettingProps = False
        End If
    End With
End Sub

Sub ImportanceLow(frm As Form, fToolbar As Boolean)
'
' alternately set or clear Low Importance
'
    With frm
        If Not fSettingProps Then
            fSettingProps = True
            .menToolsImportanceLow.CHECKED = Not .menToolsImportanceLow.CHECKED
            If Not fToolbar And Not .fSubmitted Then
                .grpToolBar(US_TBPRIOLOW).Value = Not .grpToolBar(US_TBPRIOLOW).Value
            End If
            .menToolsImportanceHigh.CHECKED = False
            If Not .fSubmitted Then .grpToolBar(US_TBPRIOHIGH).Value = False
            Call SetPriority(frm, IIf(.menToolsImportanceLow.CHECKED, PRIO_LOW, PRIO_NORMAL))
        fSettingProps = False
        End If
    End With
End Sub

Sub ReadReceipt(frm As Form, fToolbar As Boolean)
'
' alternately set or clear Read Receipt property
'
    With frm
        If Not fSettingProps Then
            fSettingProps = True
            .menToolsReadReceipt.CHECKED = Not .menToolsReadReceipt.CHECKED
            If Not fToolbar And Not .fSubmitted Then
                .grpToolBar(US_TBRECEIPT).Value = Not .grpToolBar(US_TBRECEIPT).Value
            End If
            Call SetReadReceipt(frm, .menToolsReadReceipt.CHECKED)
        fSettingProps = False
        End If
    End With
End Sub

Sub DeliveryReceipt(frm As Form)
'
' alternately set or clear Delivery Receipt property
'
    On Error Resume Next ' we'll catch the real error later
    
    With frm
        .menToolsDeliveryReceipt.CHECKED = Not .menToolsDeliveryReceipt.CHECKED
        .mfobj.message.DeliveryReceipt = .menToolsDeliveryReceipt.CHECKED
        If Not (frm.fDirty ) Then frm.mfobj.IsDirty = True
        .fDirty = True
        
    End With
End Sub


Public Function ArrangeEnv(frm As Form, twipWidth As Long, twipEnvTop As Long) As Long
Dim twipTemp As Long
Dim ictl As Integer
Dim twipFrameWidth As Long
Dim twipMaxWidth As Long
Dim twipMaxHeight As Long
Dim twipVSpace As Long
Dim twipTopGap As Long 'static
Dim twipLeftGap As Long 'static
Dim rgctlEnv(6) As Control
Dim rgctlTxt(6) As Control
Dim rgfctlVis(6) As Boolean
Dim cCtl As Integer
Dim fBigButton As Boolean
            
            
    twipFrameWidth = twipWidth
    On Error GoTo BadControl 'Skip an evelope component if it doesn't exist
    
    With frm
           
        twipTopGap = Screen.TwipsPerPixelY * 9
        twipLeftGap = Screen.TwipsPerPixelX * 3
        
        rgfctlVis(ictl + 0) = .fMAPI_From_vis
        Set rgctlTxt(ictl + 0) = .ctlFrom
        
        rgfctlVis(ictl + 1) = .fMAPI_SentDate_vis
        Set rgctlTxt(ictl + 1) = .ctlDate
        
        rgfctlVis(ictl + 2) = .fMAPI_To_vis
        Set rgctlTxt(ictl + 2) = .ctlTo
        
        rgfctlVis(ictl + 3) = .fMAPI_CC_vis
        Set rgctlTxt(ictl + 3) = .ctlCC
        
        rgfctlVis(ictl + 4) = .fMAPI_BCC_vis
        Set rgctlTxt(ictl + 4) = .ctlBCC
        
        rgfctlVis(ictl + 5) = .fMAPI_Subject_vis
        Set rgctlTxt(ictl + 5) = .ctlSubject

        cCtl = ictl + 6
        ictl = 0
        
        Set rgctlEnv(ictl + 0) = .ctlFromCap
        Set rgctlEnv(ictl + 1) = .ctlDateCap
        Set rgctlEnv(ictl + 2) = .ctlToButton
        Set rgctlEnv(ictl + 3) = .ctlCCButton
        Set rgctlEnv(ictl + 4) = .ctlBCCButton
        Set rgctlEnv(ictl + 5) = .ctlSubjectCap
        
        On Error GoTo 0 'no error handling
        If cCtl = 0 Then
            ArrangeEnv = 0
            Exit Function
        End If
        '
        ' Set up a minmum width for the command Buttons
        '                             To...
        twipMaxWidth = .TextWidth("     ")
        
        For ictl = 0 To cCtl - 1
            If rgfctlVis(ictl) Then
                twipTemp = .TextWidth(rgctlEnv(ictl).Caption)
                If twipMaxWidth < twipTemp Then twipMaxWidth = twipTemp
                twipTemp = .TextHeight(rgctlEnv(ictl).Caption)
                If twipMaxHeight < twipTemp Then twipMaxHeight = twipTemp
            End If
        Next ictl

        twipMaxHeight = twipMaxHeight + (twipMaxHeight / 2)
        
        twipVSpace = twipMaxHeight + (4 * Screen.TwipsPerPixelY)
        '
        ' round down  to pixel
        '
        twipVSpace = twipVSpace - (twipVSpace Mod Screen.TwipsPerPixelY)
        '
        ' round up  to pixel
        '
         
        twipMaxHeight = twipMaxHeight + Screen.TwipsPerPixelY * 3 - (twipMaxHeight Mod Screen.TwipsPerPixelY)
        twipFrameWidth = twipFrameWidth - (ENV_LEFTGAP * 2 + twipMaxWidth + twipLeftGap)
        '
        ' set minimum size
        '
        If twipFrameWidth < twipLeftGap * 30 Then twipFrameWidth = twipLeftGap * 30
        
        
        
        Dim twipShrink As Long, twipInset As Long
        
        Dim cLoc As Integer
        '
        ' Count visible controls
        '
        For ictl = 0 To cCtl - 1
            If rgfctlVis(ictl) Then cLoc = cLoc + 1
        Next ictl
        
        fBigButton = Not (.fSubmitted) And cLoc > 0

        cLoc = 0
            
        For ictl = 0 To cCtl - 1
            
            If rgctlEnv(ictl).height <> twipMaxHeight - Screen.TwipsPerPixelY * 2 Then
                rgctlEnv(ictl).Move ENV_LEFTGAP, twipTopGap + (cLoc * twipVSpace) + Screen.TwipsPerPixelY, twipMaxWidth, twipMaxHeight - Screen.TwipsPerPixelY * 2
                  
            End If
            If cLoc < 2 Then
                twipShrink = twipLeftGap * 2 + .BigButton.width
            Else
                twipShrink = 0
            End If
            If twipShrink > twipFrameWidth Then twipShrink = 0
            
            twipInset = 0
            If Not TypeOf rgctlTxt(ictl) Is TextBox Then
                If rgctlTxt(ictl).BorderStyles = 0 Then twipInset = Screen.TwipsPerPixelY * 4
            Else
                If rgctlTxt(ictl).BorderStyle = 0 Then twipInset = Screen.TwipsPerPixelY * 4
            End If
            
            rgctlTxt(ictl).Move ENV_LEFTGAP + twipMaxWidth + twipLeftGap, twipTopGap + (cLoc * twipVSpace) + twipInset, twipFrameWidth - twipShrink, twipMaxHeight - twipInset * 2
            '
            ' Don't make space for hidden fields
            '
            If rgfctlVis(ictl) Then cLoc = cLoc + 1
           
        Next ictl

        If fBigButton Then
            .BigButton.Move twipWidth - ENV_LEFTGAP - .BigButton.width, twipEnvTop + twipTopGap
            .BigButton.ZOrder ' move to front
            .BigButton.Visible = True
        Else
            .BigButton.Visible = False
        End If
        
        '
        '  envelope frame height
        '

        If cLoc = 0 Then
            ArrangeEnv = 0
            Exit Function
        End If
        
        '
        ' Make extra room for BigButton ?
        '
        
        If fBigButton And cLoc = 1 Then cLoc = 2
        
        ArrangeEnv = twipTopGap * 2 + (cLoc) * twipVSpace
    End With
    
    Exit Function
    
BadControl:
    ictl = ictl - 1 ' decrement counter
    Resume Next
    
End Function


Sub ResizeForm(frm As Form, oldWinState As Integer)
    With frm
        If .FormLoaded And ( _
          (.WindowState = NORMAL And (oldWinState = NORMAL Or oldWinState = MAXIMIZED)) Or _
          (.WindowState = MAXIMIZED And (oldWinState = NORMAL Or oldWinState = MINIMIZED))) Then
            Call ArrangeWindow(frm)
        End If
    
        ' If we're going from max to min, it's easier to pretend we're still maxed
        If oldWinState <> MAXIMIZED Or .WindowState <> MINIMIZED Then
          oldWinState = .WindowState
        End If
    End With
End Sub

Function ActivateRequiredControl(frm As Form, ctl As Control, strMsg As String) As Boolean
    ' If this control is on a tab page (no way to know without climbing
    ' the parent tree) then make sure that its tab page is shown

    Dim iTabPage As Integer
    If Not ctl.Visible Then Exit Function
    If TypeOf ctl Is Frame Or TypeOf ctl Is SSPanel Then
        ' Look for an option button inside to activate instead
        Dim I As Integer
        With frm
            For I = 0 To .Controls.count - 1
                If ctl Is CtrlParent(frm, .Controls(I)) Then
                    Set ctl = .Controls(I)
                    ' Fire the _Click event, which doesn't happen by SetFocus alone
                    Exit For
                End If
            Next
            If I = .Controls.count Then 'no option buttons in group box
                Exit Function
            End If
        End With
    End If
    iTabPage = TabPage(frm, ctl, True)
    
    Set frm.ctrlLastActive = Nothing ' clear this flag so even if the control has focus
    								 ' now it will trigger the scrolling code.
    
    SetFocusTo ctl
    
    DoEvents ' make sure the SetFocus event is handled
    
    Set frm.ctrlLastActive = ctl
    ActivateRequiredControl = True
    frmAdmin.MapifVbx1.NoEvents = True
    MsgBox strMsg
    frmAdmin.MapifVbx1.NoEvents = False
End Function

Sub SetFocusTo(ctl As Control)
On error goto NOFOCUS
    If Not ctl Is Nothing Then
        With ctl
             If .Visible Then
                If TypeOf ctl Is OptionButton Then
                    ' Setting focus always changes the state
                    Dim fOption As Boolean
                    fOption = .Value
                    .SetFocus
                    .Value = fOption
                Else
                    .SetFocus ' prevents runtime errors if control doesn't support SetFocus Method
                End If
            End If
        End With
    End If
    Exit Sub 
    
NOFOCUS:' prevents runtime errors if control doesn't support SetFocus Method
    SetFocusAPI (ctl.hWnd)
End Sub

Function TabPage(frm As Form, ctl, fActivate As Boolean) As Integer
    ' If this control is on a tab page (no way to know without climbing
    ' the parent tree) then return which tab page, else -1
    Dim tctl As Control, ctlParent
    Set tctl = ctl
    Set ctlParent = CtrlParent(frm, tctl)
    Do Until TypeName(ctlParent) = "MSTabCtl" Or TypeOf ctlParent Is Form
        Set tctl = ctlParent
        Set ctlParent = CtrlParent(frm, tctl)
    Loop
    If TypeOf ctlParent Is Form Then
        TabPage = -1
    Else
        TabPage = Asc(LoadResData(CLng(tctl.Tag), ResType + ResType_TAB))
        If fActivate And (ctlParent.ActiveTab <> TabPage) Then
            ctlParent.ActiveTab = TabPage
        End If
    End If
End Function

Function CreateResponseBody(modeCopy As Integer, pSess As Long, pMessIn As Long, pMessOut As Long, rspClass As String) As Boolean
        ' there is error handlng in the fuctions that call this
    
        With frmAdmin.rtfScratch
        
        .MapiSessID = pSess ' inform vbx of Session
        .MapiMsgID = pMessIn ' inform vbx of Message
     
        .Action = RE_ACTION_OPENADDRESSBOOK ' Get addressbk & Store ID
        .MessageClass = FormMsgClass ' tell control the message class (save an RPC and avoid MDISP Cache problems
    
        .Action = RE_ACTION_LOADDATAFROMMSG 'load the body
    
        .MapiMsgID = pMessOut ' inform vbx of destination Message
    
        .MessageClass = rspClass ' tell control the message class (save an RPC and avoid MDISP Cache problems
        .Action = RE_ACTION_SAVETOMSG 'Save the body
    
    
        .MapiSessID = 0 ' clear Session
        .MapiMsgID = 0 ' clear Message
        
    End With
    
    CreateResponseBody = True
    
End Function


Public Sub InitMenu(frm As Form)
'
' this routine handles enabling and disabling the view menu,
' based on the current form state.
'
    With frm
        '.menComposeReply.Enabled = .fSubmitted
        '.menComposeReplyAll.Enabled = .fSubmitted
        '.menComposeReplyToFolder.Enabled = .fSubmitted
        '.menComposeForward.Enabled = .fSubmitted
        '.mnuSep6_6.Enabled = .fSubmitted

        ' Submitted items cannot be sent
        ' also check view context flag
        .menFileSend.Enabled = Not .fSubmitted And .mfobj.MaySubmit
        

        ' enable/disable next/previous
        Dim fNext As Boolean, fPrev As Boolean
        fNext = .mfobj.NextExists
        fPrev = .mfobj.PreviousExists
        .menViewNext.Enabled = fNext
        .grpToolBar(S_TBNEXT).Enabled = fNext
        .menViewPrevious.Enabled = fPrev
        .grpToolBar(S_TBPREVIOUS).Enabled = fPrev


        Dim fInsert As Boolean, fAddress As Boolean

        If Not .ctlRichMain Is Nothing Then
            fInsert = (Not .ctlRichMain.ReadOnly) And .ctlRichMain.OLEObjects
        End If
        
        '
        ' Any editable address fields ?
        '
        fAddress = .fUnlockedAddr And Not (.ctlRichAddr Is Nothing)

        .menInsertFile.Enabled = fInsert

        .menToolsCheckNames.Enabled = fAddress
        .menToolsSelectNames.Enabled = fAddress

        If Not .fSubmitted Then
        	.grpToolBar(US_TBSEND).Enabled = .menFileSend.Enabled
            .grpToolBar(US_TBINSERT).Enabled = fInsert
            .grpToolBar(US_TBVERNAMES).Enabled = fAddress
            .grpToolBar(US_TBCHNAMES).Enabled = fAddress
        Else
            .grpToolBar(S_TBREPLY).Enabled = .menComposeReply.Enabled
            .grpToolBar(S_TBREPLYALL).Enabled = .menComposeReplyAll.Enabled
            .grpToolBar(S_TBFORWARD).Enabled = .menComposeForward.Enabled
            .grpToolBar(S_TBDELETE).Enabled = .menFileDelete.Enabled
        End If
    End With
End Sub

Function CtrlParent(frm As Form, ctl)

    ' Given a control and the form it's on, return the
    ' control's parent, which could be the form itself.
    
    If TypeOf ctl Is Form Then 'forms don't have parents
        Set CtrlParent = Nothing
        Exit Function
    End If
    
    Dim hWndParent As Integer, cControls As Integer
    On Error Resume Next
    
    If ctl.Container Is Nothing Then
        ' If the control doesn't support the Container property
        ' then the error will make it fall in here
        hWndParent = GetParent(ctl.hWnd)
        If hWndParent = 0 Then 'The ctl didn't have an hwnd, so we can't find its parent
            Set CtrlParent = Nothing
            Exit Function
        End If
        cControls = frm.Controls.count
        Do While cControls > 0
            cControls = cControls - 1
            If hWndParent <> frm.Controls(cControls).hWnd Then
                ' Either a control with no hWnd, or not the one we're looking for
            Else
                Set CtrlParent = frm.Controls(cControls)
                Exit Function
            End If
        Loop
        
        'didn't find it's parent in the Controls collection, so its parent must be the form
        Set CtrlParent = frm
    Else
        Set CtrlParent = ctl.Container
    End If
End Function

Public Function StringToMVProp(ByVal inputStr As String) As Variant
    '
    ' converts semi-colon delimited string to array of strings
    '
    
    Dim answer() As String
    Dim cIndexArr As Long
    Dim itemp As Long
    Dim szTemp As String
    
    If inputStr = "" Then
        Exit Function
    End If
    
    Do
        '
        ' check each entry in the list box
        '
        
        itemp = InStr(inputStr, ";")
        
        If itemp = 0 Then ' last entry
            szTemp = inputStr
            inputStr = ""
            itemp = Len(szTemp )
        Else
            itemp = itemp - 1
            szTemp = left(inputStr, itemp) ' we omit the ';' (since itemp=item-1)
            inputStr = Mid(inputStr, itemp + 2) ' truncate the input string, skipping the ';'
        End If
        
        
        '
        ' remove leading spaces
        '
        
        while left(szTemp , 1) = " " and itemp <> 0 
            szTemp = Mid(szTemp , 2)
            itemp = itemp - 1
        wend
        
        If itemp <> 0 Then
            '
            ' remove trailing spaces
            '
 
            while Mid(szTemp , itemp ) = " "
                itemp = itemp - 1
                szTemp = left(szTemp , itemp)
            wend
                 
            ReDim Preserve answer(cIndexArr)
            answer(cIndexArr) = szTemp
            cIndexArr = cIndexArr + 1
                
            If cIndexArr > 64 Then inputStr = "" ' limited to 64 elements
        end if
        
    Loop While inputStr <> ""
    If cIndexArr > 0 Then StringToMVProp = answer
    
End Function

Public Function MVPropToString(rgInput As Variant) As String
    Dim cArray As Long
    Dim iArray As Long
    
    cArray = UBound(rgInput)
    
    For iArray = 0 To cArray - 1
        MVPropToString = MVPropToString & CStr(rgInput(iArray)) & "; "
    Next iArray
    '
    ' no trailing ";"
    '
    
    MVPropToString = MVPropToString & CStr(rgInput(cArray))
End Function

Public Sub AddUserAddr(sessObj As Object, msgObj As Object, addrType As Long)
    Dim senderIDStr As String, addressStr As String
    Dim Addresses As Object
    
    Set Addresses = msgObj.Recipients
    
    With sessObj.currentuser

        '
        ' add the current user to the address list
        '
        senderIDStr = .Id
        addressStr = .address
        Addresses.Add .Name, addressStr, addrType, senderIDStr
    End With
    
    '
    ' flush the property cache so the Address field can read the address
    '
    
    msgObj.Update False
    
    Set Addresses = Nothing
    

End Sub
