Attribute VB_Name = "Rendering"
Option Explicit
Declare Function VBSetForegroundWindow32 Lib "msriched.vbx" (ByVal hWnd As Integer) As Integer
Declare Sub ScInsertREHwnd Lib "msriched.vbx" (ByVal hwndDst As Integer, ByVal hwndSrc As Integer)

' VBGetVersionEx32 thunks to GetVersionEx on 32bit platforms, and on 16bit as much gets filled
' in as possible. Build Number is not set on 16bit.
Declare Function VBGetVersionEx32 Lib "msriched.vbx" (osvi As OSVERSIONINFO) As Boolean
Global Const VER_PLATFORM_WIN32s = 0   'Never get this from a 16bit app
Global Const VER_PLATFORM_WIN32_95 = 1 'SDK docs refer to this as VER_PLATFORM_WIN32_WINDOWS
Global Const VER_PLATFORM_WIN32_NT = 2 'Includes Shell Upgrade Release
Global Const VER_PLATFORM_WIN16 = 3    'Not included in SDK docs since GetVersionEx is a 32bit API

Global OSVerInfo As OSVERSIONINFO

Type CtrlInfo
  ctlContainer As Control
  ctlThis As Control
  ctlCaption As Control 'the caption of the separate label control that goes with this field
  iTabPage As Integer 'only used for controls immediately parented to the tab control
End Type

' RichEdit stuff
Private Type CHARRANGE
  cpMin As Long
  cpMax As Long
End Type

Private Type FORMATRANGE
  hDC As Integer
  hdcTarget As Integer
  rc As RECT
  rcPage As RECT
  chrng As CHARRANGE
End Type

Private Type TypeCHARFORMAT
  Size As Integer
  Padding1 As Integer
  Mask As Long
  Effects As Long
  height As Long
  offset As Long
  TextColor As Long
  CharSet As Byte
  PitchAndFamily As Byte
  FaceName As String * 32
  Padding2 As Integer
End Type

' Bit masks for indicating which members are valid
Const CFM_BOLD = &H1&
Const CFM_ITALIC = &H2&
Const CFM_UNDERLINE = &H4&
Const CFM_STRIKEOUT = &H8&
Const CFM_PROTECTED = &H10&
Const CFM_SIZE = &H80000000
Const CFM_COLOR = &H40000000
Const CFM_FACE = &H20000000
Const CFM_OFFSET = &H10000000
Const CFM_CHARSET = &H8000000

' Bit masks for indicating which bits of Effects are valid
Const CFE_BOLD = &H1
Const CFE_ITALIC = &H2
Const CFE_UNDERLINE = &H4
Const CFE_STRIKEOUT = &H8
Const CFE_PROTECTED = &H10

' what EM_SETCHARFORMAT applies to
Const SCF_SELECTION = 1
Const SCF_WORD = 2

Private Type TypePARAFORMAT
  Size As Integer
  Padding As Integer
  Mask As Long
  Numbering As Integer
  reserved As Integer
  StartIndent As Long
  RightIndent As Long
  offset As Long
  Alignment As Integer
  TabCount As Integer
  Tabs(0 To 31) As Long
End Type
  
' Bit masks for indicating which members are valid
Const PFM_STARTINDENT = &H1&
Const PFM_RIGHTINDENT = &H2&
Const PFM_OFFSET = &H4&
Const PFM_ALIGNMENT = &H8&
Const PFM_TABSTOPS = &H10&
Const PFM_NUMBERING = &H20&
Const PFM_OFFSETINDENT = &H80000000

' Alignment values
Const PFA_LEFT = &H1&
Const PFA_RIGHT = &H2&
Const PFA_CENTER = &H3&
  
' Numbering options
Const PFN_BULLET = &H1&

' Richedit messages
Const EM_FORMATRANGE = 1081
Const EM_DISPLAYBAND = 1075
Const EM_REPLACESEL = 1042
Const EM_SETPARAFORMAT = 1095
Const EM_SETCHARFORMAT = 1092
Const EM_SETSEL = 1025
Const EM_EXSETSEL = 1079
Const EM_EXGETSEL = 1076

Const TABTEXTLMARGIN = 10
Const NOT_IN_TAB = -1

Dim CtrlsToRender() As CtrlInfo
Dim IndentLevel As Integer
Public frmToRender As Form 'Accessed from PrintExtra event in frmAdmin
Dim CrLf As String 'Carriage return / Line feed
Dim ParaFormat As TypePARAFORMAT
Dim CharFormat As TypeCHARFORMAT
Dim hwndRE As Integer 'Child of rtfScratch--where all the messages get sent
Dim ctlTab As Control    'Tab pages are changed, so keep track of tab control
Dim ActiveTab As Integer 'and the active tab before we changed it
Dim LineHeight As Integer
Const xMargin = 360
Const yMargin = 360
Public gIniFile As String

Const cdlPDPrintSetup = &H40&
Const IndentSize = 360 ' 1/4 inch
Const TWIPSPERPOINT = 20
Sub AddText(strText As String, fontCtl As Font, fForceBold As Boolean, foreColor As Long)
  If Not fontCtl Is Nothing Then
    CharFormat.Effects = CFE_PROTECTED Or _
           IIf(fForceBold Or fontCtl.Bold, CFE_BOLD, 0) _
        Or IIf(fontCtl.Italic, CFE_ITALIC, 0) _
        Or IIf(fontCtl.Strikethrough, CFE_STRIKEOUT, 0) _
        Or IIf(fontCtl.Underline, CFE_UNDERLINE, 0)
    CharFormat.FaceName = fontCtl.Name & Chr$(0)
    CharFormat.TextColor = foreColor
    CharFormat.height = fontCtl.Size * TWIPSPERPOINT
    SendMessage hwndRE, EM_SETCHARFORMAT, SCF_SELECTION, CharFormat
  End If
  SendMessage hwndRE, EM_REPLACESEL, False, ByVal strText
End Sub

Sub DoPrintGetFooter(FooterType As Integer)
    ' get the footer index
    FooterType = GetPrivateProfileInt(INIPRINTGROUP, INIFOOTERKEY, -1, gIniFile)
    If FooterType = -1 Then
      ' if the default value was returned, the key doesn't exist yet, so create it.
      Call WritePrivateProfileString(INIPRINTGROUP, INIFOOTERKEY, "0", gIniFile)
      FooterType = 0
    End If
End Sub

Sub DoPrintForm(frm As Form, hDevMode As Integer, hDevNames As Integer, fPrintFromForm As Boolean)
  ' If the top of the form is below the screen, then we don't need an printing status window
  ' because we're printing an unopened item and we're using Capone's print dialog
  With frmAdmin.rtfScratch
    frmAdmin.fPrinted = False
    If fPrintFromForm Then
      On Error GoTo NoDefaultPrinter
      .PrintHDC = Printer.hDC
      PrintingStatus.lblJobName = frm.Caption
      PrintingStatus.lblPage = PRNTSTATPREP
      PrintingStatus.Show
      Call SetWindowPos(PrintingStatus.hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE)
      PrintingStatus.Refresh
    Else
      .PrintHDevMode = hDevMode
      .PrintHDevNames = hDevNames
    End If
    
    Call RenderForm(frm, False)
  
    If fPrintFromForm Then
      PrintingStatus.lblPage = PRINTFOOTER0 + str$(1)
      PrintingStatus.lblPage.Refresh
    End If
    
    .PrintDocName = frm.Caption
    .PrintMarginTop = yMargin + 302 * 2
    .PrintMarginBottom = yMargin + 250 * 2
    .PrintMarginLeft = xMargin
    .PrintMarginRight = xMargin
    .Action = RE_ACTION_PRINT
    If fPrintFromForm Then Printer.EndDoc 'Free the hDC

NoDefaultPrinter:
  End With
  
  Unload PrintingStatus
  If Not frmAdmin.fPrinted Then
    MsgBox NODEFAULTPRERR
  End If
  Exit Sub
End Sub

Function DoPrintGetIniFile() As String
    ' create the INI file name and test if it exists.  If not, create
    ' the INI file too.
    
    Dim StrWinDir As String, lenStr As Integer, FileName As String

    StrWinDir = String$(200, 0)
    lenStr = GetWindowsDirectory(StrWinDir, Len(StrWinDir))
    StrWinDir = left$(StrWinDir, lenStr)
    
    FileName = StrWinDir & IIf(right$(StrWinDir, 1) = "\", "", "\") & ININAME
    
    ' if the file doesn't exist, then create it
    If Dir$(FileName) = "" Then
        WritePrivateProfileString INIPRINTGROUP, INIFOOTERKEY, "0", gIniFile
    End If
    
    DoPrintGetIniFile = FileName
End Function



Sub DoPrintSetup(frm As Form)
  frmAdmin.ComDlg.Flags = cdlPDPrintSetup
  frmAdmin.ComDlg.ShowPrinter
End Sub

Function FRenderControl(ctl, iTabPage As Integer) As Integer
  ' Determine if control should be rendered.
  ' All the following conditions must be met:
  ' 1) It's visible
  ' 2) It's not a menu (has coordinates)
  ' 3) It's on the canvas (not necessarily as an immediate child)
  ' 4) All containers between the control and the canvas are visible
  ' Additionally, figure out what the caption is called,
  ' and what tab page the control is on if any
  iTabPage = -1
  On Error Resume Next
  With ctl
    If Not (TypeOf ctl Is RichText Or TypeOf ctl Is Label Or _
            TypeOf ctl Is TextBox Or TypeOf ctl Is ComboBox Or _
            TypeOf ctl Is ListBox Or TypeOf ctl Is CheckBox Or _
            TypeOf ctl Is OptionButton Or TypeName(ctl) = STRTABCLASS Or _
            TypeOf ctl Is Frame Or TypeOf ctl Is SSPanel Or _
            TypeOf ctl Is PictureBox) Then
      'Skip this control
    ElseIf Not .Visible Then
      'Skip this control
    ElseIf .left <> .left Then
      'Skip this control because it doesn't have coordinates (menus)
    Else
      Dim ctlCurrent, ctlPrevious As Control
      Set ctlCurrent = ctl
      Do Until TypeOf ctlCurrent Is Form Or Not ctlCurrent.Visible
        Set ctlPrevious = ctlCurrent
        Set ctlCurrent = CtrlParent(frmToRender, ctlCurrent)
        If ctlCurrent Is frmToRender.Canvas_Ctrl Then
          ' don't render the label if it is a caption for another control
          If Not (TypeOf ctl Is Label And _
             right$(ctl.Name, 4) = "_Cap" And _
             Not (frmToRender.Controls(left$(ctl.Name, Len(ctl.Name) - 2) & "trl") Is Nothing)) Then
            ' we can also get here by ann error in the condition, which is ok
            FRenderControl = True
          Else
            FRenderControl = False
          End If
          Exit Do
        ElseIf TypeName(ctlCurrent) = STRTABCLASS Then
          If ctlTab Is Nothing Then
            Set ctlTab = ctlCurrent
            ActiveTab = ctlTab.ActiveTab
          End If
          iTabPage = TabPage(frmToRender, ctlPrevious, True)
        End If
      Loop
    End If
  End With
End Function

Sub AddControlToRender(CtlToAdd As Control, iTabPage As Integer)
  With frmAdmin.lstEasySort
    Dim ictl As Integer
    ictl = .ListCount
    .AddItem Format(CtlToAdd.top + CtlToAdd.left / 20, "000000")
    ReDim Preserve CtrlsToRender(ictl)
    .itemData(.NewIndex) = ictl
  End With
  With CtrlsToRender(ictl)
    Set .ctlContainer = CtrlParent(frmToRender, CtlToAdd)
    Set .ctlThis = CtlToAdd
    On Error Resume Next 'if there is no associated caption, that's ok
    Set .ctlCaption = frmToRender.Controls(left$(CtlToAdd.Name, Len(CtlToAdd.Name) - 3) & "ap")
    CtrlsToRender(ictl).iTabPage = iTabPage + 1 'using 1-based tab #s
  End With
End Sub

Sub InsertPaddingChar(ByVal iPosition As Long)
  Dim cr As CHARRANGE
  cr.cpMin = iPosition
  cr.cpMax = iPosition
  SendMessage hwndRE, EM_EXSETSEL, 0, cr
  SendMessage hwndRE, EM_REPLACESEL, False, ByVal " "
End Sub

Function PrepRenderingForReply() As Long
  ' Adds blank line, separator and indents everything except the new blank line

  CharFormat.Size = 60 ' In case we were called before RenderForm
  ParaFormat.Size = 156

  hwndRE = GetWindow(frmAdmin.rtfScratch.hWnd, GW_CHILD)
  Call SendMessage(hwndRE, EM_SETSEL, 0, ByVal 0&)
  frmAdmin.Font.Size = 10
  AddText CrLf & CrLf, frmAdmin.Font, False, frmAdmin.foreColor
  frmAdmin.Font.Size = 8
  AddText "----------" & CrLf, frmAdmin.Font, False, frmAdmin.foreColor
  Dim tcr As CHARRANGE
  tcr.cpMin = 4 'Leave the first two lines non-indented
  tcr.cpMax = -1
  SendMessage hwndRE, EM_EXSETSEL, 0, tcr
  ParaFormat.Mask = PFM_OFFSETINDENT
  ParaFormat.StartIndent = IndentSize
  SendMessage hwndRE, EM_SETPARAFORMAT, 0, ParaFormat
  SendMessage hwndRE, EM_SETSEL, 0, ByVal 0& 'clear selection, goto start

  ' return the number of characters added
  PrepRenderingForReply = 16 ' 3 CrLf's + 10 dashes
End Function

Function MakeSmallHeader(Msg As Object) As Long
  ' puts minimal header information into rtfscratch
  ' returns number a charaters added to rtf
  
  Dim strOut As String
  If CrLf = "" Then
    CrLf = Chr$(13) & Chr$(10)
    With CharFormat
      .Size = 60
      .Mask = CFM_BOLD Or CFM_COLOR Or CFM_FACE Or CFM_SIZE _
           Or CFM_ITALIC Or CFM_STRIKEOUT Or CFM_UNDERLINE Or CFM_CHARSET
      .CharSet = DEFAULT_CHARSET
    End With
  End If
  
  hwndRE = GetWindow(frmAdmin.rtfScratch.hWnd, GW_CHILD)
  frmAdmin.rtfScratch.TextBlocks = 0 'Ensure contents are empty
    
  With ParaFormat
    .Size = 156
    .Mask = PFM_STARTINDENT Or PFM_TABSTOPS
    .offset = 0
    .StartIndent = 0
    .Tabs(0) = 1500
    .TabCount = 1
  End With
  SendMessage hwndRE, EM_SETPARAFORMAT, 0, ParaFormat

  Call SendMessage(hwndRE, EM_SETSEL, 0, ByVal 0&)
  frmAdmin.Font.Size = 10

  ' add the sender line
  AddText strHEADSENDER, frmAdmin.Font, True, frmAdmin.foreColor
  strOut = Msg.Sender.Name
  AddText Chr$(9) & strOut & CrLf, frmAdmin.Font, False, frmAdmin.foreColor
  
  ' add the sentdate line
  AddText strHEADSENT, frmAdmin.Font, True, frmAdmin.foreColor
  strOut = Format(Msg.TimeSent, "Long Date") & " " & Format(Msg.TimeSent, "Long Time")
  AddText Chr$(9) & strOut & CrLf & CrLf, frmAdmin.Font, False, frmAdmin.foreColor
  
  ' return the number of chars added
  Dim cr As CHARRANGE
  SendMessage hwndRE, EM_EXGETSEL, 0, cr
  MakeSmallHeader = cr.cpMax

End Function

Sub RenderCaption(ctlCaption As Control)
  Dim strCaption As String, fontCap As New StdFont
  Dim fNeedStrip As Boolean, foreColor As Long
  If Not ctlCaption Is Nothing Then
    If TypeName(ctlCaption) = STRTABCLASS Then
      strCaption = Trim$(ctlCaption.ActiveCaption)
    Else
      strCaption = Trim$(ctlCaption.Caption)
    End If
    
    If TypeOf ctlCaption Is CommandButton Then
      If right$(strCaption, 3) = "..." Then
        strCaption = left$(strCaption, Len(strCaption) - 3)
      End If
      foreColor = &H80000012
    Else
      foreColor = ctlCaption.foreColor
    End If
    
    On Error Resume Next 'allow examining of property that may not exist
    fNeedStrip = (TypeName(ctlCaption) = STRTABCLASS) _
              Or (TypeOf ctlCaption Is SSPanel) _
              Or (TypeOf ctlCaption Is CommandButton) _
              Or (TypeOf ctlCaption Is Frame)
    fNeedStrip = fNeedStrip Or (TypeOf ctlCaption Is Label And ctlCaption.UseMnemonic)
    If fNeedStrip Then
      strCaption = StripMnemonic(strCaption)
    End If
    
    If right$(strCaption, 1) <> ":" Then strCaption = strCaption & ":"
    strCaption = strCaption & " "
    If TypeName(ctlCaption) = STRTABCLASS Then
      SetFont fontCap, ctlCaption
    Else
      Set fontCap = ctlCaption.Font
    End If
    AddText strCaption, fontCap, True, foreColor
  End If
  Set fontCap = Nothing
End Sub

Sub RenderEnvFields(fHideFromDate As Boolean)
  Dim EnvFields(5, 1) As Control, iEnv As Integer, fEnvExist As Boolean
  Dim fontEnv As New StdFont, cChars As Long, strCtlText As String, sizeOld As Integer
  
  On Error Resume Next
  With frmToRender
    Set EnvFields(0, 0) = .ctlFrom
    Set EnvFields(1, 0) = .ctlDate
    Set EnvFields(2, 0) = .ctlTo
    Set EnvFields(3, 0) = .ctlCC
    Set EnvFields(4, 0) = .ctlBCC
    Set EnvFields(5, 0) = .ctlSubject
    Set EnvFields(0, 1) = .ctlFromCap
    Set EnvFields(1, 1) = .ctlDateCap
    Set EnvFields(2, 1) = .ctlToButton
    Set EnvFields(3, 1) = .ctlCCButton
    Set EnvFields(4, 1) = .ctlBCCButton
    Set EnvFields(5, 1) = .ctlSubjectCap
  End With
      
  For iEnv = 0 To 5
    ' Skip the control if it doesn't exist (causes an error)
    ' or if it's From or Date and we're hiding it
    ' Remember, an error condition drops into the True case, which
    ' is why the odd-looking if-then-else constructs
    
    If (fHideFromDate And iEnv <= 1) Or Not EnvFields(iEnv, 0).Visible Then
      ' Skip it
    Else
      fEnvExist = True
      If TypeOf EnvFields(iEnv, 0) Is TextBox Then
        strCtlText = EnvFields(iEnv, 0).Text
        Set fontEnv = EnvFields(iEnv, 0).Font
      Else
        strCtlText = EnvFields(iEnv, 0).Text(0)
        SetFont fontEnv, EnvFields(iEnv, 0)
      End If

      'Envelope fields are emitted without font information.
      fontEnv.Size = 10
      sizeOld = EnvFields(iEnv, 1).Font.Size
      EnvFields(iEnv, 1).Font.Size = 10
      
      RenderLine EnvFields(iEnv, 1), Chr$(9) & strCtlText, fontEnv, EnvFields(iEnv, 0).foreColor
      EnvFields(iEnv, 1).Font.Size = sizeOld
    End If
  Next
  
  'Only add blank line if there is at least one envelope field
  If fEnvExist Then
    AddCrLf Nothing, 0
  End If
End Sub

Function RenderForm(frm As Form, fHideFromDate As Boolean) As Long
  Dim iControl As Integer, ctl As Control, iTabPage As Integer
  
  If CrLf = "" Then
    CrLf = Chr$(13) & Chr$(10)
    With CharFormat
      .Size = 60
      .Mask = CFM_BOLD Or CFM_COLOR Or CFM_FACE Or CFM_SIZE _
           Or CFM_ITALIC Or CFM_STRIKEOUT Or CFM_UNDERLINE Or CFM_CHARSET Or CFM_PROTECTED
      .CharSet = DEFAULT_CHARSET
    End With
  End If
  
  Set frmToRender = frm
  Set ctlTab = Nothing
  frmAdmin.lstEasySort.Clear
  frmAdmin.rtfScratch.TextBlocks = 0 'Ensure contents are empty
  hwndRE = GetWindow(frmAdmin.rtfScratch.hWnd, GW_CHILD)
  
  LockWindowUpdate frm.hWnd
  
  For iControl = 0 To frmToRender.Controls.count - 1
    Set ctl = frmToRender.Controls(iControl)
    If FRenderControl(ctl, iTabPage) Then
      AddControlToRender ctl, iTabPage
    End If
  Next
  
  With ParaFormat
    .Size = 156
    .Mask = PFM_STARTINDENT Or PFM_TABSTOPS
    .offset = 0
    .StartIndent = 0
    .Tabs(0) = 1500
    .TabCount = 1
  End With
  SendMessage hwndRE, EM_SETPARAFORMAT, 0, ParaFormat
  
  Call RenderEnvFields(fHideFromDate)
  Dim cr As CHARRANGE
  SendMessage hwndRE, EM_EXGETSEL, 0, cr
  RenderForm = cr.cpMax
  
  With ParaFormat
    .Tabs(0) = 0
    .TabCount = 0
  End With
  SendMessage hwndRE, EM_SETPARAFORMAT, 0, ParaFormat
  
  ParaFormat.Mask = PFM_STARTINDENT
  
  IndentLevel = 0
  
  'frmAdmin.rtfScratch.MapiSessID = frm.mfobj.MAPISession ' addref
  'frmAdmin.rtfScratch.MapiMsgID = frm.mfobj.MAPIMessage ' addref

  RenderKids frmToRender.Controls("Canvas_ctrl")
  
  'frm.mfobj.MAPISession = 0 ' release ref
  'frm.mfobj.MAPIMessage = 0 ' release ref
  'frmAdmin.rtfScratch.MapiMsgID = 0
  
  If Not ctlTab Is Nothing Then
    ctlTab.ActiveTab = ActiveTab
  End If

  LockWindowUpdate 0
  SetFocusTo frm.ctrlLastActive
  Set ctlTab = Nothing
End Function
Sub RenderKids(ctlRenderContainer As Control)
  Dim iControl As Integer, fSameTabPage As Boolean
  Dim ctl As Control, ctlCaption As Control, ctlContainer As Control
  Static iTabPage As Integer 'tells us what tab page we're on, if any (1-based)

  For iControl = 0 To frmAdmin.lstEasySort.ListCount - 1
    With CtrlsToRender(frmAdmin.lstEasySort.itemData(iControl))
      Set ctl = .ctlThis
      Set ctlContainer = .ctlContainer
      Set ctlCaption = .ctlCaption
      fSameTabPage = iTabPage = .iTabPage
    End With
    If ctlRenderContainer Is ctlContainer And IIf(iTabPage, fSameTabPage, True) Then
        ' We found a control parented to the container we're dealing with
        ' and if we're in a tab, the control we found is on the current page
      With ctl
        If TypeOf ctl Is RichText Then
          RenderRichText ctl, ctlCaption
        
        ElseIf TypeOf ctl Is Label Then
          If .UseMnemonic Then
            RenderLine Nothing, .Caption, .Font, .foreColor
          Else
            RenderLine Nothing, StripMnemonic(.Caption), .Font, .foreColor
          End If
          
        ElseIf TypeOf ctl Is TextBox Then
          If .MultiLine Then
            RenderLine ctlCaption, "", Nothing, 0
            RenderLine Nothing, .Text, .Font, .foreColor
            AddText CrLf, Nothing, False, 0
          Else
            RenderLine ctlCaption, .Text, .Font, .foreColor
          End If
        
        ElseIf TypeOf ctl Is ComboBox Then
          RenderLine ctlCaption, .Text, .Font, .foreColor
        
        ElseIf TypeOf ctl Is ListBox Then
          RenderListBox ctl, ctlCaption
        
        ElseIf TypeOf ctl Is CheckBox Then
          RenderLine Nothing, "[" & IIf(.Value, "X", " ") & "] " _
                     & StripMnemonic(.Caption), .Font, .foreColor
        
        ElseIf TypeOf ctl Is OptionButton Then
          RenderLine Nothing, "(" & IIf(.Value, "*", " ") & ") " _
                     & StripMnemonic(.Caption), .Font, .foreColor
        
        ElseIf TypeName(ctl) = STRTABCLASS Then
          RenderLine ctlCaption, "", Nothing, 0
          For iTabPage = 1 To .TabCount
            .ActiveTab = iTabPage - 1
            RenderLine ctl, "", Nothing, 0
            IndentLevel = IndentLevel + 1
            RenderKids ctl
            IndentLevel = IndentLevel - 1
            AddText CrLf, Nothing, False, 0
          Next
          iTabPage = 0
          
        ElseIf TypeOf ctl Is Frame _
            Or TypeOf ctl Is SSPanel _
            Or TypeOf ctl Is PictureBox Then
          On Error Resume Next
          If .Caption = "" Then
            'If the container doesn't have a caption or the caption is empty
            RenderKids ctl
          Else
            RenderLine ctl, "", Nothing, 0
            IndentLevel = IndentLevel + 1
            RenderKids ctl
            IndentLevel = IndentLevel - 1
            AddText CrLf, Nothing, False, 0
          End If
          On Error GoTo 0
        End If
      End With
    End If
  Next
End Sub

Sub RenderListBox(ctl As Control, ctlCaption As Control)
  With ctl
    If .MultiSelect Then
      Dim iRow As Integer, fIsFirst As Boolean
      RenderCaption ctlCaption
      fIsFirst = True
      For iRow = 0 To ctl.ListCount - 1
        If ctl.Selected(iRow) Then
          AddText IIf(fIsFirst, "", "; ") & ctl.List(iRow), .Font, False, .foreColor
          fIsFirst = False
        End If
      Next
      AddText CrLf, Nothing, False, 0
    Else
      RenderLine ctlCaption, .Text, .Font, .foreColor
    End If
  End With
End Sub


Sub RenderRichText(ctl As Control, ctlCaption As Control)
  Dim fontCtl As New StdFont
  SetFont fontCtl, ctl
  If ctl.Text(0) <> "" Or Not ctlCaption Is Nothing Then
    RenderCaption ctlCaption
    If ctl.MultiLine Then
      ParaFormat.StartIndent = IndentLevel * IndentSize
      SendMessage hwndRE, EM_SETPARAFORMAT, 0, ParaFormat
      AddText CrLf, Nothing, False, 0
    End If
    Dim cr As CHARRANGE, cpMin As Integer
    SendMessage hwndRE, EM_EXGETSEL, 0, cr
    cpMin = cr.cpMin 'Keep track of where we were so we can indent everything
    ScInsertREHwnd hwndRE, GetWindow(ctl.hWnd, GW_CHILD)
    If ctl.MultiLine Then
      AddText CrLf, Nothing, False, 0
    End If
    SendMessage hwndRE, EM_EXGETSEL, 0, cr
    cr.cpMin = cpMin
    SendMessage hwndRE, EM_EXSETSEL, 0, cr 'select the rich text we just copied
    ParaFormat.Mask = PFM_OFFSETINDENT
    ParaFormat.StartIndent = IndentLevel * IndentSize
    SendMessage hwndRE, EM_SETPARAFORMAT, 0, ParaFormat
    ParaFormat.Mask = PFM_STARTINDENT
    SendMessage hwndRE, EM_SETSEL, -1, ByVal -1& 'clear selection, goto end
    AddText CrLf, Nothing, False, 0
    ' Make sure we clear any bulleting or non-left alignment
    ParaFormat.Mask = ParaFormat.Mask Or PFM_ALIGNMENT Or PFM_NUMBERING
    ParaFormat.Alignment = PFA_LEFT
    ParaFormat.Numbering = 0
    SendMessage hwndRE, EM_SETPARAFORMAT, 0, ParaFormat
    ParaFormat.Mask = ParaFormat.Mask And Not (PFM_ALIGNMENT Or PFM_NUMBERING)
  End If
End Sub

Function DoSaveAs(frm As Form, SaveFormat As Integer, fNeedFileDlg As Boolean) As String
  Const SAVE_FORMAT_TEXT = 1
  Const SAVE_FORMAT_RICHTEXT = 2
  With frmAdmin
    If fNeedFileDlg Then
      .ComDlg.CancelError = True
      .ComDlg.DefaultExt = "txt"
      .ComDlg.FileName = ""
      .ComDlg.Filter = "Text (*.txt)|*.txt"
      .ComDlg.Flags = cdlOFNHideReadOnly Or cdlOFNNoReadOnlyReturn _
                   Or cdlOFNOverwritePrompt Or cdlOFNPathMustExist
      On Error Resume Next
      Do
        Err = 0
        .ComDlg.Action = 2
        If Err Then Exit Do 'pressed escape
        If Dir(.ComDlg.FileName) <> "" Then
          Kill .ComDlg.FileName
          If Err Then MsgBox "Can't delete file " & .ComDlg.FileName, , .ComDlg.DialogTitle
        End If
      Loop While Err
  
      .ComDlg.CancelError = False

      If Err = 0 And .ComDlg.FileName <> "" Then
        Screen.MousePointer = HOURGLASS
        Dim fno As Integer
        fno = FreeFile
        Open .ComDlg.FileName For Binary Access Write As fno
        If Err Then
          MsgBox CANTOPENFILE & .ComDlg.FileName, , .ComDlg.DialogTitle
          Exit Function
        End If
      Else
        Exit Function
      End If
    End If
      
    Call RenderForm(frm, False)
    
    If fNeedFileDlg Then SaveFormat = SAVE_FORMAT_TEXT
    
    Select Case SaveFormat
      Case SAVE_FORMAT_TEXT
        DoSaveAs = .rtfScratch.Text(0)
        If .rtfScratch.TextBlocks > 1 Then
          DoSaveAs = DoSaveAs & .rtfScratch.Text(1)
        End If
      Case SAVE_FORMAT_RICHTEXT
        DoSaveAs = .rtfScratch.RTF(0)
        If .rtfScratch.RTFBlocks > 1 Then
          DoSaveAs = DoSaveAs & .rtfScratch.RTF(1)
        End If
    End Select
  
    If fNeedFileDlg Then
      Put fno, , DoSaveAs
      Close fno
    End If
    
    Screen.MousePointer = default
  End With
End Function

Sub SetFont(fontCtl As Font, ctl As Control)
  On Error Resume Next
  With ctl
    fontCtl.Bold = .FontBold
    fontCtl.Italic = .FontItalic
    fontCtl.Name = .FontName
    fontCtl.Size = .FontSize
    fontCtl.Strikethrough = .FontStrikethru
    fontCtl.Underline = .FontUnderline
  End With
End Sub

Sub RenderLine(ctlCaption As Control, strFieldVal As String, fontCtl As Font, foreColor As Long)
  If (Not ctlCaption Is Nothing) Or strFieldVal <> "" Then
    RenderCaption ctlCaption
    If strFieldVal <> "" Then
      AddText strFieldVal, fontCtl, False, foreColor
    End If
    ParaFormat.StartIndent = IndentLevel * IndentSize
    SendMessage hwndRE, EM_SETPARAFORMAT, 0, ParaFormat
    AddText CrLf, Nothing, False, 0
  End If
End Sub



Function StripMnemonic(ByVal str As String)
  Dim istr As Integer
  istr = InStr(str, "&")
  Do While istr
    str = left$(str, istr - 1) & Mid$(str, istr + 1)
    istr = InStr(istr + 1, str, "&")
  Loop
  StripMnemonic = str
End Function

Sub AddCrLf(fontCtl As Font, foreColor As Long)
  AddText CrLf, fontCtl, False, foreColor
End Sub
