// ExRaidoc.cpp : implementation of the CExRaidDoc class
//

#include "stdafx.h"
#include "ExRaid.h"

#define INITGUID
#define USES_PS_PUBLIC_STRINGS
#include <mapix.h>
#include <edkmdb.h>

#include <mapiguid.h>

#include <time.h>

#include "ExRaiset.h"
#include "ExRaidoc.h"
#include "ExRaiVw.h"

#include "dboptdlg.h"
#include "pfoptdlg.h"

// Changed from PR_IPM_SUBTREE_ENTRYID to the below value
// Due to changes in the public folder store.

static SizedSPropTagArray(1, sptIPMRoot) =
{
	1,
	{
		PR_IPM_PUBLIC_FOLDERS_ENTRYID,
	}
};

static SizedSPropTagArray(2, sptStoreCols) =
{
	2,
	{
		PR_ENTRYID,
		PR_DISPLAY_NAME,
	}
};

typedef enum _PROPTAGINDEX {
    ptiBugID,
    ptitimestamp,
    ptiLinkCount,
    ptiRev,
    ptiStatus,
    ptiAssignedTo,
    ptiActiveDB,
    ptiIssueType,
    ptiSeverity,  		
    ptiPriority,		// 10
    ptiChangedDate,
    ptiChangedBy,
    ptiEnvironment,
    ptiOpenedDate,
    ptiOpenedRev,
    ptiSource,
    ptiSourceID,
    ptiLang,  			
    ptiResolvedDate,	
    ptiResolvedBy,		// 20
    ptiResolution,
    ptiFixedRev,
    ptiCodeChange,
    ptiClosedDate,
    ptiClosedBy,
    ptiKBArticle,
    ptihowfound,		// 30
    ptiCause,
	ptiMSEXCHANGE01,
    ptiMax
} PROPTAGINDEX;
        
static COLHEADERS colhExRaid[] = {
    {"BugID",        PT_LONG},
    {"timestamp",    PT_BINARY},
    {"LinkCount",    PT_LONG},
    {"Rev",          PT_LONG},
    {"Status",       PT_STRING8},
    {"AssignedTo",   PT_STRING8},
    {"ActiveDB",     PT_STRING8},
    {"IssueType",    PT_STRING8},
    {"Severity",     PT_STRING8}, 
    {"Priority",     PT_STRING8}, // 10
    {"ChangedDate",  PT_SYSTIME},
    {"ChangedBy",    PT_STRING8},
    {"Environment",  PT_STRING8},
    {"OpenedDate",   PT_SYSTIME},
    {"OpenedRev",    PT_STRING8},
    {"Source",       PT_STRING8},
    {"SourceID",     PT_STRING8},
    {"Lang",         PT_STRING8},  
    {"ResolvedDate", PT_SYSTIME},
    {"ResolvedBy",   PT_STRING8}, // 20
    {"Resolution",   PT_STRING8},
    {"FixedRev",     PT_STRING8},
    {"CodeChange",   PT_STRING8},
    {"ClosedDate",   PT_SYSTIME},
    {"ClosedBy",     PT_STRING8},
    {"KBArticle",    PT_STRING8},
    {"HowFound",     PT_STRING8}, // 30
    {"Cause",        PT_STRING8},
	{"MS_EXCHANGE_01", PT_BOOLEAN},  
    // This is a magic property needed by Forms.
};

#define ulcolhMax sizeof(colhExRaid) / sizeof(COLHEADERS)
#define ulMaxColHName 14
#define ulMaxWColHName (ulMaxColHName + 1) * 2

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CExRaidDoc

IMPLEMENT_DYNCREATE(CExRaidDoc, CDocument)

BEGIN_MESSAGE_MAP(CExRaidDoc, CDocument)
	//{{AFX_MSG_MAP(CExRaidDoc)
	ON_COMMAND(ID_OPTIONS_RAIDDATABASE, OnOptionsRaiddatabase)
	ON_COMMAND(ID_OPTIONS_PUBLICFOLDER, OnOptionsPublicfolder)
	ON_COMMAND(ID_SYNCRONIZE_NOW, OnSyncronizeNow)
	ON_COMMAND(ID_SYNCRONIZE_ALL, OnSyncronizeAll)
	ON_COMMAND(ID_SYNCRONIZE_RUNNING, OnSyncronizeRunning)
	ON_UPDATE_COMMAND_UI(ID_SYNCRONIZE_RUNNING, OnUpdateSyncronizeRunning)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CExRaidDoc construction/destruction

CExRaidDoc::CExRaidDoc()
{
	// TODO: add one-time construction code here
	
}

CExRaidDoc::~CExRaidDoc()
{
}

BOOL CExRaidDoc::OnNewDocument()
{
    CWinApp * pApp = AfxGetApp();
	HKEY    hk;
    CString szSection;
    CString szEntry;
    CString szDefault;
    CString szSubKey;
    CString szSoftware;
    CString szPFOptions;
    CString szError;
    CString szMaxTimestamp;
    LONG    lRet;
    DWORD   cbData, dwType;
	int     i;
	
	if (!CDocument::OnNewDocument())
		return FALSE;

    // Initialize variables

	m_fRunning = FALSE;
    
    // Load ExRaid Specific Registry Information (defaults).
    
    szSection.LoadString(IDS_SECTION_DATABASE);

    szEntry.LoadString(IDS_REG_DATABASE);
    szDefault.LoadString(IDS_DEFAULT_DATABASE);
    m_szDatabase = pApp->GetProfileString(szSection, szEntry, szDefault);
    
    szEntry.LoadString(IDS_REG_USERID);
    szDefault.LoadString(IDS_DEFAULT_USERID);
    m_szUserID = pApp->GetProfileString(szSection, szEntry, szDefault);
    
    szEntry.LoadString(IDS_REG_PASSWORD);
    szDefault.LoadString(IDS_DEFAULT_PASSWORD);
    m_szPassword = pApp->GetProfileString(szSection, szEntry, szDefault);
    
    szEntry.LoadString(IDS_REG_OUTPUTFILE);
    szDefault.LoadString(IDS_DEFAULT_OUTPUTFILE);
    m_szOutputFile = pApp->GetProfileString(szSection, szEntry, szDefault);
    
    szSection.LoadString(IDS_SECTION_PUBLICFOLDER);

    szEntry.LoadString(IDS_REG_PROFILENAME);
    szDefault.LoadString(IDS_DEFAULT_PROFILENAME);
    m_szProfileName = pApp->GetProfileString(szSection, szEntry, szDefault);
    
    szEntry.LoadString(IDS_REG_FOLDERNAME);
    szDefault.LoadString(IDS_DEFAULT_FOLDERNAME);
    m_szFolderName = pApp->GetProfileString(szSection, szEntry, szDefault);
    
    szEntry.LoadString(IDS_REG_MESSAGECLASS);
    szDefault.LoadString(IDS_DEFAULT_MESSAGECLASS);
    m_szMessageClass = pApp->GetProfileString(szSection, szEntry, szDefault);
    
    szEntry.LoadString(IDS_REG_SYNCTIME);
    m_cSyncTime = pApp->GetProfileInt(szSection, szEntry, DEFAULT_SYNCTIME);

	szEntry.LoadString(IDS_REG_SYNCTIMEUNITS);
    m_stUnits = pApp->GetProfileInt(szSection, szEntry, DEFAULT_SYNCTIMEUNITS);

	szEntry.LoadString(IDS_REG_SYNCRUNNING);
	m_fRunning = pApp->GetProfileInt(szSection, szEntry, FALSE);

	// Set Default SQL connect string

	m_exRaidSet.m_szDefaultConnect = "ODBC;DSN=" + m_szDatabase + ";PWD=" + m_szPassword + ";UID=" + m_szUserID;

    // Restore Maxtimestamp
    
    m_cbaMaxTimestamp.SetSize(cbTimestamp);
    cbData = cbTimestamp;
    szSoftware.LoadString(IDS_REG_SOFTWARE);
    szPFOptions.LoadString(IDS_SECTION_PUBLICFOLDER);
    szMaxTimestamp.LoadString(IDS_REG_MAXTIMESTAMP);
    szSubKey = szSoftware + "\\" + pApp->m_pszRegistryKey + "\\" + 
        pApp->m_pszAppName + "\\" + szPFOptions;

    lRet = RegOpenKeyEx(HKEY_CURRENT_USER, szSubKey, 0, KEY_ALL_ACCESS, &hk);
    if (lRet != ERROR_SUCCESS)
    {
        AfxFormatString1(szError, IDS_ERROR_REGOPENKEYEX, szSubKey);
		AfxMessageBox(szError, MB_ICONEXCLAMATION);
        return FALSE;
    }

    lRet = RegQueryValueEx(hk, 
                           szMaxTimestamp, 
                           0, 
                           &dwType, 
                           &(m_cbaMaxTimestamp[0]),
                           &cbData);
    if (lRet != ERROR_SUCCESS)
    {
        for (i = 0 ; i < cbTimestamp ; i++)
        {
            m_cbaMaxTimestamp[i] = 0;
        }
    }
    
    lRet = RegCloseKey(hk);
    if (lRet != ERROR_SUCCESS)
    {
        AfxFormatString1(szError, IDS_ERROR_REGCLOSEKEY, szSubKey);
        AfxMessageBox(szError, MB_ICONEXCLAMATION);
        return FALSE;
    }        
	

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CExRaidDoc diagnostics

#ifdef _DEBUG
void CExRaidDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CExRaidDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CExRaidDoc commands

void CExRaidDoc::OnOptionsRaiddatabase() 
{
	DBOptDlg dlg;
	
	dlg.m_szDatabase = m_szDatabase;
	dlg.m_szUserID = m_szUserID;
	dlg.m_szPassword = m_szPassword;
	dlg.m_szOutputFile = m_szOutputFile;

	if (dlg.DoModal() == IDOK)
	{
		// If a new database is selected, clear the timestamp.
		if (m_szDatabase != dlg.m_szDatabase)
		{
			int i;

			m_cbaMaxTimestamp.SetSize(cbTimestamp);
		    for (i = 0 ; i < cbTimestamp ; i++)
		    {
		        m_cbaMaxTimestamp[i] = 0;
		    }
		}
			
		m_szOutputFile = dlg.m_szOutputFile;
		m_szDatabase = dlg.m_szDatabase;
		m_szUserID = dlg.m_szUserID;
		m_szPassword = dlg.m_szPassword;
		m_exRaidSet.m_szDefaultConnect = "ODBC;DSN=" + m_szDatabase + ";PWD=" + m_szPassword + ";UID=" + m_szUserID;
	}
}

void CExRaidDoc::OnOptionsPublicfolder() 
{
	PFOptDlg dlg;
	
	dlg.m_szProfileName = m_szProfileName;
	dlg.m_szFolderName = m_szFolderName;
	dlg.m_szMessageClass = m_szMessageClass;
	dlg.m_cSyncTime = m_cSyncTime;
	dlg.m_stUnits = m_stUnits;
	
	if (dlg.DoModal() == IDOK)
	{
		// If the folder name has changed, then clear the timestamp.
		if (m_szFolderName != dlg.m_szFolderName)
		{
			int i;

			m_cbaMaxTimestamp.SetSize(cbTimestamp);
		    for (i = 0 ; i < cbTimestamp ; i++)
		    {
		        m_cbaMaxTimestamp[i] = 0;
		    }
		}

		m_szProfileName = dlg.m_szProfileName;
		m_szFolderName = dlg.m_szFolderName;
		m_szMessageClass = dlg.m_szMessageClass;
		m_cSyncTime = dlg.m_cSyncTime;
		m_stUnits = dlg.m_stUnits;
	}
}

void CExRaidDoc::OnCloseDocument() 
{
	CWinApp * pApp = AfxGetApp();
    CString szSection;
    CString szEntry;
    BOOL    fRet;
    HKEY    hk = NULL;
    CString szSubKey;
    CString szSoftware;
    CString szPFOptions;
    CString szError;
    CString szMaxTimestamp;
    LONG    lRet;
    DWORD   cbData;
	RECT    rcApp;
	CWnd *  pMainWnd;

    // Save ExRaid Specific Registry Information.
    
    szSection.LoadString(IDS_SECTION_DATABASE);

    szEntry.LoadString(IDS_REG_DATABASE);
    fRet = pApp->WriteProfileString(szSection, szEntry, m_szDatabase);
    ASSERT(fRet);
    
    szEntry.LoadString(IDS_REG_USERID);
    fRet = pApp->WriteProfileString(szSection, szEntry, m_szUserID);
    ASSERT(fRet);
    
    szEntry.LoadString(IDS_REG_PASSWORD);
    fRet = pApp->WriteProfileString(szSection, szEntry, m_szPassword);
    ASSERT(fRet);
    
    szEntry.LoadString(IDS_REG_OUTPUTFILE);
    fRet = pApp->WriteProfileString(szSection, szEntry, m_szOutputFile);
    ASSERT(fRet);
    
    szSection.LoadString(IDS_SECTION_PUBLICFOLDER);

    szEntry.LoadString(IDS_REG_PROFILENAME);
    fRet = pApp->WriteProfileString(szSection, szEntry, m_szProfileName);
    ASSERT(fRet);
    
    szEntry.LoadString(IDS_REG_FOLDERNAME);
    fRet = pApp->WriteProfileString(szSection, szEntry, m_szFolderName);
    ASSERT(fRet);
    
    szEntry.LoadString(IDS_REG_MESSAGECLASS);
    fRet = pApp->WriteProfileString(szSection, szEntry, m_szMessageClass);
    ASSERT(fRet);
    
    szEntry.LoadString(IDS_REG_SYNCTIME);
    fRet = pApp->WriteProfileInt(szSection, szEntry, m_cSyncTime);
    ASSERT(fRet);

    szEntry.LoadString(IDS_REG_SYNCTIMEUNITS);
    fRet = pApp->WriteProfileInt(szSection, szEntry, m_stUnits);
    ASSERT(fRet);

	szEntry.LoadString(IDS_REG_SYNCRUNNING);
	fRet = pApp->WriteProfileInt(szSection, szEntry, m_fRunning);
	ASSERT(fRet);

    // Save Maxtimestamp
    
    cbData = m_cbaMaxTimestamp.GetSize();
    szSoftware.LoadString(IDS_REG_SOFTWARE);
    szPFOptions.LoadString(IDS_SECTION_PUBLICFOLDER);
    szMaxTimestamp.LoadString(IDS_REG_MAXTIMESTAMP);
    szSubKey = szSoftware + "\\" + pApp->m_pszRegistryKey + "\\" + 
        pApp->m_pszAppName + "\\" + szPFOptions;

    lRet = RegOpenKeyEx(HKEY_CURRENT_USER, szSubKey, 0, KEY_ALL_ACCESS, &hk);
    if (lRet != ERROR_SUCCESS)
    {
        AfxFormatString1(szError, IDS_ERROR_REGOPENKEYEX, szSubKey);
        AfxMessageBox(szError, MB_ICONEXCLAMATION);
        goto Cleanup;
    }

    lRet = RegSetValueEx(hk, 
                         szMaxTimestamp, 
                         0, 
                         REG_BINARY, 
                         &(m_cbaMaxTimestamp[0]),
                         cbData);
    if (lRet != ERROR_SUCCESS)
    {
        AfxFormatString1(szError, IDS_ERROR_REGSETVALUEEX, szMaxTimestamp);
        AfxMessageBox(szError, MB_ICONEXCLAMATION);
        goto Cleanup;
    }

	// Save Window Position

	pMainWnd = pApp->GetMainWnd();
	pMainWnd->GetWindowRect(&rcApp);

	szSection.LoadString(IDS_SECTION_SETTINGS);
	szEntry.LoadString(IDS_REG_LEFT);
	fRet = pApp->WriteProfileInt(szSection, szEntry, rcApp.left);
	ASSERT(fRet);
	szEntry.LoadString(IDS_REG_TOP);
	fRet = pApp->WriteProfileInt(szSection, szEntry, rcApp.top);
	ASSERT(fRet);
	szEntry.LoadString(IDS_REG_RIGHT);
	fRet = pApp->WriteProfileInt(szSection, szEntry, rcApp.right);
	ASSERT(fRet);
	szEntry.LoadString(IDS_REG_BOTTOM);
	fRet = pApp->WriteProfileInt(szSection, szEntry, rcApp.bottom);
	ASSERT(fRet);

Cleanup:

	if (hk)
	{
	    lRet = RegCloseKey(hk);
	    if (lRet != ERROR_SUCCESS)
	    {
	        AfxFormatString1(szError, IDS_ERROR_REGCLOSEKEY, szSubKey);
	        AfxMessageBox(szError, MB_ICONEXCLAMATION);
	    }
	}

	CDocument::OnCloseDocument();
}

HRESULT
HrProptagsFromColHeaders(LPMAPIFOLDER pfolder, PCOLHEADERS pcolh, LPSPropValue lpspv, int * pcprop, CString * pszError)
{
	int		     i, iChar;
	HRESULT      hresult;
    MAPINAMEID	 rgnameid[ulcolhMax];
    LPMAPINAMEID pnameid = rgnameid;
    LPMAPINAMEID rgpmnid[ulcolhMax];
    LPMAPINAMEID * ppmnid = rgpmnid;
    WCHAR		 rgpwch[ulcolhMax][ulMaxWColHName];
    PCOLHEADERS  pcolhTmp = pcolh;
    LPSPropTagArray	lpSPropTagsT = NULL;
    LPSPropValue lpspvTmp;

	// make a named property.

    for (i = 0 ; i < (int) ulcolhMax ; i++)
    {
        iChar = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, pcolhTmp->pszName, -1, rgpwch[i], ulMaxWColHName);
        if (iChar == 0)
        {
            (*pszError).LoadString(IDS_ERROR_MULTIBYTETOWIDECHAR);
            hresult = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, IDS_ERROR_MULTIBYTETOWIDECHAR);
            return hresult;
        }

        pnameid->lpguid = (GUID*)&PS_PUBLIC_STRINGS;
        pnameid->ulKind = MNID_STRING;
        pnameid->Kind.lpwstrName = rgpwch[i];
        rgpmnid[i] = pnameid;
        pnameid++;
        pcolhTmp++;
    }

	hresult = pfolder->GetIDsFromNames(ulcolhMax, ppmnid, MAPI_CREATE, &lpSPropTagsT);
    if (hresult != S_OK)
    {
        (*pszError).LoadString(IDS_ERROR_GETIDSFROMNAMES);
        return hresult;
    }

    pcolhTmp = pcolh;
    lpspvTmp = lpspv;
    for (i = 0 ; i < ulcolhMax ; i++)
    {
        switch (pcolhTmp->ulType)
        {
            case PT_STRING8:
            case PT_LONG:
            case PT_BINARY:
            case PT_SYSTIME:
			case PT_BOOLEAN:
                lpspvTmp->ulPropTag = PROP_TAG(pcolhTmp->ulType, PROP_ID(lpSPropTagsT->aulPropTag[i]));
                break;
            default:
                (*pszError).LoadString(IDS_ERROR_PT_UNKNOWN);
                hresult = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 2);
                return hresult;
                break;
        }
        pcolhTmp++;
		lpspvTmp++;
    }

    (*pcprop) = ulcolhMax;
    MAPIFreeBuffer(lpSPropTagsT);
    return S_OK;

}

HRESULT
HrOpenPublicFolder(LPMAPISESSION pmapisess, CString szFolderPath, LPMDB * ppmdb, LPMAPIFOLDER * ppfolder, CString * pszError)
{
	HRESULT			hresult;
	LPMAPITABLE 	ptableStores = NULL;
	LPMAPITABLE 	ptableFolders = NULL;
	LPSPropValue	ppropRoot = NULL;
	ULONG			cpropRoot;
	LPSPropValue	pprop;
	char *			szTok;
	LPSRowSet		lpRows = NULL;
	SPropValue		val;
	SRestriction	rgsres[1];
	ULONG			ulObjType;

	szTok = strtok(szFolderPath.GetBuffer(0), "\\");
	if (!szTok)
	{
		hresult = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 1);
		AfxFormatString1((*pszError), IDS_ERROR_FOLDERPATHNOTFOUND, szFolderPath);
		goto Cleanup;
	}

	// get stores table
	hresult = pmapisess->GetMsgStoresTable(NULL, &ptableStores);
	if (hresult != S_OK)
	{
        (*pszError).LoadString(IDS_ERROR_GETMSGSTORESTABLE);
		goto Cleanup;
	}

	// setup restriction for all find rows
	val.ulPropTag = PR_DISPLAY_NAME;
	rgsres[0].rt = RES_PROPERTY;
	rgsres[0].res.resProperty.relop = RELOP_EQ;
	rgsres[0].res.resProperty.ulPropTag = PR_DISPLAY_NAME;
	rgsres[0].res.resProperty.lpProp = &val;

	// find store with correct name
	{
		// setup find info to find store
		val.Value.LPSZ = szTok;

		// set columns in table
		hresult = ptableStores->SetColumns((LPSPropTagArray) &sptStoreCols, TBL_NOWAIT);
		if (hresult != S_OK)
		{
            (*pszError).LoadString(IDS_ERROR_SETCOLUMNS);
			goto Cleanup;
		}

	 	hresult = ptableStores->FindRow(rgsres, BOOKMARK_BEGINNING, 0);
		if (hresult != S_OK)
		{
            (*pszError).LoadString(IDS_ERROR_FINDROW);
			goto Cleanup;
		}

		hresult = ptableStores->QueryRows(1, 0, &lpRows);
		if (hresult != S_OK)
		{
	        (*pszError).LoadString(IDS_ERROR_QUERYROWS);
			goto Cleanup;
		}

		// open store
		pprop = lpRows->aRow[0].lpProps;
		hresult = pmapisess->OpenMsgStore(NULL, 
                                          pprop->Value.bin.cb, 
                                          (LPENTRYID)pprop->Value.bin.lpb, 
                                          NULL, 
                                          MDB_WRITE | MAPI_DEFERRED_ERRORS, 
                                          ppmdb);
		if (hresult != S_OK)
		{
            AfxFormatString1((*pszError), IDS_ERROR_OPENMSGSTORE, szTok);
			goto Cleanup;
		}

		MAPIFreeBuffer(lpRows->aRow[0].lpProps);
		MAPIFreeBuffer(lpRows);
		lpRows = NULL;
	}

	// find Root of Public Folder tree.
	hresult = (*ppmdb)->GetProps((LPSPropTagArray) &sptIPMRoot, 0, &cpropRoot, &ppropRoot);
	if (hresult != S_OK)
	{
        (*pszError).LoadString(IDS_ERROR_GETPROPSIPMROOT);
		goto Cleanup;
	}

	// open Root	
	hresult = (*ppmdb)->OpenEntry(ppropRoot->Value.bin.cb, 
                                  (LPENTRYID) ppropRoot->Value.bin.lpb,
                                  NULL,
                                  MAPI_MODIFY, 
                                  &ulObjType,
                                  (IUnknown**) ppfolder);
	if (hresult != S_OK)
	{
		(*pszError).LoadString(IDS_ERROR_OPENFOLDERIPMROOT);
        goto Cleanup;
	}

	// while more path components look through folder tree for folder
	while (val.Value.LPSZ = strtok(NULL, "\\"))
	{
		LPMAPIFOLDER  pfolderTemp;
		
		hresult = (*ppfolder)->GetHierarchyTable(MAPI_DEFERRED_ERRORS, &ptableFolders);
		if (hresult != S_OK)
		{
            pszError->LoadString(IDS_ERROR_GETHIERARCHYTABLE);
            AfxMessageBox((*pszError), MB_ICONEXCLAMATION);
			goto Cleanup;
        }

		hresult = ptableFolders->SetColumns((LPSPropTagArray) &sptStoreCols, TBL_BATCH);
        if (hresult != S_OK)
        {
            pszError->LoadString(IDS_ERROR_SETCOLUMNS);
            AfxMessageBox((*pszError), MB_ICONEXCLAMATION);
			goto Cleanup;
        }

	 	hresult = ptableFolders->FindRow(rgsres, BOOKMARK_BEGINNING, 0);
        if (hresult == S_OK)
        {
            hresult = ptableFolders->QueryRows(1, 0, &lpRows);
            if (hresult != S_OK)
            {
                pszError->LoadString(IDS_ERROR_QUERYROWS);
                AfxMessageBox((*pszError), MB_ICONEXCLAMATION);
                goto Cleanup;
            }
            
            pprop = lpRows->aRow[0].lpProps;
            hresult = (*ppfolder)->OpenEntry(pprop->Value.bin.cb, 
                                             (LPENTRYID)pprop->Value.bin.lpb, 
                                             NULL,
                                             MAPI_DEFERRED_ERRORS | MAPI_BEST_ACCESS, 
                                             &ulObjType,
                                             (IUnknown**)&pfolderTemp);
            if (hresult != S_OK)
            {
                AfxFormatString1((*pszError), IDS_ERROR_OPENENTRYFOLDER, val.Value.LPSZ);
                AfxMessageBox((*pszError), MB_ICONEXCLAMATION);
                goto Cleanup;
            }
            
            MAPIFreeBuffer(lpRows->aRow[0].lpProps);
            MAPIFreeBuffer(lpRows);
            lpRows = NULL;
            
        }
        else
		{
			hresult = (*ppfolder)->CreateFolder(FOLDER_GENERIC,
												val.Value.LPSZ,
												NULL,
												NULL,
												MAPI_DEFERRED_ERRORS | OPEN_IF_EXISTS,
												&pfolderTemp);
			if (hresult != S_OK)
			{
				AfxFormatString1((*pszError), IDS_ERROR_CREATEFOLDER, val.Value.LPSZ);
				goto Cleanup;
			}
		}
            
		(*ppfolder)->Release();
		(*ppfolder) = pfolderTemp;
	}
				
Cleanup:
	if (lpRows)
	{
		MAPIFreeBuffer(lpRows->aRow[0].lpProps);
		MAPIFreeBuffer(lpRows);
	}
	if (ppropRoot)
		MAPIFreeBuffer(ppropRoot);
	if (ptableFolders)
		ptableFolders->Release();
	if (ptableStores)
		ptableStores->Release();

	return hresult;
}

HRESULT
HrOpenPublicFolderContents(LPMAPIFOLDER pfolder, LPSPropValue pspv, LPMAPITABLE * pptableContents, CString * pszError)
{
    HRESULT         hr;

    SizedSSortOrderSet(1, sort);
    // PR_DISPLAY_NAME will be replaced below by other properties.
    static SizedSPropTagArray(3, sptContentsCols) =
    {
        3,
        {
            PR_ENTRYID,
            PR_DISPLAY_NAME, 
            PR_DISPLAY_NAME,
        }
    };

    hr = pfolder->GetContentsTable(MAPI_DEFERRED_ERRORS, pptableContents);
    if (hr != S_OK)
    {
		(*pszError).LoadString(IDS_ERROR_GETCONTENTSTABLE);
		goto Cleanup;
    }

    sptContentsCols.aulPropTag[1] = pspv[ptiBugID].ulPropTag;
    sptContentsCols.aulPropTag[2] = pspv[ptitimestamp].ulPropTag;

    // set columns in table
    hr = (*pptableContents)->SetColumns((LPSPropTagArray) &sptContentsCols, TBL_BATCH);

    if (hr != S_OK)
    {
		(*pszError).LoadString(IDS_ERROR_SETCOLUMNS);
		goto Cleanup;
    }

    sort.cSorts = 1;
    sort.cCategories = 0;
    sort.cExpanded = 0;
    sort.aSort[0].ulPropTag = pspv[ptiBugID].ulPropTag;
    sort.aSort[0].ulOrder = TABLE_SORT_ASCEND;

    hr = (*pptableContents)->SortTable((LPSSortOrderSet)&sort, TBL_BATCH);
	//hr = (*pptableContents)->SortTable((LPSSortOrderSet)&sort, 0);
    switch (hr)
	{
		case S_OK:
			break;
		default:
	        (*pszError).LoadString(IDS_ERROR_SORTTABLE);
			goto Cleanup;
    }

Cleanup:
    // Nothing to clean up in this case.
    // Left in for future error handling.
    return hr;
}

#define cbMaxFileSize 0x00FFFFFF
#define	cpropMax	100
void CExRaidDoc::OnSyncronize(BOOL fSyncAll) 
{
    INT   			i, cpropCols, ipropCol, iRet;
    HRESULT 		hr;
    LPMAPISESSION 	pms = NULL;
    LPMDB 			pmdb = NULL;
    LPMAPIFOLDER 	pmfExRaid = NULL;
    LPMESSAGE    	pm = NULL;
    SPropValue 		rgpropvalCols[cpropMax];
    ULONG			ulObjType;
    SRestriction	rgsres[1];
    LPSPropValue	pprop;
    CByteArray      cbaNewMaxTimestamp;
    LPSRowSet		lpRows = NULL;
    LPMAPITABLE 	ptableContents = NULL;
	CString 		szTimestamp;
	char *  		pszTimestamp;
	int     		cb;
	BOOL            fRet;
	LONG            lCurrentBugID;
    CTime           ctLastUpdated;
	CTime           ctBeginUpdate = CTime::GetCurrentTime();
	CTimeSpan       ctsTimeToUpdate;
    HANDLE          hOutputFile = INVALID_HANDLE_VALUE;
    float           flBugsPerSec = (float) 0.0;
    CString         szOutput;
	CString			szEmailAddr;
	CString			szDomainName("microsoft.com");
	SECURITY_ATTRIBUTES sa = {sizeof(SECURITY_ATTRIBUTES), NULL, TRUE};
	DWORD           dwError;
	DWORD           dwFile;
	DWORD           dwBytesWritten;
	
	// Get the RecordSet, m_exRaidSet, set to beginning
	// Reset the RecordSet to the beginning
    // EXPERF: Could assume beginning to avoid these calls as long
    //  as everybody sets to beginning.

	// Clear out error handling string.
	m_szError.Empty();

	// Reset maximum timestamp to 00000000
	cbaNewMaxTimestamp.SetSize(cbTimestamp);
    for (i = 0 ; i < cbTimestamp ; i++)
    {
        cbaNewMaxTimestamp[i] = 0;
    }

	// Check for a full syncronization with SQL Raid database.
	if (fSyncAll)
	{
		for (i = 0 ; i < cbTimestamp ; i++)
		{
			m_cbaMaxTimestamp.SetAt(i, cbaNewMaxTimestamp[i]);
		}
	}

	// Reset counters
	m_cBugsCreated = 0;
	m_cBugsUpdated = 0;
	m_cBugsRead = 0;

	// Query Raid SQL database for all bugs > than saved timestamp
    pszTimestamp = szTimestamp.GetBufferSetLength((cbTimestamp + 2) * 2);
	cb = sprintf(pszTimestamp, "0x");
	pszTimestamp += cb;
	for (i = 0 ; i < cbTimestamp ; i++)
	{
		cb = sprintf(pszTimestamp, "%02x", m_cbaMaxTimestamp[i]);
		pszTimestamp += cb;
	}
	szTimestamp.ReleaseBuffer((cbTimestamp + 1) * 2);
			
	m_exRaidSet.m_strFilter = "timestamp > " + szTimestamp;

	try 
	{
		if (m_exRaidSet.CanRestart())
		{
			fRet = m_exRaidSet.Requery();
			if (!fRet)
			{
		        m_szError.LoadString(IDS_ERROR_REQUERY);
		        goto Cleanup;
			}
		}
	}
	catch (CDBException* e)
	{
		m_szError.LoadString(IDS_ERROR_DBEXCEPTION);
		if (e->m_strError != "")
			m_szError = m_szError + ": " + e->m_strError;
		hr = !S_OK;
		m_exRaidSet.Close();
		m_exRaidSet.Open();
		goto Cleanup;
	}	

	// If no records, then Public Folder is up to date.
	if (m_exRaidSet.IsBOF() && m_exRaidSet.IsEOF())
		goto Cleanup;

    hr = MAPILogonEx(NULL,
                     m_szProfileName.GetBuffer(0),
                     NULL,
                     MAPI_EXPLICIT_PROFILE | MAPI_EXTENDED | MAPI_LOGON_UI | MAPI_NO_MAIL,
                     &pms);
    if (hr != S_OK)
    {
        AfxFormatString1(m_szError, IDS_ERROR_MAPILOGONEX, m_szProfileName);
        goto Cleanup;
	}
        
    // Open Folder
    hr = HrOpenPublicFolder(pms, m_szFolderName, &pmdb, &pmfExRaid, &m_szError);
    if (hr != S_OK)
        goto Cleanup;
    
    // Get Column Headers

    hr = HrProptagsFromColHeaders(pmfExRaid, colhExRaid, rgpropvalCols, &cpropCols, &m_szError);
	if (hr != S_OK)
		goto Cleanup;
                                  
    // Open Contents Table of Public Folder
    
    hr = HrOpenPublicFolderContents(pmfExRaid, rgpropvalCols, &ptableContents, &m_szError);
    if (hr != S_OK)
        goto Cleanup;
    
	// Set up restriction for FindRow on BugID, later code will add BugID to find.
    rgsres[0].rt = RES_PROPERTY;
    rgsres[0].res.resProperty.relop = RELOP_EQ;
    rgsres[0].res.resProperty.ulPropTag = rgpropvalCols[ptiBugID].ulPropTag;

	// Iterate through the RecordSet, writing to the file
    while (!m_exRaidSet.IsEOF())
    {
        rgpropvalCols[ptiBugID].Value.l = m_exRaidSet.m_BugID;
		lCurrentBugID = m_exRaidSet.m_BugID;
        rgpropvalCols[ptitimestamp].Value.bin.cb = m_exRaidSet.m_timestamp.GetSize();
        rgpropvalCols[ptitimestamp].Value.bin.lpb = &(m_exRaidSet.m_timestamp[0]);

        // Save new maximum timestamp, to be set after all bugs are processed.
        iRet = memcmp(&(m_exRaidSet.m_timestamp[0]),
                      &(cbaNewMaxTimestamp[0]), 
                      cbaNewMaxTimestamp.GetSize());
        if (iRet > 0)
        {
            memcpy(&(cbaNewMaxTimestamp[0]),&(m_exRaidSet.m_timestamp[0]), cbaNewMaxTimestamp.GetSize());
        }

		rgpropvalCols[ptiLinkCount].Value.l = m_exRaidSet.m_LinkCount;
        rgpropvalCols[ptiRev].Value.l = m_exRaidSet.m_Rev;
        rgpropvalCols[ptiStatus].Value.lpszA = m_exRaidSet.m_Status.GetBuffer(0);
        rgpropvalCols[ptiAssignedTo].Value.lpszA = m_exRaidSet.m_AssignedTo.GetBuffer(0);
        rgpropvalCols[ptiActiveDB].Value.lpszA = m_exRaidSet.m_ActiveDB.GetBuffer(0);
        rgpropvalCols[ptiIssueType].Value.lpszA = m_exRaidSet.m_IssueType.GetBuffer(0);
        rgpropvalCols[ptiSeverity].Value.lpszA = m_exRaidSet.m_Severity.GetBuffer(0);
        rgpropvalCols[ptiPriority].Value.lpszA = m_exRaidSet.m_Priority.GetBuffer(0);
        if (!FTimeTToFileTime(m_exRaidSet.m_ChangedDate.GetTime(), &(rgpropvalCols[ptiChangedDate].Value.ft)))
            goto Cleanup;
        rgpropvalCols[ptiChangedBy].Value.lpszA = m_exRaidSet.m_ChangedBy.GetBuffer(0);
        rgpropvalCols[ptiEnvironment].Value.lpszA = m_exRaidSet.m_Environment.GetBuffer(0);
        if (!FTimeTToFileTime(m_exRaidSet.m_OpenedDate.GetTime(), &(rgpropvalCols[ptiOpenedDate].Value.ft)))
            goto Cleanup;
        rgpropvalCols[ptiOpenedRev].Value.lpszA = m_exRaidSet.m_OpenedRev.GetBuffer(0);
        rgpropvalCols[ptiSource].Value.lpszA = m_exRaidSet.m_Source.GetBuffer(0);
        rgpropvalCols[ptiSourceID].Value.lpszA = m_exRaidSet.m_SourceID.GetBuffer(0);
        rgpropvalCols[ptiLang].Value.lpszA = m_exRaidSet.m_Lang.GetBuffer(0);
        if (!FTimeTToFileTime(m_exRaidSet.m_ResolvedDate.GetTime(), &(rgpropvalCols[ptiResolvedDate].Value.ft)))
            goto Cleanup;
        rgpropvalCols[ptiResolvedBy].Value.lpszA = m_exRaidSet.m_ResolvedBy.GetBuffer(0);
        rgpropvalCols[ptiResolution].Value.lpszA = m_exRaidSet.m_Resolution.GetBuffer(0);
        rgpropvalCols[ptiFixedRev].Value.lpszA = m_exRaidSet.m_FixedRev.GetBuffer(0);
        rgpropvalCols[ptiCodeChange].Value.lpszA = m_exRaidSet.m_CodeChange.GetBuffer(0);
        if (!FTimeTToFileTime(m_exRaidSet.m_ClosedDate.GetTime(), &(rgpropvalCols[ptiClosedDate].Value.ft)))
            goto Cleanup;
        rgpropvalCols[ptiClosedBy].Value.lpszA = m_exRaidSet.m_ClosedBy.GetBuffer(0);
        rgpropvalCols[ptiKBArticle].Value.lpszA = m_exRaidSet.m_KBArticle.GetBuffer(0);
        rgpropvalCols[ptihowfound].Value.lpszA = m_exRaidSet.m_howfound.GetBuffer(0);
        rgpropvalCols[ptiCause].Value.lpszA = m_exRaidSet.m_Cause.GetBuffer(0);
		rgpropvalCols[ptiMSEXCHANGE01].Value.b = TRUE;
        
        // Add PR_MESSAGE_CLASS and PR_MESSAGE_FLAGS, Title as PR_SUBJECT
		// and OpenedBy as PR_SENDER.
        rgpropvalCols[ulcolhMax].ulPropTag = PR_MESSAGE_CLASS;
        rgpropvalCols[ulcolhMax].Value.lpszA = m_szMessageClass.GetBuffer(0);

        rgpropvalCols[ulcolhMax + 1].ulPropTag = PR_MESSAGE_FLAGS;
        rgpropvalCols[ulcolhMax + 1].Value.ul = MSGFLAG_UNMODIFIED;

		rgpropvalCols[ulcolhMax + 2].ulPropTag = PR_SUBJECT;
		rgpropvalCols[ulcolhMax + 2].Value.lpszA = m_exRaidSet.m_Title.GetBuffer(0);

		rgpropvalCols[ulcolhMax + 3].ulPropTag = PR_SENDER_NAME_A;
		rgpropvalCols[ulcolhMax + 3].Value.lpszA = m_exRaidSet.m_OpenedBy.GetBuffer(0);

		rgpropvalCols[ulcolhMax + 4].ulPropTag = PR_SENDER_ADDRTYPE_A;
		rgpropvalCols[ulcolhMax + 4].Value.lpszA = "SMTP";

		rgpropvalCols[ulcolhMax + 5].ulPropTag = PR_SENDER_EMAIL_ADDRESS_A;
		szEmailAddr = m_exRaidSet.m_OpenedBy + "@" + szDomainName;
		rgpropvalCols[ulcolhMax + 5].Value.lpszA = szEmailAddr.GetBuffer(0);

		rgpropvalCols[ulcolhMax + 6].ulPropTag = PR_SENT_REPRESENTING_NAME_A;
		rgpropvalCols[ulcolhMax + 6].Value.lpszA = m_exRaidSet.m_OpenedBy.GetBuffer(0);

		rgpropvalCols[ulcolhMax + 7].ulPropTag = PR_SENT_REPRESENTING_ADDRTYPE_A;
		rgpropvalCols[ulcolhMax + 7].Value.lpszA = "SMTP";

		rgpropvalCols[ulcolhMax + 8].ulPropTag = PR_SENT_REPRESENTING_EMAIL_ADDRESS_A;
		rgpropvalCols[ulcolhMax + 8].Value.lpszA = szEmailAddr.GetBuffer(0);

	    rgpropvalCols[ulcolhMax + 9].ulPropTag = PR_BODY;
	    rgpropvalCols[ulcolhMax + 9].Value.lpszA = m_exRaidSet.m_Description.GetBuffer(0);

		ipropCol = ulcolhMax + 10;

        // Check for existing message.  Open if exists.
        
        rgsres[0].res.resProperty.lpProp = &(rgpropvalCols[ptiBugID]);

        // PERF: This has been optimized by the ExchangeIS by creating
        // an index on this property and calling JetSeek to find the ID.
        // Trying to FindRow from other than BOOKMARK_BEGINNING will
        // actually have worse performance due to position tracking.

        hr = ptableContents->FindRow(rgsres, BOOKMARK_BEGINNING, 0);
        if (hr == S_OK)
        {
            // Get EntryID, BugID, timestamp
            hr = ptableContents->QueryRows(1, 0, &lpRows);
            if (hr != S_OK)
            {
                m_szError.LoadString(IDS_ERROR_QUERYROWS);
                goto CreateMessage;
            }
                
            // Compare timestamp to see if message needs to be updated.
            
            iRet = memcmp(lpRows->aRow[0].lpProps[2].Value.bin.lpb, // timestamp
                          &(m_exRaidSet.m_timestamp[0]), 
                          m_exRaidSet.m_timestamp.GetSize());
            
            // If Exchange timestamp >= Raid timestamp, then message doesn't
            // need to be updated.  Go to the next message.
			            
            if (iRet >= 0)
            {
                MAPIFreeBuffer(lpRows->aRow[0].lpProps);
                MAPIFreeBuffer(lpRows);
                lpRows = NULL;                
                goto NextRecord;
            }
            
        
            // Open message
            pprop = lpRows->aRow[0].lpProps;
            hr = pmfExRaid->OpenEntry(pprop->Value.bin.cb,
                                      (LPENTRYID) pprop->Value.bin.lpb,
                                      NULL,
                                      MAPI_MODIFY | MAPI_DEFERRED_ERRORS,
                                      &ulObjType,
                                      (IUnknown **) &pm);
            if (hr != S_OK)
            {
                m_szError.LoadString(IDS_ERROR_OPENENTRYMESSAGE);
                goto Cleanup;
            }
            
            MAPIFreeBuffer(lpRows->aRow[0].lpProps);
            MAPIFreeBuffer(lpRows);
            lpRows = NULL;                
			m_cBugsUpdated++;
        }
        else
        {
        CreateMessage:
            hr = pmfExRaid->CreateMessage(NULL, MAPI_DEFERRED_ERRORS, &pm);
            if (hr != S_OK)
            {
                m_szError.LoadString(IDS_ERROR_CREATEMESSAGE);
                goto Cleanup;
            }
			m_cBugsCreated++;
        }

        // SetProps, SaveChanges and Release public folder message.

        hr = pm->SetProps(ipropCol, rgpropvalCols, NULL);
        if (hr != S_OK)
        {
            m_szError.LoadString(IDS_ERROR_SETPROPS);
            goto Cleanup;
        }
        
        hr = pm->SaveChanges(MAPI_DEFERRED_ERRORS);
        if (hr != S_OK)
        {
            m_szError.LoadString(IDS_ERROR_SAVECHANGES);
            goto Cleanup;
        }
            
        pm->Release();
        pm = NULL;
        
NextRecord: // Currently, continue to write full import file to disk
        // Goto next record
		try
		{
        	m_exRaidSet.MoveNext();
		}
		catch (CDBException *e)
		{
			m_szError.LoadString(IDS_ERROR_DBEXCEPTION);
			if (e->m_strError != "")
				m_szError = m_szError + ": " + e->m_strError;
			goto Cleanup;
		}

		if (lCurrentBugID == m_exRaidSet.m_BugID && !m_exRaidSet.IsEOF())
		{
			m_szError.LoadString(IDS_ERROR_MOVENEXT);
			goto Cleanup;
		}
		m_cBugsRead++;
    }

Cleanup:
    // Update date time, then update view
    ctLastUpdated = CTime::GetCurrentTime();    
	ctsTimeToUpdate = ctLastUpdated - ctBeginUpdate;

	m_szTimeToUpdate = ctsTimeToUpdate.Format("%H:%M:%S");
    m_szLastUpdated = ctLastUpdated.Format("%x %#I:%M:%S %p");

	m_cSeconds = ctsTimeToUpdate.GetTotalSeconds();
	if (m_cSeconds == 0)
		m_cSeconds = 1;
    
    flBugsPerSec = (float) (m_cBugsCreated + m_cBugsUpdated) / (float) m_cSeconds;
    m_szBugsPerSec.Format("%.2f", flBugsPerSec);

    szOutput.Format(",%ld,%ld,%ld,%ld,%ld,", 
    				m_cSeconds, 
    				m_cBugsCreated, 
    				m_cBugsUpdated, 
    				m_cBugsCreated + m_cBugsUpdated, 
    				m_cBugsRead);
	szOutput = m_szLastUpdated + szOutput + m_szBugsPerSec + "\r\n";
    
	// Write entry to log file
    hOutputFile = CreateFile(m_szOutputFile,
                             GENERIC_READ | GENERIC_WRITE,
                             0,
                             &sa,
                             OPEN_ALWAYS,
                             FILE_ATTRIBUTE_NORMAL,
                             NULL);
    
    if (hOutputFile == INVALID_HANDLE_VALUE)
    {
        dwError = GetLastError();
        AfxFormatString1(m_szError, IDS_ERROR_CREATEFILE, m_szOutputFile);
        goto FileCleanup;
    }

	// Seek to end of file
    dwFile = SetFilePointer(hOutputFile, 0, NULL, FILE_END);
    if (dwFile == 0xFFFFFFFF)
    {
        dwError = GetLastError();
        AfxFormatString1(m_szError, IDS_ERROR_SETFILEPOINTER, m_szOutputFile);
        goto FileCleanup;
    }
	
    fRet = WriteFile(hOutputFile, 
                     szOutput.GetBuffer(0), 
                     szOutput.GetLength(), 
                     &dwBytesWritten, 
                     NULL);
    if (!fRet)
    {
        dwError = GetLastError();
        AfxFormatString1(m_szError, IDS_ERROR_WRITEFILE, m_szOutputFile);
        goto FileCleanup;
    }

	if (!m_szError.IsEmpty())
	{
		CString szHr;

		szHr.Format("0x%08x", hr);
		szOutput = m_szError + "  " + szHr + "\r\n";
		fRet = WriteFile(hOutputFile, 
						 szOutput.GetBuffer(0), 
						 szOutput.GetLength(), 
						 &dwBytesWritten, 
						 NULL);
		if (!fRet)
		{
			dwError = GetLastError();
			AfxFormatString1(m_szError, IDS_ERROR_WRITEFILE, m_szOutputFile);
			goto FileCleanup;
		}
	}
FileCleanup:
    if (hOutputFile != INVALID_HANDLE_VALUE)
    {
        fRet = CloseHandle(hOutputFile);
        if (!fRet)
        {
            dwError = GetLastError();
            AfxFormatString1(m_szError, IDS_ERROR_CLOSEHANDLE, m_szOutputFile);
        }
    }

    // Save new timestamp only if Sync succeeded

	if (hr == S_OK)
	{
	    iRet = memcmp(&(cbaNewMaxTimestamp[0]),
	    			  &(m_cbaMaxTimestamp[0]),
	                  m_cbaMaxTimestamp.GetSize());
	    if (iRet > 0)
		{
	        memcpy(&(m_cbaMaxTimestamp[0]), &(cbaNewMaxTimestamp[0]), cbaNewMaxTimestamp.GetSize());
		}
	}
    
    // Clean up MAPI objects
    if (lpRows)
    {
        if (lpRows->aRow[0].lpProps)
        {
            MAPIFreeBuffer(lpRows->aRow[0].lpProps);
        }
        MAPIFreeBuffer(lpRows);
    }
                           
    if (pm)
        pm->Release();
    if (ptableContents)
        ptableContents->Release();
    if (pmfExRaid)
        pmfExRaid->Release();
    if (pmdb)
        pmdb->Release();
    if (pms)
    {
        hr = pms->Logoff(NULL, 0, 0);
        if (hr != S_OK)
        {
            m_szError.LoadString(IDS_ERROR_LOGOFF);
        }
        pms->Release();
    }
        
	if (m_szError.IsEmpty())
		m_szError = "Update Successful.";

	// IDC_LASTUPDATED will update all display variables.
    UpdateAllViews(NULL, IDC_LASTUPDATED, (CObject *) this);
}

BOOL FTimeTToFileTime(time_t tt, LPFILETIME pft)
{
    struct tm * ptm;
    BOOL fRet;
    SYSTEMTIME st;
	DWORD dwError;
	CString szError;

	if (tt == 0)
	{
		pft->dwLowDateTime = 0;
		pft->dwHighDateTime = 0;
		return TRUE;
	}

	ptm = gmtime(&tt);
    st.wYear = 1900 + ptm->tm_year;
    st.wMonth = ptm->tm_mon + 1;
    st.wDayOfWeek = ptm->tm_wday;
    st.wDay = ptm->tm_mday;
    st.wHour = ptm->tm_hour;
	st.wMinute = ptm->tm_min;
    st.wSecond = ptm->tm_sec;
    st.wMilliseconds = 0;
    fRet = SystemTimeToFileTime(&st, pft);
    if (!fRet)
    {
        dwError = GetLastError();
        szError.LoadString(IDS_ERROR_SYSTEMTIMETOFILETIME);
        AfxMessageBox(szError, MB_ICONEXCLAMATION);
        return FALSE;
    }
    return TRUE;
}


void CExRaidDoc::OnSyncronizeNow() 
{
	OnSyncronize(FALSE);
}

void CExRaidDoc::OnSyncronizeAll() 
{
	OnSyncronize(TRUE);	
}

void CExRaidDoc::OnSyncronizeRunning() 
{
	SyncronizeRunning(FALSE);
}

void CExRaidDoc::SyncronizeRunning(BOOL fStartUp)
{
	POSITION posView;
	CView * pCView;
	BOOL 	fRet;
	UINT 	uiRet;
	UINT    nMilliSec;
	CString szError;

	posView = GetFirstViewPosition();
	ASSERT(posView);
	pCView = GetNextView(posView);
	ASSERT(pCView);
	
	// If Current status is off, then turn menu item on
	if (!fStartUp)
		m_fRunning = !m_fRunning;
	
	// If menu item is on, then create and start timer
	// If menu item is off, then kill current timer if exists.	
	if (m_fRunning)
	{
		switch (m_stUnits)
		{
			case stHours:
				// 3,600,000 ms / hr
				nMilliSec = m_cSyncTime * 3600000;
				break;
			case stMinutes:
				// 60,000 ms / min
				// nMilliSec = m_cSyncTime * 1000;  // Change to seconds for debugging.
				nMilliSec = m_cSyncTime * 60000;
				break;
			default:
				ASSERT(TRUE);
				break;
		}

		// Sync files now
		if (!fStartUp)
			OnSyncronize(FALSE);

		// Create the timer
		uiRet = pCView->SetTimer(idSyncTimer, nMilliSec, NULL);
		if (!uiRet)
		{
	        szError.LoadString(IDS_ERROR_SETTIMER);
    	    AfxMessageBox(szError, MB_ICONEXCLAMATION);
			m_fRunning = FALSE;
			goto Cleanup;
		}
	}
	else
	{
		fRet = pCView->KillTimer(idSyncTimer);
		if (!fRet)
		{
	        szError.LoadString(IDS_ERROR_KILLTIMER);
    	    AfxMessageBox(szError, MB_ICONEXCLAMATION);
			m_fRunning = TRUE;
			goto Cleanup;
		}
	}
Cleanup:
	if (!fStartUp)
		UpdateAllViews(NULL, IDC_SYNCRUNNING, (CObject *) this);	
}

void CExRaidDoc::OnUpdateSyncronizeRunning(CCmdUI* pCmdUI) 
{
	if (m_fRunning)
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

