/******************************************************************************
Module name: SprdSht.cpp
Written by:  Jeffrey Richter
Purpose:     Demonstrates virtual memory and SEH
NOTE:        Error checking has omitted for demonstration purposes only.
******************************************************************************/


#define STRICT
#include <Windows.h>


/////////////////////////////////////////////////////////////////////////////


// Declare the contents of a single cell in the spreadsheet
typedef struct { BYTE bDummy[100]; } CELL;

// Declare the data type for an entire spreadsheet
typedef CELL SPREADSHEET[256][1024];

// Create a global pointer that points to the entire spreadsheet
SPREADSHEET *g_pss;


// Forward refernce to exception filter function
LONG Filter (LPEXCEPTION_POINTERS pep, int nRow, int nCol);


/////////////////////////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinst, HINSTANCE, LPSTR pszCmdLine, int nCmdShow) {

   // When process initializes, reserve an address space 
   // region large enough for the whole spreadsheet
   g_pss = (SPREADSHEET*) VirtualAlloc(NULL, 
      sizeof(*g_pss), MEM_RESERVE | MEM_TOP_DOWN, PAGE_READWRITE);


   // Write code assuming that all storage is committed
   // NOTE: All accesses to the spreadsheet must be wrapped in SEH block

   int nRow = 10, nCol = 100;

   __try {
      CELL cellUserData;   // Cell info enterred by user

      // Write the users cell data into the spreadsheet
      (*g_pss)[nRow][nCol] = cellUserData;
   }
   __except (Filter(GetExceptionInformation(), nRow, nCol)) {
      // Nothing to do in here
   }
   return(0);
}


/////////////////////////////////////////////////////////////////////////////


// This is the eception filter that commits storage to portions of the
// spreadsheet when absolutely necessary.
LONG Filter (LPEXCEPTION_POINTERS pep, int nRow, int nCol) {

   // Make sure the exception is for us
   if (pep->ExceptionRecord->ExceptionCode != EXCEPTION_ACCESS_VIOLATION)
      return(EXCEPTION_CONTINUE_SEARCH);

   // Get the address of the attempted memory access
   PVOID pvAttemptedAccess = (PVOID) pep->ExceptionRecord->ExceptionInformation[1];

   // If the address is below the spreadsheet's region, continue search
   if (pvAttemptedAccess < g_pss)
      return(EXCEPTION_CONTINUE_SEARCH);

   // If the address is above the spreadsheet's region, continue search
   if (pvAttemptedAccess >  ((PBYTE) g_pss + sizeof(*g_pss) - 1))
      return(EXCEPTION_CONTINUE_SEARCH);

   // Sparsely commit storage to the region
   VirtualAlloc(&(*g_pss)[nRow][nCol], sizeof(CELL), MEM_COMMIT, PAGE_READWRITE);
   return(EXCEPTION_CONTINUE_EXECUTION);
}


//////////////////////////////// End of File //////////////////////////////////
