/******************************************************************************
Module name: TLS.cpp
Written by:  Jeffrey Richter
Purpose:     Demonstrates the use of thread local storage.
NOTE:        These code fragments have error checking omitted 
             for demonstration purposes only.
******************************************************************************/


DWORD g_dwTlsIndex = TLS_OUT_OF_INDEXES;
typedef struct { BYTE b; } MYDATA, *PMYDATA;


/////////////////////////////////////////////////////////////////////////////


BOOL WINAPI DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {

   switch (fdwReason) {
      case DLL_PROCESS_ATTACH:

         // Allocate a TLS slot
         g_dwTlsIndex = TlsAlloc();
         break;

      case DLL_THREAD_ATTACH:
         break;

	   case DLL_THREAD_DETACH:

         // Free the TLS memory block associated with the thread
         // NOTE: It's OK to pass NULL to free
         free(TlsGetValue(g_dwTlsIndex));
		   break;

	   case DLL_PROCESS_DETACH:

         // Free the TLS memory block associated with the thread
         // NOTE: It's OK to pass NULL to free
         free(TlsGetValue(g_dwTlsIndex));

         // Free the TLS slot
         TlsFree(g_dwTlsIndex);
         break;
   }

   return(TRUE);
}


/////////////////////////////////////////////////////////////////////////////


void MyFunc (PMYDATA pMyData) {

   // Are we priming the function?
   if (pMyData != NULL) {

      // Did we ever allocate a TLS memory block?
      if (TlsGetValue(g_dwTlsIndex) == NULL) {

         TlsSetValue(g_dwTlsIndex, malloc(sizeof(*pMyData)));
      }

      // Save the prime data in the TLS memory block
      CopyMemory(TlsGetValue(g_dwTlsIndex), pMyData, sizeof(*pMyData));

      // Get a pointer to the primed data in the TLS memory block
      pMyData = (PMYDATA) TlsGetValue(g_dwTlsIndex);
   }

   // Use the TLS data (pointed to by pMyData)
   .
   .
   .
}


//////////////////////////////// End of File //////////////////////////////////
