/******************************************************************************
Module name: MMF.cpp
Written by:  Jeffrey Richter
Purpose:     Demonstrates sharing data across process boundaries with MMFs.
NOTE:        Error checking has omitted for demonstration purposes only.
******************************************************************************/


#define STRICT
#include <Windows.h>


/////////////////////////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinst, HINSTANCE, LPSTR pszCmdLine, int nCmdShow) {

	// Creates a named memory-mapped file backed by the paging file
	HANDLE hfileMap = CreateFileMapping(
      (HANDLE) 0xffffffff,       // Use paging file for storage
      NULL,                      // Default security
      PAGE_READWRITE,            // Access
      0, strlen(pszCmdLine) + 1, // Amount of storage to commit
      "MyFileMapping");          // Name of MMF for sharing

   // Determine if we created a ew MMF or opened an existing one.
   BOOL fCreatedNewMMF = (GetLastError() != ERROR_ALREADY_EXISTS);

	// Map the MMF into our address space
	LPSTR pszData = (LPSTR) MapViewOfFile(
      hfileMap,                  // Handle of MMF
      FILE_MAP_ALL_ACCESS,       // Access
      0, 0,                      // Start with the 1st byte
      0);                        // Map the whole file

   if (fCreatedNewMMF) {
      strcpy(pszData, pszCmdLine);
   }

	// Display a message box with the data in the MMF
	MessageBox(NULL, pszData, "Data in MMF", MB_OK);

	// Remove the MMF's view from our address space
	UnmapViewOfFile(pszData);

   // Close the MMF's handle
	CloseHandle(hfileMap);

 	return(0);
}


//////////////////////////////// End of File //////////////////////////////////
