// 
// EXAMPLE 1:  OLE DS in an Internet Server Application
//

//
// ISA Entry Point - Return the ISA Version
//

BOOL WINAPI 
GetExtensionVersion( HSE_VERSION_INFO  *pVer )
{
    pVer->dwExtensionVersion = MAKELONG(HSE_VERSION_MINOR,
                                       HSE_VERSION_MAJOR);
    lstrcpyn( pVer->lpszExtensionDesc,
              "OLE DS Administrator",
               HSE_MAX_EXT_DLL_NAME_LEN );
    return TRUE;
}

//
// ISA Entry Point - Called for Client Requests
//

// TLS is used to store OLE initialization flag

static DWORD g_dwTLSIndex = (DWORD)-1;

DWORD WINAPI HttpExtensionProc(EXTENSION_CONTROL_BLOCK *pECB)
{
	BSTR	bstrNewUser;
	BSTR	bstrContainer;
	BSTR	bstrPassword;

	IOleDsContainer* pContainer;
	IOleDsUser*	 pUser;
	IOleDsFSAccountRestrictions* pAcct;

// Init OLE if Not already done

    if (FALSE == TlsGetValue(g_dwTLSIndex)) {
      OleInitialize(NULL);
      TlsSetValue(g_dwTLSIndex, (void*)TRUE);
    }

// Error handling, 
// Command Parsing Omitted for Brevity.
// Assume bstrUser contains new user ID at this point

// Get the Container to place the user in

    OleDsGetObject(bstrContainer,
                   IID_IOleDsContainer,
                   &pContainer);
// Create the user
    pContainer->Create(TEXT("user"),
                       bstrNewUser),
                       &pUser);
// Write the user to the DS
    pUser->SetInfo();
// Set the user's password
    pUser->AccountRestrictions(&pAcct);
    pAcct->SetPassword(bstrPassword);

    pAcct->Release();
    pUser->Release();
    pContainer->Release();
//
// This app Should write something back to the client
// here indicating details of operation just completed,  // using WriteClient(...).

// DONE!        
    return HSE_STATUS_SUCCESS;

}

===================================================

'
' OLE DS - Example 2
'
' Binding to a DS Object

 Visual Basic
Dim MyUser as IOleDs
Set MyUser = GetObject(WinNT://MSFT/Margretj)

// C/C++
IOleDs *pMyUser;
OleDsGetObject(	TEXTWinNT://MSFT/Margretj,
			IID_IOleDs,
			&pMyUser);

====================================================

'
' OLE DS - Example 3
'
' Simple manipulation of DS Objects
'
Dim MyUser as IOleDsUser
Set MyUser = GetObject(WinNT://MSFT/Margretj)
 Print the FullName property
Debug.Print MyUser.BusinessInfo.FullName
 Set the FullName property
MyUser.BusinessInfo.FullName = Margaret K. Johnson
MyUser.SetInfo
 Refresh the cache, and print FullName again
MyUser.GetInfo
Debug.Print MyUser.BusinessInfo.FullName

==================================================

'
' OLE DS Example 4
'
' Schema Management - Derive a new Class from
'                     an existing class
'
dim schema as IOleDsContainer
dim PqClass as IOleDsClass  class to derive from
dim FqClass as IOleDsClass  derived class to create

 bind to the schema
set schema = GetOBject(NS://Root/SCHEMA)
set PqClass = GetObject(NS://Root/SCHEMA/PrintQueue)

 create a new class in the schema container
set FqClass = schema.create(class,FaxQueue)

 set the name , CLSID, and Primary Interface 
' of the new class
FqClass.Name = FaxQueue
FqClass.CLSID = PqClass.CLSID
FqClass.PrimaryInterface = PqClass.PrimaryInterface

 indicate the class derived from
FqClass.DerivedFrom = PqClass.OleDsPath 

 write the new class into the schema
FqClass.setinfo
' Done!

==================================================

'
' OLE DS Example 5
'
' Schema Management - Add a Property to an Existing
'                     Class (the one we defined in
'                            Example 2) 
Dim Schema as IOleDsContainer
Dim FQClass as IOleDsContainer
Dim FSFaxClass as IOleDsFunctionalSet
Dim FaxClass as IOleDsProperty
Dim FSAlias as IOleDsFunctionalSetAlias

' Bind to the Schema container
set Schema = GetOBject(NS://Root/SCHEMA)
set FQClass = GetObject(NS://Root/SCHEMA/FaxQueue)

create the new functional set
' (Give it a name that will avoid collisions)
set FSFaxClass 
	= Schema.create(FunctionalSet,MSFsFax)

set the new FS properties
FSFaxClass.CLSID = {string GUID of CLSID}
FSFaxCLass.PrimaryInterface 
	= {String IID of primary interface}

Write the new Functional Set to the Schema
FSFaxClass.SetInfo

Add a property to the functional set
set FaxClass = 
	FSFaxClass.Create("property",FaxClass)
FaxClass.Syntax = Integer
FaxClass.MinRange = 1
FaxClass.MaxRange = 3

this is an optional property
FaxClass.Optional = True

' Update the DS with the new property
FaxClass.SetInfo

Now add a new FS Alias to the FaxQueue object
Set FSAlias = FaxQueue.Create(FunctionalSetALias,FSFaxClass)
FSAlias.AliasedObject = FSFaxClass.OleDsPath
FSAlias.SetInfo
Done !

===================================================

'
' OLE DS Example 6: Adding users to groups
'                   Adds users in a given OU to
'                   a group
 
dim MyUserContainer as IOleDsContainer
dim MyUser as IOleDsUser
dim MyGroup as IOleDsGroup
dim Filter as Variant

 filter out all objects except users
Filter = Array("user");

set MyUserContainer = 
	GetOBject("NS://ABC/Manufacturing/Austin")

' set the filter
MyContainer.Filter = Filter  
set MyGroup = 
	GetObject("NS://ABC/Manufacturing_Users")

for each MyUser in MyUserContainer
    if not MyGroup.GeneralInfo.IsMember(MyUser) then
	MyGroup.GeneralInfo.Members.Add(MyUser)
    end if
next MyUser

===================================================

' OLE DS Example 7: 
' Adding users to groups - extended version
'         Adds users in ALL OUs to a group
'

dim MyUserContainer as IOleDsContainer
dim MyOuContainer as IOleDsContainer
dim MyUser as IOleDsUser
dim MyGroup as IOleDsGroup
dim Filter as Variant

Filter = Array("ou")

set MyOuContainer = 
    GetOBject("NS://ABC/Manufacturing")

MyOuContainer.Filter = Filter

Filter = Array("user")

for each MyUserContainer in MyOuContainer
    MyUserContainer.Filter = Filter
    for each MyUser in MyUserContainer
        if not MyGroup.GeneralInfo.IsMember(MyUser) then
	    MyGroup.GeneralInfo.Members.Add(MyUser)
	end if
    next MyUser
next MyUserContainer

