/***************************************
* CDBLibrary class source code         *
*                                      *
* DBLIBXX.CPP                          *
*                                      *
* Created by Tom Dowdell,  Sept 1995   *
*                                      *
***************************************/

#include "dblibxx.h"

CDBLibrary * CDBLibrary::TempThis = NULL;

/***************************************
* DLLEntryPoint                        *
***************************************/
extern "C" DLLEXPORT int APIENTRY DllEntryPoint (HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	return TRUE;
};

/***************************************
* CDBLibrary::CDBLibrary               *
***************************************/
CDBLibrary::CDBLibrary()
{
	// Allocate a LOGINREC
	pLoginRec = dblogin();
	
	// Initialize our DBPROCESS
	pDBProc = NULL;              
};

/***************************************
* CDBLibrary::~CDBLibrary              *
***************************************/
CDBLibrary::~CDBLibrary()
{
	// If the connection is open, close it
	if (pDBProc)
		Close();

	// If a LOGINREC was allocated, free it
	if (pLoginRec)
		dbfreelogin(pLoginRec);
};

/***************************************
* CDBLibrary::SetLogin                 *
***************************************/
BOOL CDBLibrary::SetLogin(LPCTSTR AppName, LPCTSTR HostName, LPCTSTR NatLang)
{
	// Make sure a LOGINREC was allocated
	if (!pLoginRec)
		return FALSE;

	// Call the appropriate DBSET
	// macros to prepare the LOGINREC

	if (AppName)
		DBSETLAPP(pLoginRec,AppName);

	if (HostName)
		DBSETLAPP(pLoginRec,HostName);

	if (NatLang)
		DBSETLAPP(pLoginRec,NatLang);

	return TRUE;
};

/***************************************
* CDBLibrary::Open                     *
***************************************/
BOOL CDBLibrary::Open(LPCTSTR ServerName, LPCTSTR UserName, LPCTSTR Password, LPCTSTR Database, DWORD TimeOut, DWORD PacketSize)
{
DBPROCINFO ProcInfo;

	// Set our static error and
	// message handlers
	dbprocerrhandle(pLoginRec,(DBERRHANDLE_PROC)err_handler);
	dbprocmsghandle(pLoginRec,(DBMSGHANDLE_PROC)msg_handler);

	// temporarily save the this*
	// until dbopen succeeds	
	TempThis = this;

	// If no user name is specified use
	// integrated security
	if (UserName == NULL)
		DBSETLSECURE(pLoginRec);
	else
		DBSETLUSER(pLoginRec, UserName);

	// Set the password
	if (Password == NULL)
		DBSETLPWD(pLoginRec,"");
	else
		DBSETLPWD(pLoginRec,Password);

	// Not necessary, but request
	// SQL Server 6.0 functionality
	DBSETLVERSION(pLoginRec, DBVER60);

	// Set the packet size
	if (PacketSize != INVALID_DWORD)
		DBSETLPACKET(pLoginRec,(USHORT)PacketSize);

	// Set the login timeout
	if (TimeOut != INVALID_DWORD)
		DBSETLTIME(pLoginRec,TimeOut);

	// Open our connection. Abort on fail.
   if ((pDBProc = dbopen(pLoginRec, ServerName)) == (PDBPROCESS)NULL)
		goto Abort;	

	// Store the "this" pointer for
	// usage in err_handler and msg_handler
	dbsetuserdata(pDBProc, (BYTE *)this);
	
	// Disable this since we now have a call
	// the dbsetuserdata to store the this*
	// for the handlers
	TempThis = NULL;

	// Use the requested database
	if (Database && *Database && FAIL == dbuse(pDBProc,Database))
		goto Abort;

	// Call dbprocinfo for information
	// about our connection
	ProcInfo.SizeOfStruct = sizeof(DBPROCINFO);
	if (FAIL == dbprocinfo(pDBProc,&ProcInfo))
		goto Abort;

	// Store dbprocinfo results in
	// our class variables
	this->ServerType = ProcInfo.ServerType;
	this->ServerMajor = ProcInfo.ServerMajor;
	this->ServerMinor = ProcInfo.ServerMinor;
	this->ServerRevision = ProcInfo.ServerRevision;
	lstrcpy(this->ServerName,ProcInfo.ServerName);
	lstrcpy(this->NetLibName,ProcInfo.NetLibName);
	lstrcpy(this->NetLibConnStr,ProcInfo.NetLibConnStr);

	return TRUE;

Abort:  // Clean up in the event of an error

	// Close the connection if open
	if (pDBProc)
		dbclose(pDBProc);

	return FALSE;

};

/***************************************
* Close                                *
***************************************/
BOOL CDBLibrary::Close()
{
	// Abort if a connection was
	// never established
	if (!pDBProc)
		return FALSE;

	// close the connection
	if (FAIL == dbclose(pDBProc))
		return FALSE;

	// Reset our DBPROCESS
	pDBProc = NULL;

	return TRUE;
};

/***************************************
* Dead                                 *
***************************************/
BOOL CDBLibrary::Dead()
{
	// Return TRUE if a connection was
	// never established
	if (NULL == pDBProc)
		return TRUE;
	
	// Return the result of dbdead call
	return dbdead(pDBProc);
};

/***************************************
* CDBLibrary::ErrHandler               *
***************************************/
int CALLBACK CDBLibrary::err_handler(PDBPROCESS dbproc, INT severity, INT dberr, INT oserr, LPCSTR dberrstr, LPCSTR oserrstr)
{
CDBLibrary * p;

	// If DB-Library 6.5 adds "procless"
	// dbsetuserdata then this
	// check will not be necessary.
	if (TempThis)
		// Use our temporary this *
		p = TempThis;
	else
		// Obtain the this* and typecast
		// it to a CDBLibrary*
		p = (CDBLibrary *)dbgetuserdata(dbproc);

	if (!p)
		return INT_CANCEL;

	// Set our internal public variables
	p->Severity = severity;
	p->DBErr = dberr;
	p->OSErr = oserr;
	lstrcpy(p->DBErrStr,dberrstr);
	lstrcpy(p->OSErrStr,oserrstr);

	// Call the dynamic error handler
	return p->ErrHandler(severity, dberr, oserr, dberrstr, oserrstr);
};

/***************************************
* CDBLibrary::MsgHandler               *
***************************************/
int CALLBACK CDBLibrary::msg_handler(PDBPROCESS dbproc, DBINT msgno, INT msgstate, INT severity, LPCSTR msgtext)
{
CDBLibrary * p;

	// If DB-Library 6.5 adds "procless"
	// dbsetuserdata then this
	// check will not be necessary
	if (TempThis)
		// Use our temporary this *
		p = TempThis;
	else
		// Obtain the this* and typecast
		// it to a CDBLibrary*
 		p = (CDBLibrary *)dbgetuserdata(dbproc);

	if (!p)
		return 0;

	// Set our internal public variables
	p->MsgNo = msgno;
	p->MsgState = msgstate;
	p->MsgSeverity = severity;
	lstrcpy(p->MsgText,msgtext);

	// Call the dynamic message handler
	return p->MsgHandler(msgno, msgstate, severity, msgtext);
};

/***************************************
* CDBLibrary::ErrHandler               *
***************************************/
int CDBLibrary::ErrHandler(INT severity, INT dberr, INT oserr, LPCSTR dberrstr, LPCSTR oserrstr)
{
	// This is a virtual member function stub.  By default, just return INT_CONTINE
	return INT_CONTINUE;
};

/***************************************
* CDBLibrary::MsgHandler               *
***************************************/
int CDBLibrary::MsgHandler(DBINT msgno, INT msgstate, INT severity, LPCSTR msgtext)
{
	// This is a virtual member function
	// stub.  By default, just return 0
	return 0;
};

/***************************************
* Command                              *
***************************************/
BOOL CDBLibrary::Command(LPCTSTR CommandStr)
{
	// Abort if a connection was never established
	if (NULL == pDBProc)
		return FALSE;

	// Issue the command
	if (FAIL == dbcmd(pDBProc,CommandStr))
		return FALSE;

	return TRUE;
};

/***************************************
* Execute                              *
***************************************/
BOOL CDBLibrary::Execute(LPCTSTR CommandStr)
{
	// Abort if a connection was never established
	if (NULL == pDBProc)
		return FALSE;

	// If a command string was passed in the call Command
	if (CommandStr && !Command(CommandStr))
		return FALSE;

	// Execute the command(s)
	if (FAIL == dbsqlexec(pDBProc))
		return FALSE;

	return TRUE;
};

/***************************************
* Results                              *
***************************************/
RETCODE CDBLibrary::Results()
{
	// Check for results
	return dbresults(pDBProc);
};

/***************************************
* DataLen                              *
***************************************/
DBINT CDBLibrary::DataLen(COLNUM ColNum)
{
	// return the data length
	return dbdatlen(pDBProc,(INT)ColNum);
};

/***************************************
* Data                                 *
***************************************/
LPVOID CDBLibrary::Data(COLNUM ColNum)
{
	// return a * to the data
	return (LPVOID)dbdata(pDBProc,(INT)ColNum);
};

/***************************************
* Data (String)                        *
***************************************/
BOOL CDBLibrary::Data(COLNUM ColNum, LPTSTR String)
{
LPTSTR p;
DBINT StringLen;

	// Get a pointer to the data and typecast to an LPTSTR
	if (NULL == (p = (LPTSTR)Data(ColNum)))
		return FALSE;

	// Copy and null terminate the string
	StringLen = DataLen(ColNum);
	memcpy((LPVOID)String, (LPVOID)p, StringLen);
	String[StringLen] = 0;

	return TRUE;
};

/***************************************
* Data (LONG)                          *
***************************************/
BOOL CDBLibrary::Data(COLNUM ColNum, LONG * Long)
{
LPLONG p;

	// Get a pointer to the data and typecast to an LONG *
	if (NULL == (p = (LPLONG)Data(ColNum)))
		return FALSE;

	// Store the LONG
	*Long = *p;

	return TRUE;
};

/***************************************
* Data (SHORT)                         *
***************************************/
BOOL CDBLibrary::Data(COLNUM ColNum, SHORT * Short)
{
SHORT * p;

	// Get a pointer to the data and typecast to an SHORT *
	if (NULL == (p = (SHORT *)Data(ColNum)))
		return FALSE;

	// Store the SHORT
	*Short = *p;

	return TRUE;
};

/***************************************
* Data (DOUBLE)                        *
***************************************/
BOOL CDBLibrary::Data(COLNUM ColNum, DOUBLE * Double)
{
DOUBLE * p;

	// Get a pointer to the data and typecast to an DOUBLE *
	if (NULL == (p = (DOUBLE *)Data(ColNum)))
		return FALSE;

	// Store the DOUBLE
	*Double = *p;

	return TRUE;
};

/***************************************
* Data (FLOAT)                         *
***************************************/
BOOL CDBLibrary::Data(COLNUM ColNum, FLOAT * Float)
{
FLOAT * p;

	// Get a pointer to the data and typecast to an FLOAT *
	if (NULL == (p = (FLOAT *)Data(ColNum)))
		return FALSE;

	// Store the FLOAT
	*Float = *p;

	return TRUE;
};

/***************************************
* NextRow                              *
***************************************/
RETCODE CDBLibrary::NextRow()
{
	// return the next row
	return dbnextrow(pDBProc);
};

/***************************************
* Cancel                               *
***************************************/
BOOL CDBLibrary::Cancel()
{
	// Cancel the operation
	return (dbcancel(pDBProc) == FAIL) ? FALSE: TRUE;
};

/***************************************
* CancelQuery                          *
***************************************/
BOOL CDBLibrary::CancelQuery()
{
	// Cancel the query
	return (dbcanquery(pDBProc) == FAIL) ? FALSE: TRUE;
};

/***************************************
* ColInfo                              *
***************************************/
BOOL CDBLibrary::ColInfo(SSFLAG Type, COLNUM ColNum, COLNUM AltColNum, LPDBCOL pDBCol)
{
	// return column information
	if (FAIL == dbcolinfo(pDBProc,Type,(INT)ColNum,(INT)AltColNum,pDBCol))
		return FALSE;

	return TRUE;
};

/***************************************
* NumCols                              *
***************************************/
COLNUM CDBLibrary::NumCols()
{
	// return the number of result columns
	return (ROWNUM)dbnumcols(pDBProc);
};
