#ifndef _H_DBLIBXX
#define _H_DBLIBXX

/***************************************
* CDBLibrary class include file        *
*                                      *
* DBLIBXX.H                            *
*                                      *
* Created by Tom Dowdell,  Sept 1995   *
*                                      *
***************************************/

#include "windows.h"

#define DBNTWIN32
#include "sqlfront.h"
#include "sqldb.h"


/***************************************
* #defines                             *
***************************************/

// Have a value we can substitute
// as a default parameter
#define INVALID_DWORD 0xFFFFFFFF

// In order to migrate away from having
// fixed datatypes throughout the code,
// abstract them into our own types.
#define SSVARLEN  DWORD
#define SSFLAG    DWORD
#define COLNUM    DWORD
#define ROWNUM    DWORD


// DLLEXPORT is a macro used to
// mark a class as exported from a DLL
#define DLLEXPORT __declspec(dllexport)


// Maximum string length of
// a message or error (only a guess)
#define MAX_ERROR_STR 500


/***************************************
* CDBLibrary                           *
***************************************/
class DLLEXPORT CDBLibrary
{
public:

	// CDBLibrary constructor and
	// destructor members
	CDBLibrary();
	~CDBLibrary();

	// Miscellaneous members
	operator PDBPROCESS () CONST { return pDBProc; };

	// I/O members
	BOOL      SetLogin      (LPCTSTR AppName, LPCTSTR HostName = NULL, LPCTSTR NatLang = NULL);
	BOOL      Open          (LPCTSTR ServerName = NULL, LPCTSTR UserName = NULL, LPCTSTR Password = NULL, LPCTSTR Database = NULL, DWORD TimeOut = INVALID_DWORD, DWORD PacketSize = INVALID_DWORD);
	BOOL      Close         ();
	BOOL      Dead          ();
	
	// Error members
	virtual int ErrHandler(INT severity, INT dberr, INT oserr, LPCSTR dberrstr, LPCSTR oserrstr);
	virtual int MsgHandler(DBINT msgno, INT msgstate, INT severity, LPCSTR msgtext);

	// Query Processing
	BOOL      Command(LPCTSTR CommandStr);
	BOOL      Execute(LPCTSTR CommandStr = NULL);
	RETCODE   Results();
	DBINT     DataLen(COLNUM ColNum);
	LPVOID    Data(COLNUM ColNum);
	BOOL      Data(COLNUM ColNum, LPTSTR   StringPtr);
	BOOL      Data(COLNUM ColNum, LONG   * LongPtr);
	BOOL      Data(COLNUM ColNum, SHORT  * ShortPtr);
	BOOL      Data(COLNUM ColNum, DOUBLE * DoublePtr);
	BOOL      Data(COLNUM ColNum, FLOAT  * FloatPtr);
	BOOL      Cancel();
	BOOL	    CancelQuery();
	RETCODE   NextRow();
	BOOL      ColInfo(SSFLAG Type, COLNUM ColNum, COLNUM AltColNum, LPDBCOL pDBCol);
	COLNUM    NumCols();

	// Values set by a call to dbgetprocinfo
	// at connect time
	SSFLAG  ServerType;
	DWORD   ServerMajor;
	DWORD   ServerMinor;
	DWORD   ServerRevision;
	CHAR ServerName[MAXSERVERNAME+1];
	CHAR NetLibName[MAXNETLIBNAME+1];
	CHAR NetLibConnStr[MAXNETLIBCONNSTR+1];


	// Values set when an error occurs
	INT Severity;
	INT DBErr;
	INT OSErr;
	CHAR DBErrStr[MAX_ERROR_STR];
	CHAR OSErrStr[MAX_ERROR_STR];
	
	// Values set when a message occurs
	DBINT MsgNo;
	INT MsgState;
	INT MsgSeverity;
	CHAR MsgText[MAX_ERROR_STR];


private:

	// static callback functions invoked by
	// DB-Library that call the per-process
	// ErrHandler and MsgHandler, respectively.
	static int CALLBACK err_handler(PDBPROCESS dbproc, INT severity, INT dberr, INT oserr, LPCSTR dberrstr, LPCSTR oserrstr);
	static int CALLBACK msg_handler(PDBPROCESS dbproc, DBINT msgno, INT msgstate, INT severity, LPCSTR msgtext);
	static CDBLibrary * TempThis;

	PLOGINREC pLoginRec;
	PDBPROCESS pDBProc;

};

#endif
