/***************************************
* TESTDBXX Sample Application          *
*                                      *
* TESTDBXX.CPP                         *
*                                      *
* Created by Tom Dowdell,  Sept 1995   *
*                                      *
***************************************/

#include "dblibxx.h"
#include "stdio.h"

/***************************************
* #defines                             *
***************************************/

// Buffer lengths
#define AUID_LEN        11
#define AULNAME_LEN     40
#define AUFNAME_LEN     20

// Login timeout in seconds
#define LOGIN_TIMEOUT   15

// Total result columns we expect back
#define TOT_RESULT_COLS  3


/***************************************
* CMyDBLibrary                         *
***************************************/

// We're going to derive CMyDBLibrary
// from CDBLibrary class so we can
// create our own error and message
// handlers.

class CMyDBLibrary: public CDBLibrary
{

public:

	// Override the default handlers in
	// our derived class

	virtual int ErrHandler(INT severity, INT dberr, INT oserr, LPCSTR dberrstr, LPCSTR oserrstr)
	{
		printf("ERROR HANDLER INVOKED\nSeverity: %d  Error: %d %s\n",severity, dberr, dberrstr);
		return INT_CANCEL;
	};

	virtual int MsgHandler(DBINT msgno, INT msgstate, INT severity, LPCSTR msgtext)
	{
		printf("MESSAGE HANDLER INVOKED\nSeverity: %d  Message: %d %s\n",severity, msgno, msgtext);
		return 0;
	};
};


/***************************************
* main                                 *
***************************************/
void main()
{
CMyDBLibrary DBLibrary;      // Our class variable
RETCODE RetCode;             // Return code for Results and NextRow
CHAR AuID[AUID_LEN+1];       // Author ID buffer
CHAR AuLName[AULNAME_LEN+1]; // Author Last Name buffer
CHAR AuFName[AUFNAME_LEN+1]; // Author First Name buffer
COLNUM TotResultCols;        // Total result columns
COLNUM CurCol;               // Column counter
DBCOL DBCol;                 // Column information structure

	// Open a connection to pubs database.
	// Timeout after 15 seconds
	if (!DBLibrary.Open("","sa","","pubs",LOGIN_TIMEOUT))
		return;

	// Execute a command
	if (!DBLibrary.Execute("select au_id, au_lname, au_fname from authors"))
		return;

	// Process Results
	while (NO_MORE_RESULTS != (RetCode = DBLibrary.Results()))
	{
		// Abort and Cancel for failures
		if (FAIL == RetCode)
		{
			DBLibrary.Cancel();
			return;
		}

		// Determine how many columns where returned
		TotResultCols = DBLibrary.NumCols();
		if (TOT_RESULT_COLS != TotResultCols)
			return;

		// Get Column specific information and print
		DBCol.SizeOfStruct = sizeof(DBCOL);
		for (CurCol = 0; CurCol < TotResultCols; CurCol++)
		{
			if (!DBLibrary.ColInfo(CI_REGULAR,CurCol+1,0,&DBCol))
			{
				DBLibrary.Cancel();
				return;
			}

			printf("\nColumn information\n");
			printf(
				"Col#: %d  Name: %s  Variable Length: %s\n",
				CurCol+1,
				DBCol.Name,
				(DBCol.VarLength) ? "Yes" : "No"
				);
		};

		printf("\nResult Set\n\n");
				
		// Process the rows
		while (NO_MORE_ROWS != (RetCode = DBLibrary.NextRow()))
		{
			if (FAIL == RetCode)
				DBLibrary.CancelQuery();
				if (!DBLibrary.Data(1,AuID) || !DBLibrary.Data(2,AuLName) || !DBLibrary.Data(3,AuFName))
				{
					DBLibrary.Cancel();
					return;
				};
			printf("ID: %s  Last Name: %-15s  First Name: %-15s\n",AuID,AuLName,AuFName);
		};
	}
}
