/*
******************************************************************	
	FirstSQL.EXE			Stewart MacLeod				2/22/96

	This simple program demonstrates how to create a character
	based "console" application for	Microsoft SQL Server 
	on Windows NT Server or Windows 95.

	Syntax: FirstSQL.EXE  SqlServerName UserName <UserPassword>	  
******************************************************************
*/			
#define DBNTWIN32		// Tells SQL Server Its Windows NT Or Windows 95
#include <windows.h>	// Main Windows Header File
#include <sqlfront.h>	// Every DBLIB program requires 
#include <sqldb.h>		// <sqlfront.h> and <sqldb.h>
#include <stdio.h>		// C Standard Library

int SQLAPI ErrorHandler(DBPROCESS *lpDbProc, int iSeverity,int iDbError, int iOsError, const char *lpszDbErrorMsg, const char *lpszOsErrorMsg);
int SQLAPI MessageHandler(DBPROCESS *lpDbProc, DBINT iMsgNumber, int iMsgState, int iSeverity, const char *lpszMessageText, const char *szServerName, const char *szProcName, unsigned short iLineNumber);
#define SQLSTATEMENT "select au_lname, au_fname from authors order by au_lname"

void main( int argc, char ** argv)
{
	char 		szServerName[MAXNAME+1];	// SQL Server Name
	char 		szDatabaseName[MAXNAME+1];	// DB Name
	char		szUserName[MAXNAME+1];		// User Name
	char 		szUserPassword[MAXNAME+1];	// User Password
	DBPROCESS	*lpDbProc;					// Connection To SQL Server
	LOGINREC   	*login;						// Login Record
	INT			iRetCode;					// Return Code
	STATUS		iRowCode;					// Row Fetch Status Code
	unsigned char	szFirstName[20+1];		// Author First Name
	unsigned char	szLastName[40+1];		// Author Last Name
	/*
	**********************************************************************
	//	Set the Defaults 
	**********************************************************************
	*/							  	
	if( argc < 3 || argc > 4){
		printf("\nSyntax: %s SqlServerName UserName <Password>\n", argv[0]);
		exit(1);
	}
	strcpy( szServerName, argv[1] );		// Get SQL Server Name
	strcpy( szUserName, argv[2] );			// User Name
	strcpy( szDatabaseName, "pubs" );		// User PUBS database

	if(argc == 4)
		strcpy( szUserPassword, argv[3] );	// and Password
	else
		*szUserPassword = '\0';
	/*
	**********************************************************************
		Initialize DBLIBRARY And Install Our Error And Message Handlers	
	**********************************************************************
	*/						   	
	printf("\nVersion: %s\n", dbinit()); 	// This returns a pointer to the version string	
	dberrhandle( ErrorHandler ); 			// Global Error Handler
	dbmsghandle( MessageHandler );			// Global Message Handler
	/*
	**********************************************************************
		Specify The UserName, Password, and Login Timeout
	**********************************************************************
	*/		
    login = dblogin();						// Allocate A Login Record
	dbsetlogintime( 30 );					// Specify Login Timeout (30 sec)
	DBSETLUSER( login, szUserName );		// Specify The User Name
	DBSETLPWD( login, szUserPassword);		// And Their Password 	
	/*
	**********************************************************************
		Connect To The Specified Server By Opening A DBPROCESS
	**********************************************************************
	*/																	
	lpDbProc = dbopen( login, szServerName );// Connect To SQL Server
	if( !lpDbProc ) {						// Unable To Connect?
		printf("\nUnable To Connect To %s\n", szServerName );	
		exit(1);							// Exit With Error
	}	
	/*
	**********************************************************************
		Use The PUBS Database 
	**********************************************************************
	*/	
	if( dbuse( lpDbProc, szDatabaseName ) == FAIL ){ // Unable To Use Database?
		printf("Unable To Use %s Database On %s", szDatabaseName,szServerName );	
		exit(1);							// Exit With Error
	} 	
	/*
	**********************************************************************
		Compile And Execute The SQL Statement	
	**********************************************************************
	*/											 
	iRetCode = dbcmd(lpDbProc, SQLSTATEMENT ); 	// Put SQL Statement In The Buffer		 
	if( iRetCode == FAIL ) {
	 	printf("\nError: Unable To Add SQL Statement To Command Buffer\n");	
		exit(1);							// Exit With Error
	}			
	iRetCode = dbsqlexec(lpDbProc);  		// Send To Server and Execute
	if( iRetCode == FAIL ) {
	 	printf("\nError: Compiling And Executing SQL Statement \n");	
		exit(1);							// Exit With Error
	}		
	/*
	**********************************************************************
		Fetch Any And All Results	
	**********************************************************************
	*/	
	while ((iRetCode = dbresults(lpDbProc)) != NO_MORE_RESULTS)
	 {
		if (iRetCode == FAIL )	break;							   
		/*
		******************************************************************
			Bind The Author First Name And Last Name To Our Variables
		******************************************************************
		*/	
		dbbind( lpDbProc, 1, NTBSTRINGBIND, 0, szLastName );
		dbbind( lpDbProc, 2, NTBSTRINGBIND, 0, szFirstName );

		printf("\n%s\n", SQLSTATEMENT );
		printf("\n%-40s %-20s", "Last Name", "First Name" );
		printf("\n%-40s %-20s", "---------", "----------" );
		/*
		******************************************************************
			Fetch Each Row And Print
		******************************************************************
		*/	
		while ( (iRowCode = dbnextrow(lpDbProc)) != NO_MORE_ROWS)
			{				 				
				if( iRowCode == FAIL )  break;						 
				printf("\n%-40s %-20s", szLastName, szFirstName); 
			}			 
			if( iRetCode == FAIL || iRowCode == FAIL ) {
	 			printf("\nError: Fetching Data\n");	
				exit(1);					// Exit With Error	
			}
		}	
	/*
	**********************************************************************
		Close The Connection To SQL Server And Exit
	**********************************************************************
	*/		
	dbclose( lpDbProc );					// Close The Connection
	exit(0);								// Exit The Program
}	
/*
**************************************************************************
	ErrorHandler() - Notifies The User If Any SQL Server Error Messages
	Are Received.
**************************************************************************
*/										
int SQLAPI ErrorHandler(DBPROCESS *lpDbProc, int iSeverity,int iDbError, 
						int iOsError, const char *lpszDbErrorMsg, 
						const char *lpszOsErrorMsg)
{	
   	printf( "Error:%d\nSeverity: %d\n\n%s\n%s\n", 
   			 iDbError,  iSeverity, lpszDbErrorMsg, lpszOsErrorMsg );
	if ((lpDbProc == NULL) || (DBDEAD(lpDbProc)))	// Test for a broken connection
		return(INT_EXIT);							// Terminate if connection broken
	return(INT_CANCEL);								// Return FAIL to DBLIB
}			
/*
**************************************************************************
	MessageHandler() - Notifies The User If Any SQL Server Messages
	Are Received.
**************************************************************************
*/										
int SQLAPI MessageHandler(DBPROCESS *lpDbProc, DBINT iMsgNumber,
						 int iMsgState, int iSeverity, 
						 const char *lpszMessageText, 
						 const char *szServerName, 
						 const char *szProcName, 
						 unsigned short iLineNumber){
    /*
    ***********************************************************
      Filter Out 5701  Database Context Changed ("Use DB")
		 5703  Database Language Changed
    ***********************************************************
    */
    if( iMsgNumber == 5701 || iMsgNumber == 5703 ) {
		return( 0 );
	} 
	printf( "Message: %ld\nState: %d\nSeverity: %d\n\n%s\n",
			iMsgNumber, iMsgState, iSeverity, lpszMessageText);	
	if ((lpDbProc == NULL) || (DBDEAD(lpDbProc))) {	// Check For Broken Connection
			printf("\nDead or NULL Database Connection");
	}	
	return(0);
}
