/*
******************************************************************
	
	SQLDEMO.EXE			Stewart MacLeod					2/22/96

	This simple program demonstrates how to create a multi-threaded
	Microsoft SQL Server 6.0 for Windows NT Server or Windows 95.

******************************************************************
*/			


/*
******************************************************************
	Specify The Include Files
******************************************************************
*/			

#define DBNTWIN32		// Tells SQL Server Its Windows NT Or Windows 95

#include <windows.h>	// Main Windows Header File
#include <windowsx.h>
#include <sqlfront.h>	// Every DBLIB program requires 
#include <sqldb.h>		// <sqlfront.h> and <sqldb.h>

#include <process.h>	// Win32 Process Info To Create Threads
#include <stdio.h>		// C Standard Library
#include "sqldemo.h"	// Our own header file

/*
*************************************************************************
	Global Variable Definitions
*************************************************************************
*/			
											  	
HANDLE					hStartFlag;						// Traffic Light
struct THREADINFO		ThreadInfo[SQL_MAX_THREADS];	// Thread Info
char 					szServerName[MAXSERVERNAME+1];	// SQL Server Name
char 					szDatabaseName[MAXNAME+1];		// DB Name
char					szUserCount[MAXNAME+1];			// User Count Str
DWORD 					dwUserCount;					// User Count Value
/*
**************************************************************************
	WinMain() - Main Window Function And Program Entry Point
**************************************************************************
*/			

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, 
					  LPSTR lpszCmdLine, int nCmdShow ) {

	INT		iRetCode;									// Return Code

	/*
	**********************************************************************
		Main Thread Runs With Above Normal Priority
	**********************************************************************
	*/
	
	if( ! SetThreadPriority( GetCurrentThread(),THREAD_PRIORITY_ABOVE_NORMAL ) ) {
	
		MessageBox( NULL, "Fatal Error: WinMain() Is Unable To SetThreadPriority", 
					SQL_APP_ERROR, MB_OK | MB_ICONSTOP );

		ExitProcess( SQL_FAILURE );
	}
	   	
	/*
	**********************************************************************
		Create The "Traffic Light" Using A Win32 Manual Reset Event.   
	**********************************************************************
	*/						 	

  	hStartFlag = CreateEvent( NULL, TRUE, FALSE, NULL );
    
	/*
	**********************************************************************
		Initialize DBLIBRARY And Install Our Error And Message Handlers	
	**********************************************************************
	*/
	
	dbinit(); // This returns a pointer to the version string	

	/*
	**********************************************************************
		Set the Defaults And Create The Main Dialog Box	
	**********************************************************************
	*/							  	

	if( strlen( lpszCmdLine ) )
		strcpy( szServerName, strupr(lpszCmdLine) );
	else
		GetMachineName( szServerName );

	strcpy( szDatabaseName, DEFAULT_DATABASE );

	strcpy( szUserCount, DEFAULT_USERS );

	iRetCode = DialogBox(hInstance, MAKEINTRESOURCE(IDD_SQLDIALOG), NULL,
						(int(__stdcall *)(void))Dlg_Proc );

	return( iRetCode );
}


/*
**************************************************************************
	CreateUser() - This function randomly selects a user's name, 
	application, and hostname.  Next it opens a connection (DBPROCESS) 
	to SQL Server. 	After the DBPROCESS is successfully opened, randomly 
	selected SQL statements (from GetSqlStatement()) are executed until
	the user stops the application by clicking on the "Stop" button.  
	
	The Stop button resets the (manual) WIN32 Event (hStartFlag) which is 
	used as a "traffic light".   When traffic light is red (hStartFlag event
	is non-signaled), each thread closes their DBPROCESS and terminates
	(suicide) gracefully via _endthreadex(). 
**************************************************************************
*/			

DWORD WINAPI CreateUser( LPVOID lpv )
{
	DBPROCESS	*lpDbProc;						// pointer To DBPROCESS
	LOGINREC	*login;							// pointer To Login Info
	BOOL		bError;							// Error Return Code
	DWORD		dwWaitState;					// Traffic Light's "Color"
	struct 		THREADINFO *ti;					// Thread Information
	CHAR		szMsg[MAX_STRING_SIZE];			// Reusable Message String
	CHAR		szUserName[MAXNAME];			// User Name
	CHAR		szAppName[MAXNAME];				// Application Name
	CHAR		szHostName[MAXNAME];			// Host (Machine) Name
	CHAR		szSqlStatement[MAX_SQL_STRING];	// SQL Text String

	/*
	**********************************************************************
		Each User Runs With Normal Priority
	**********************************************************************
	*/
	
	if( ! SetThreadPriority( GetCurrentThread(),THREAD_PRIORITY_NORMAL ) ) {
	
		MessageBox( NULL, "Error: CreateUser() Is Unable To SetThreadPriority", 
					SQL_APP_ERROR, MB_OK );

		_endthreadex( SQL_FAILURE );
	}
	   	
	/*
	**********************************************************************
		Use The Thread Number To Seed The Pseudo Random Number Generator
	**********************************************************************
	*/
		
	ti = (struct THREADINFO *) lpv;
	
	srand( ti->dwThreadParam );

	/*
	**********************************************************************
		Wait For The Traffic Light To Turn Green (WIN32 Event Is Signalled)
	**********************************************************************
	*/										

	dwWaitState = WaitForSingleObject( hStartFlag, INFINITE );	

	/*
	**********************************************************************
	 	Randomly Pick A User, Application, And Host Name To Use
	**********************************************************************
	*/	

	GetFirstName( szUserName );			// Get The User Name
	GetAppName( szAppName );			// Get The Application Name
	GetHostName( szHostName );			// And Host Name

	/*
	**********************************************************************
		Set The Defaults And Open A DBPROCESS To Connect To SQL Server
	**********************************************************************
	*/		

    login = dblogin();						// Allocate A Login Record
	dbsetlogintime( SQL_LOGIN_TIMEOUT );	// Specify Login Timeout (sec)
	dbsettime( SQL_QUERY_TIMEOUT );			// Specify Query Timeout (sec)
	DBSETLUSER( login, szUserName );		// Specify The User Name
	DBSETLPWD( login, szUserName );			// The Password is The User Name	
	DBSETLAPP( login, szAppName );			// Optional Name of Application
	DBSETLHOST (login, szHostName );		// Optional Computer Name
	DBSETLVERSION( login, DBVER60 );		// Require SQL Server 6.0 

	dbprocerrhandle( login, ErrorHandler ); // Use New Per Proc Error Handler
	dbprocmsghandle( login, MessageHandler );// And New Message Handler

	/*
	**********************************************************************
		Open The DBPROCESS To The Specified SQL Server	
	**********************************************************************
	*/	
	
	lpDbProc = dbopen( login, szServerName ); // Connect To SQL Server

	if( !lpDbProc ) {						// Unable To Connect?

		sprintf( szMsg, "Unable To Open DBRPOC for Thread %d\nServer %s User %s", 
				 ti->dwThreadParam , szServerName, szUserName );	

		MessageBox( NULL, szMsg, SQL_APP_ERROR, MB_OK | MB_ICONSTOP );

		_endthreadex( SQL_FAILURE );		// Exit With Error
	}
	
	/*
	**********************************************************************
		Use The Database Specified	
	**********************************************************************
	*/
	
	if( dbuse( lpDbProc, szDatabaseName ) == FAIL ){ // Unable To Use Database?

		sprintf( szMsg, "Unable To Use Database %s\nThread: %d\nServer: %s\nUser: %s", 
				 szDatabaseName, ti->dwThreadParam , szServerName, szUserName );	

		MessageBox( NULL, szMsg, SQL_APP_ERROR, MB_OK | MB_ICONSTOP );
	
		_endthreadex( SQL_FAILURE );		// Exit With Error
	}
	
	/*
	**********************************************************************
		Randomly Pick SQL Statements To Execute	Until The User Says STOP		
	**********************************************************************
	*/	

	for(;;) {
	
		GetSqlStatement( szSqlStatement );   // Grab A SQL Statement

		bError = ProcessSql( lpDbProc, szSqlStatement); // Execute it

		if( bError == SQL_FAILURE ) {		// Error Executing SQL 

			sprintf( szMsg, "Error Processing SQL Statement.\nSQL=%s\n",
					 szSqlStatement );

			MessageBox( NULL, szMsg, SQL_APP_ERROR, MB_OK | MB_ICONSTOP );						

			
			_endthreadex( SQL_FAILURE );	// Exit With Error
 		}

		/*
		******************************************************************
			Is It Already Time To Stop?
		******************************************************************
		*/
	
   		dwWaitState = WaitForSingleObject( hStartFlag, 0 );	// Check Event

		if( dwWaitState == WAIT_TIMEOUT ) {  // Not Signalled?
			
   			dbclose(lpDbProc);				// Close Connection To Server
			_endthreadex(SQL_SUCCESS);		// Exit
			break;
	
		}
	}
	
    return( SQL_SUCCESS );
}


/*
**************************************************************************
	ProcessSql() - This function compiles, executes, and processes
	the result(s) of the specified SQL statement(s).
**************************************************************************
*/													

BOOL ProcessSql( DBPROCESS *lpDbProc, char * SqlStatement )
{
	RETCODE		iResultCode;
	RETCODE		iRowCode;	
	DWORD		x;	
	DWORD		nCol;

	/*
	**********************************************************************
		Compile And Execute The SQL Statement	
	**********************************************************************
	*/
											 
	iResultCode = dbcmd(lpDbProc, SqlStatement ); // Put Command In Buffer

	if( iResultCode == FAIL ) {

		dbcancel( lpDbProc );  // Cancel the query and return error
		return( SQL_FAILURE );

	}	
	
	iResultCode = dbsqlexec(lpDbProc);  // Send To Server and Execute

	if( iResultCode == FAIL ) {	// Error?

		dbcancel( lpDbProc );  // Cancel the query and return error
		return( SQL_FAILURE );

	}	

	/*
	**********************************************************************
		Fetch Any And All Results	
	**********************************************************************
	*/							 	
		
	while ((iResultCode = dbresults(lpDbProc)) != NO_MORE_RESULTS)
	 {
		if (iResultCode == FAIL ){
			break;
		}
							   
		/*
		******************************************************************
			If This Statement Can Return Data, Then Fetch All Rows
		******************************************************************
		*/
	
		if( DBCMDROW(lpDbProc) == SUCCEED ) {

			nCol = dbnumcols(lpDbProc);  // get number of columns

			while ( (iRowCode = dbnextrow(lpDbProc)) != NO_MORE_ROWS)
				{				 				
					if( iRowCode == FAIL ) {
					    break;
					}

					for( x = 1; x <= nCol; x++ ) { // for each column

						dbdata( lpDbProc, x ); // get the column's data

						/*
						**************************************************
						 process each row and column here.
						**************************************************
						*/

					}
				}
		}		
	 }
	
	if( iResultCode == FAIL || iRowCode == FAIL ) {

		dbcancel( lpDbProc );  // Cancel the query and return error
		return( SQL_FAILURE );

	}
	else {  	

		return( SQL_SUCCESS );
	}
}


/*
**************************************************************************
	ErrorHandler() - Notifies The User If Any SQL Server Error Messages
	Are Received.
**************************************************************************
*/										
int SQLAPI ErrorHandler(DBPROCESS *lpDbProc, int iSeverity,int iDbError, 
						int iOsError, const char *lpszDbErrorMsg, 
						const char *lpszOsErrorMsg)
{
    CHAR		szStringBuffer[MAX_STRING_SIZE];

	
   	sprintf( szStringBuffer, "Error:%d\nSeverity: %d\n\n%s\n%s\n", 
   			 iDbError,  iSeverity, lpszDbErrorMsg, lpszOsErrorMsg );

	MessageBox( NULL, szStringBuffer, "SQL Server Error",  
				MB_OK | MB_ICONSTOP );				// Display Message box for error
	
	if ((lpDbProc == NULL) || (DBDEAD(lpDbProc)))	// Test for a broken connection
		return(INT_EXIT);							// Terminate if connection broken
		
	return(INT_CANCEL);								// Return FAIL to DBLIB
}

/*
**************************************************************************
	MessageHandler() - Notifies The User If Any SQL Server Messages
	Are Received.
**************************************************************************
*/										

int SQLAPI MessageHandler(DBPROCESS *lpDbProc, DBINT iMsgNumber,
						 int iMsgState, int iSeverity, 
						 const char *lpszMessageText, 
						 const char *szServerName, 
						 const char *szProcName, 
						 unsigned short iLineNumber)
{

    CHAR		szStringBuffer[MAX_STRING_SIZE];
    
    /*
    ***********************************************************
      Filter Out 5701  Database Context Changed ("Use DB")
		 5703  Database Language Changed
    ***********************************************************
    */

    if( iMsgNumber == 5701 || iMsgNumber == 5703 ) {
		return( 0 );
	}
		

	sprintf( szStringBuffer, "Message: %ld\nState: %d\nSeverity: %d\n\n%s\n",
			iMsgNumber, iMsgState, iSeverity, lpszMessageText);
		

	MessageBox( NULL, szStringBuffer, "SQL Server Message", 
				MB_OK | MB_ICONSTOP );
	
	if ((lpDbProc == NULL) || (DBDEAD(lpDbProc))) {	// Check For Broken Connection
			printf("\nDead or NULL Database Connection");
	}
	
	return(0);
}


/*
**************************************************************************
	Dlg_OnInitDialog() - This function initializes and sets the defaults
	for the main dialog box.	
**************************************************************************
*/													

BOOL Dlg_OnInitDialog( HWND hWnd, HWND hWndFocus, LPARAM lParam) {

    	
	SetWindowText(GetDlgItem(hWnd, IDC_EDITSERVER), szServerName );
	SetWindowText(GetDlgItem(hWnd, IDC_EDITDATABASE), szDatabaseName );
	SetWindowText(GetDlgItem(hWnd, IDC_EDITUSERS), szUserCount );

 	EnableWindow(GetDlgItem(hWnd, IDC_START), TRUE );
	EnableWindow(GetDlgItem(hWnd, IDC_STOP), FALSE );
	EnableWindow(GetDlgItem(hWnd, IDC_EXIT), TRUE );

	SetFocus(GetDlgItem(hWnd, IDC_EDITSERVER));

	return( FALSE );
}


/*
**************************************************************************
	Dlg_OnCommand() - This is the main function for the SQLUSER.EXE
	program.  
	
	1. The user specifies the desired SQLServerName, DatabaseName, 
	and UserCount.   When they choose "Start", a WIN32 thread is created 
	for each user specified. CreateUser() will not start until the 
	"Traffic Light", hStartFlag WIN32 Event, is signaled and
	the "traffic light" is green.  After the WIN32 event,
	hStartFlag, is signaled the "traffic light" is green and the 
	users start connecting to SQL Server.

	2. Random SQL Statements are executed until the user clicks on
	the "Stop" button.  When the user clicks on "Stop", the hStartFlag is 
	reset (non-signalled) and the "traffic light" turns red.   This tells
	each thread to terminate gracefully via _endthreadex().

	3. When the user clicks on "Exit", the dialog box is destroyed and
	the SQLDEMO.EXE terminates.
**************************************************************************
*/										
void Dlg_OnCommand( HWND hWnd, int id, HWND hWndCtl, UINT CodeNotify ) {

	CHAR 		szStringBuffer[MAX_STRING_SIZE];
	DWORD		dwCurrentThread;
	BOOL		bError;	 
	
	switch(id) {

		case IDC_START:
		
			GetWindowText(GetDlgItem(hWnd, IDC_EDITSERVER), szServerName, sizeof(szServerName));
			GetWindowText(GetDlgItem(hWnd, IDC_EDITDATABASE), szDatabaseName, sizeof(szDatabaseName));
			GetWindowText(GetDlgItem(hWnd, IDC_EDITUSERS), szUserCount, sizeof(szUserCount));

			dwUserCount = atol( szUserCount );
			
			if( dwUserCount < 1 ) {
			
				MessageBox( NULL, 
				"Invalid Number Of Users Specified.\nMust Be Greater Than Zero",
				SQL_APP_ERROR, MB_OK | MB_ICONSTOP );
				break;
			}

			if( dwUserCount > MAX_USERS ) {
			
				MessageBox( NULL, 
				"Invalid Number Of Users Specified.\nNo More Than 15 Allowed",
				SQL_APP_ERROR, MB_OK | MB_ICONSTOP );
				break;
			}
			
			if( dbsetmaxprocs( (INT) dwUserCount ) == FAIL ) { // Max number of Threads
					MessageBox( NULL, 						   // per application
					"Error configuring maximum number of DBPROCESSES",
					SQL_APP_ERROR, MB_OK | MB_ICONSTOP );
					break;
			
			}

		    EnableWindow( GetDlgItem(hWnd, IDC_START ), FALSE );	
			EnableWindow( GetDlgItem(hWnd, IDC_STOP), TRUE );	
			EnableWindow( GetDlgItem(hWnd, IDC_EXIT), FALSE );	

			SetFocus(GetDlgItem(hWnd, IDC_STOP));

		    EnableWindow( GetDlgItem(hWnd, IDC_EDITSERVER ), FALSE );	
			EnableWindow( GetDlgItem(hWnd, IDC_EDITDATABASE ), FALSE );	
			EnableWindow( GetDlgItem(hWnd, IDC_EDITUSERS ), FALSE );		
	
			for( dwCurrentThread=0; dwCurrentThread < dwUserCount; dwCurrentThread++) {

				ThreadInfo[dwCurrentThread].dwThreadParam = dwCurrentThread;

				ThreadInfo[dwCurrentThread].hThread = _beginthreadex(  //create the thread
					NULL,
					THREAD_STACK_SIZE, 
					(unsigned int(__stdcall *)(void*))CreateUser, 
					(LPVOID) &ThreadInfo[dwCurrentThread], 
					0,
					&ThreadInfo[dwCurrentThread].dwThreadId );
				
				if( !&ThreadInfo[dwCurrentThread].hThread ){  // die on error

					sprintf( szStringBuffer, "Unable To Create Thread %s", dwCurrentThread );

					MessageBox( NULL, szStringBuffer, 
								SQL_APP_ERROR, MB_OK | MB_ICONSTOP );

					exit(SQL_FAILURE);
				}
			}								   	
				
			bError = SetEvent( hStartFlag ); // Set the "Traffic Light" to GREEN!
	
			break;

		case IDC_STOP:						  
			
			bError = ResetEvent( hStartFlag ); // Set the "Traffic Light" to RED!
			   
			EnableWindow( GetDlgItem(hWnd, IDC_START ), TRUE );	
			EnableWindow( GetDlgItem(hWnd, IDC_STOP), FALSE );	
			EnableWindow( GetDlgItem(hWnd, IDC_EXIT), TRUE );	

			EnableWindow( GetDlgItem(hWnd, IDC_EDITSERVER ), TRUE );	
			EnableWindow( GetDlgItem(hWnd, IDC_EDITDATABASE ), TRUE );	
			EnableWindow( GetDlgItem(hWnd, IDC_EDITUSERS ), TRUE );	
			
			break;

		case IDC_EXIT:

			EndDialog(hWnd, id);	// Destroy The Dialog and Exit

			break;		

	 }
			
}		

/*
**************************************************************************
	Dlg_Proc() - Dialog procedure to handle dialog box messages
**************************************************************************
*/										

BOOL CALLBACK Dlg_Proc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam ) {

	BOOL	fProcessed = TRUE;
		
	switch( uMsg ){

		HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog );
		HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand );
		
		default:

			fProcessed = FALSE;
			break;
		
	}

	return( fProcessed );
	
}		

/*
**************************************************************************
	GetMachineName() - Gets the Machine Name from the Registry
**************************************************************************
*/										
																   
BOOL GetMachineName(char *lpszMachine)
{
 ULONG	ulRegError;
 HKEY	hkRegKey;
 DWORD	dwRegSize;
 char 	szMsg[80];
 char   szKey[] = "SYSTEM\\CurrentControlSet\\Control\\ComputerName\\ComputerName";
 char   szValue[]="ComputerName";
 
 *lpszMachine = '\0';

 ulRegError = RegOpenKey( HKEY_LOCAL_MACHINE, szKey, &hkRegKey);

 if(ulRegError != ERROR_SUCCESS ) {
 	sprintf( szMsg, "\nError Opening Registry Key: %s", szKey );
	MessageBox( NULL, szMsg, "Error", MB_OK | MB_ICONSTOP );
 	return(FALSE);
 }

 dwRegSize = sizeof( szMsg );
 
 ulRegError = RegQueryValueEx( hkRegKey, szValue,
			 	NULL, NULL, 
			 	(unsigned char *)lpszMachine, &dwRegSize);

 if(ulRegError != ERROR_SUCCESS ) {
 	MessageBox( NULL, "Error Reading Current Machine Name", "Error", MB_OK | MB_ICONSTOP );
 	return(FALSE);
 }

ulRegError = RegCloseKey(hkRegKey);

if(ulRegError != ERROR_SUCCESS ) {
 	 	MessageBox( NULL, "Error Closing Registry Current Machine Name", "Error", MB_OK | MB_ICONSTOP );
 	return(FALSE);
 }

return( TRUE );
}


/*
**************************************************************************
	List Of User Names.  We Assume Their Password Is Just Their Name
**************************************************************************
*/										

char *lpszFirst[] =
{
"Andrea","Andrew","Charles","Chris","Dennis","Ellen","Jerry","Kevin",
"Kate","Laurie","Mary","Patrick","Stewart","Tim","Wendy"
};

/*
**************************************************************************
	A List Of Application Names
**************************************************************************
*/										

char *AppNames[] = {
"WinPlus","FinanceTrak","SalesTrak","MfgPlanner","Inventory","Access","Excel"
};

/*
**************************************************************************
	A List Of Machine Names HOSTNAME
**************************************************************************
*/										

char *HostNames[] = {
"BLDG03-0123","BLDG26-3440","BLDG12-4213","BLDG26-0003","BLDG01-0013",
"BLDG16-1083","BLDG01-1144",
};

/*
**************************************************************************
	A List Of Queries To Execute Against The PUBS Database
**************************************************************************
*/										


char *SqlQueries[] = {
"select au_fname, au_lname, phone from authors where au_lname like \'W%\'",
"select au_fname, au_lname, city, state, zip, phone from authors order by au_lname, au_fname",
"select au_lname, title from authors a, titleauthor ta, titles t where a.au_id = ta.au_id and t.title_id = ta.title_id and a.state = \'CA\' order by au_lname",
"select count(*) from titleauthor",
"select * from sales",
"select pub_name, city, state from publishers where state = \'CA\'",
"select stor_id, sum(qty) from sales group by stor_id",
"select state, count(*) from authors group by state",
"select au_fname, au_lname, phone from authors where au_id = \'409-56-7008\'",
"select au_fname, au_lname, phone from authors where au_id = \'267-41-2394\'", 
"select au_fname, au_lname, phone from authors where au_id = \'672-71-3249\'", 
"select au_fname, au_lname, phone from authors where au_lname = \'Carson\' order by au_lname, au_fname",                                   
"select au_fname, au_lname, phone from authors where au_lname = \'DeFrance\' order by au_lname, au_fname",                                 
"select au_fname, au_lname, phone from authors where au_lname = \'del Castillo\' order by au_lname, au_fname",                             
"select au_fname, au_lname, phone from authors where au_lname = \'Dull\' order by au_lname, au_fname",                                     
"select au_fname, au_lname, phone from authors where au_lname = \'Green\' order by au_lname, au_fname",                                    
"select au_fname, au_lname, phone from authors where au_lname = \'Greene\' order by au_lname, au_fname",                                   
"select au_fname, au_lname, phone from authors where au_lname = \'Gringlesby\' order by au_lname, au_fname",                               
"select city, count(*) from authors where state = \'CA\' group by city",  
"select city, count(*) from authors where state = \'UT\' group by city",  
"select title, au_lname, price, ytd_sales from authors a, titleauthor ta, titles t where a.au_id = ta.au_id and t.title_id = ta.title_id and t.type = \'trad_cook\'", 
"select title, au_lname, price, ytd_sales from authors a, titleauthor ta, titles t where a.au_id = ta.au_id and t.title_id = ta.title_id and t.type = \'UNDECIDED\'", 
"select pub_name, city, state from publishers",
"select stor_name, t.title_id, \'sales\' = sum(s.qty * t.price ) from sales s, titles t, stores r where s.title_id = t.title_id and r.stor_id = r.stor_id group by r.stor_name, t.title_id", 
"select count(*) from  authors",                        
"select count(*) from  discounts",                      
"select count(*) from  employee",                       
"select count(*) from  jobs",                           
"select count(*) from  publishers",                     
"select count(*) from  roysched",                       
"select count(*) from  stores",                         
"select count(*) from  titleauthor",                    
"select count(*) from  titles"                         
};


/*
**************************************************************************
	GetAppName() - Randomly Select An Application Name To Use
**************************************************************************
*/													   
void GetAppName( char * lpszAppName )
{
  int max_entry;
  int index;

  max_entry = sizeof( AppNames ) / sizeof (char *);
  index = rand() % max_entry;

  strcpy( lpszAppName, (char *) *(AppNames + index) );
}

/*
**************************************************************************
	GetSqlStatement() - Randomly Select An SQL Statement To Execute
**************************************************************************
*/													   
void GetSqlStatement( char * lpszSqlStatement )
{
  int max_entry;
  int index;

  max_entry = sizeof( SqlQueries ) / sizeof (char *);
  index = rand() % max_entry;

  strcpy( lpszSqlStatement, (char *) *(SqlQueries + index) );
}

/*
**************************************************************************
	GetHostName() - Randomly Select A Host Name To Use For The "Machine"
**************************************************************************
*/													   
void GetHostName( char * lpszHostName )
{
  int max_entry;
  int index;

  max_entry = sizeof( HostNames ) / sizeof (char *);
  index = rand() % max_entry;

  strcpy( lpszHostName, (char *) *(HostNames + index) );
}

/*
**************************************************************************
	GetFirstName() - Randomly Select A User Name To Use
**************************************************************************
*/													   
void GetFirstName( char * lpszFirstName )
{
  int max_entry;
  int index;

  max_entry = sizeof( lpszFirst ) / sizeof (char *);
  index = rand() % max_entry;

  strcpy( lpszFirstName, (char *) *(lpszFirst + index) );
}
													   
/*
**************************************************************************
	End Of SQLDEMO
**************************************************************************
*/				
