/*
*************************************************************
Stewart MacLeod			SQL Server 6.X		DRI

Debit Savings Credit Checking transaction using DRI

*************************************************************
*/

if exists (select * from sysobjects where name = 'savings' and type = 'U')
begin
	drop table savings
end
go

if exists (select * from sysobjects where name = 'checking' and type = 'U')
begin
	drop table checking
end
go


if exists (select * from sysobjects where name = 'customer' and type = 'U')
begin
	drop table customer
end
go


create table customer
(
	custid	integer 
		not null 
		identity( 1,1)
		constraint pk_customer 
		primary key clustered,
	name	character(30) 
		not null,
	phone	character(14)
		not null 
		default '(000) 000-0000'
		constraint ck_phone 
		check (phone like '([2-9][0-9][0-9]) [0-9][0-9][0-9]-[0-9][0-9][0-9][0-9]')
	
)
go

insert into customer(name,phone) values ('stewart','(206) 936-7374')
insert into customer(name,phone) values ('deby','(206) 936-4616')
insert into customer(name,phone) values ('gary','(206) 936-3615')
insert into customer(name,phone) values ('ron','(206) 936-4433')
insert into customer(name,phone) values ('jane','(206) 936-4216')
insert into customer(name,phone) values ('fred','(206) 936-1215')
go


create table savings
(
	custid	integer 
		not null 
		constraint fk_savings_custid foreign key 
		references customer(custid),
	balance	decimal(10,2) 
		not null
		constraint ck_savings_balance check (balance > 50)
	
)
go

insert into savings(custid,balance) values (1, 1000.00)
insert into savings(custid,balance) values (2, 2000.00)
insert into savings(custid,balance) values (3, 3000.00)
insert into savings(custid,balance) values (4, 4000.00)
insert into savings(custid,balance) values (5, 5000.00)
insert into savings(custid,balance) values (6, 6000.00)
go

create table checking
(
	custid	integer 
		not null 
		constraint fk_checking_custid foreign key 
		references customer(custid),
	balance	decimal(10,2) 
		not null
		constraint ck_checking_balance check (balance > 50)
	
)
go

insert into checking(custid,balance) values (1, 1000.00)
insert into checking(custid,balance) values (2, 2000.00)
insert into checking(custid,balance) values (3, 3000.00)
insert into checking(custid,balance) values (4, 4000.00)
insert into checking(custid,balance) values (5, 5000.00)
insert into checking(custid,balance) values (6, 6000.00)
go

declare @err integer
declare @ctr integer

begin transaction

	update savings set balance = balance - 100 where custid = 1

	select @err = @@error, @ctr = @@rowcount

	if( @err <> 0 )
	begin
		rollback tran
		goto done
	end

	if( @ctr <> 1 )
	begin
		rollback tran
		goto done
	end

	update checking set balance = balance + 100 where custid = 1

	select @err = @@error

	if( @err <> 0 )
	begin
		rollback tran
		goto done
	end
	
	if( @ctr <> 1 )
	begin
		rollback tran
		goto done
	end


commit transaction

done:

if(@err = 0 and @ctr = 1)
	select 'Success'
else
	select 'Failure'

go

/*
***************************************************************
drop all the tables
***************************************************************
*/

if exists (select * from sysobjects where name = 'savings' and type = 'U')
begin
	drop table savings
end
go

if exists (select * from sysobjects where name = 'checking' and type = 'U')
begin
	drop table checking
end
go


if exists (select * from sysobjects where name = 'customer' and type = 'U')
begin
	drop table customer
end
go


