if exists (select * from sysobjects where id = object_id('dbo.sp_processinfo') and sysstat & 0xf = 4)
	drop procedure dbo.sp_processinfo
GO

create procedure sp_processinfo
as
/* ====================================================*/
/* sp_processinfo - returns detailed lock              */
/* information for the server                          */
/* Microsoft SQL Server - stewartm	               */
/* ====================================================*/

declare @lkdbnm varchar(32),@lkdbid smallint,@lkobjid int,@UserDBID smallint,@usrdbnm varchar(32),@User_ID smallint, @grpid smallint, @stmt varchar(255)

/* ===========================================================*/
/* build temp table with processes and their associated locks */
/* ===========================================================*/

select
	'FullName'	= case when p.spid > 9 
                    then lo.name 
                    else 'System' 
                  end,'
     UserID'= p.uid,
	'UName'	= '                                ','ProcessID'= p.spid,
    'Status'= p.status,	'DBID'		= p.dbid,'DbName'= d.name,
    'Command'= p.cmd,'Host'= p.hostname,'Application'= p.program_name,
	'Blocking'	= case when l.type >= 256 
                    then 100 
                    else 0 
                  end,
    'Blockedby'= p.blocked,'LockType'	= l.type,
	'LKObjDBID'	= l.dbid,'LKObjDB'	= '                                ','LKObjID'	= l.id,
	'LKObj'		= case when l.id = 0 
                    then (select name from spt_values where number = 0 
                        and type = 'G') 
                    else '                                                                                     ' 
                  end,
	'LockedPage'	= l.page,'GroupID'	= p.gid,'GName'	= '                                ','CPUUsage'	= p.cpu,
	'PhysicalIO'	= p.physical_io,'HostProcess'	= p.hostprocess
    into #info 
    from master..sysprocesses p,	
         master..syslocks l,
         master..syslogins lo,
         master..sysdatabases d
    where	p.spid 	*= l.spid 
        and p.suid = lo.suid 
        and p.dbid= d.dbid

/* ======================= */
/* flag blocking processes */	
/* ======================= */

update #info set Blocking = 1 where ProcessID in (select Blockedby from #info where Blockedby > 0)

/* =============================================*/
/* get list of locked object IDs */
/* =============================================*/

exec ('declare c1 cursor for select distinct LockedDBName = d.name, LKObjID , LKObjDBID from #info, master..sysdatabases d
where LKObjDBID = d.dbid  and LKObjID > 0 FOR READ ONLY')

open c1

fetch c1 into @lkdbnm, @lkobjid, @lkdbid

while @@fetch_status >= 0
   begin
	/* set database.owner.name of locked objects in #info */
	select @stmt ='Update #info set LKObj=' + "'" + @lkdbnm + "'" + '+ "." + u.name + "." + o.name' + 
 	',LKObjDB=' + "'"+@lkdbnm + "'" + ' from '+@lkdbnm+'..sysobjects o,'+@lkdbnm+'..sysusers u ' +
	'where o.id='+convert(char(10),@lkobjid)+ 
	' and LKObjID=o.id' +  
	' and u.uid=o.uid and LKObjDBID=' + convert(char,@lkdbid)  
	exec(@stmt )
	/* next please */
	fetch c1 into @lkdbnm, @lkobjid, @lkdbid
   end
deallocate c1

/* =============================================*/
/* resolve UserID, GroupID for all processes    */
/* get a list of each database we need to visit */
/* =============================================*/

exec ('declare c1 cursor for select distinct DBID, DbName from #info FOR READ ONLY')

open c1

fetch c1 into  @UserDBID, @usrdbnm

while @@fetch_status >= 0
   begin
	/* set user/group in #info */
	select @stmt = 'Update #info set GName=g.name,UName=u.name from '+@usrdbnm+'..sysusers u,'+
	@usrdbnm+'..sysusers g,#info '+	'where u.uid=UserID and g.uid=GroupID and DBID=' + convert(char(10),@UserDBID)
	exec(@stmt )
	/* next please */
	fetch c1 into  @UserDBID, @usrdbnm
   end
deallocate c1

/* ====== return the #info table ===== */
SELECT
	FullName , ProcessID, 	Status,	DName = DbName,	Command,Host, Application, Blockedby,LockType,LKObj,
	GName = substring(GName,1,10),CPUUsage, PhysicalIO, HostProcess,UName,Blocking
FROM #info ORDER BY FullName, ProcessID

return(0)

GO

GRANT  EXECUTE  ON dbo.sp_processinfo  TO public
GO

