/*
******************************************************************
	
	EXAMPLE_XP.DLL			Stewart MacLeod				2/22/96

	This simple program demonstrates how to create a SQL
    Server Extended Stored Procedure.   

    exec master..xp_drive_info

    returns the fixed drives, total size, and available space.
 

******************************************************************
*/			
/*
******************************************************************
	Specify The Include Files and manifest constants
******************************************************************
*/			
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#ifdef __cplusplus
	extern "C" {
    #include <srv.h>
}
#endif

#define  XP_SUCCESS		0
#define  XP_FAILURE		1
#define  MAXNAME       30
#define  COL1_NAME      "Drive"
#define  COL2_NAME      "Total Size"
#define  COL3_NAME      "Available Space"

#define  XP_DRIVE_INFO_ERROR_NUMBER     50000
#define  XP_DRIVE_INFO_SYNTAX_ERROR     "Syntax Error: xp_drive_info() does take any parameters"
#define  XP_GET_DRIVE_INFO_ERROR        "Error getting disk information: xp_drive_info()"
/*
**************************************************************************
	GetDriveInformation() - Gets the total and available disk space
**************************************************************************
*/                                                                              
RETCODE xp_drive_info(SRV_PROC * lpServerProc)
{
	DBFLT4   floatTotalDisk;
    DBFLT4   floatAvailableDisk; 
    LPSTR    lpszDriveStrings;
	DWORD    dwDrives;
	DWORD    dwLength = 0;
	DWORD    dwSectorsPerCluster;
	DWORD    dwBytesPerSector;
	DWORD    dwFreeClusters;
	DWORD    dwClusters;
    DBCHAR   szDriveLetter[MAXNAME];
    DWORD    dwCount=0;
    DWORD    iParamCount;
    /*
    ******************************************************************
        Get XP parameter count
    ******************************************************************
    */
    iParamCount = srv_rpcparams(lpServerProc);
    /*
    ******************************************************************
        Fail and return error message if invalid param
    ******************************************************************
    */    
    if (iParamCount != -1) {       // -1 == no parameters
         /*
         *************************************************************
            Send a message back to the client
         **************************************************************
         */        
         srv_sendmsg(lpServerProc, SRV_MSG_ERROR, XP_DRIVE_INFO_ERROR_NUMBER, SRV_INFO, (DBTINYINT)0,
                    NULL, 0, 0, XP_DRIVE_INFO_SYNTAX_ERROR, SRV_NULLTERM);	     
         /*
         *************************************************************
            Indicates that no more rows are coming in results set
         **************************************************************
         */                
         srv_senddone(lpServerProc, (SRV_DONE_ERROR | SRV_DONE_MORE), 0, 0);
         /*
         *************************************************************
            Return status variable  exec @var = xp_function
         **************************************************************
         */                
	     return(XP_FAILURE);   
    }

	floatTotalDisk = floatAvailableDisk = (DBFLT4) 0.0000;    
	dwDrives = GetLogicalDriveStrings( 0, NULL );
	lpszDriveStrings = ( char *)malloc( dwDrives );
    /*
    ******************************************************************
        Fail if no memory
    ******************************************************************
    */          
	if( ! lpszDriveStrings ) {
        srv_sendmsg(lpServerProc, SRV_MSG_ERROR, XP_DRIVE_INFO_ERROR_NUMBER, SRV_INFO, (DBTINYINT)0,
                    NULL, 0, 0, XP_GET_DRIVE_INFO_ERROR, SRV_NULLTERM);	     
        srv_senddone(lpServerProc, (SRV_DONE_ERROR | SRV_DONE_MORE), 0, 0);
         
		return XP_FAILURE;
	}            
    /*
    ******************************************************************
        Describe each column name, datatype, length, and location
    ******************************************************************
    */      
    srv_describe(lpServerProc, 1, COL1_NAME, SRV_NULLTERM, SRVCHAR, MAXNAME, SRVCHAR, strlen(szDriveLetter),
                 (BYTE *)&szDriveLetter);
    
    srv_describe(lpServerProc, 2, COL2_NAME, SRV_NULLTERM, SRVFLT4, 4, SRVFLT4, 4,
                 (BYTE *)&floatTotalDisk);
    
    srv_describe(lpServerProc, 3, COL3_NAME, SRV_NULLTERM, SRVFLT4, 4, SRVFLT4, 4,
                 (BYTE *)&floatAvailableDisk);

	dwDrives = GetLogicalDriveStrings( dwDrives, lpszDriveStrings );
	for( ;; ) {
		if(GetDriveType( lpszDriveStrings + dwLength ) == DRIVE_FIXED ) {
			if( (GetDiskFreeSpace( lpszDriveStrings + dwLength,
					&dwSectorsPerCluster, &dwBytesPerSector, 
					&dwFreeClusters, &dwClusters )) == FALSE ) {             
				floatTotalDisk = floatAvailableDisk = (DBFLT4)0.0000;                                     				
			}
            else {
			    floatTotalDisk = (DBFLT4) ((dwBytesPerSector * dwSectorsPerCluster * dwClusters) / (1024.0 * 1024.0) );
			    floatAvailableDisk = (DBFLT4) ((dwBytesPerSector * dwSectorsPerCluster * dwFreeClusters) / (1024.0 * 1024.0) );
            }
            dwCount++;
            /*
            *************************************************************
                Send the row back to the client
            **************************************************************
            */                
            strcpy( szDriveLetter, lpszDriveStrings + dwLength );
            srv_setcollen(lpServerProc, 1, strlen(szDriveLetter) );		
            srv_sendrow(lpServerProc);		
		}            
        
        dwLength += strlen( lpszDriveStrings + dwLength ) + 1;  
		/*
        *************************************************************
           Mark end of results set
        **************************************************************
        */                        
        if( ! strlen( lpszDriveStrings + dwLength ) ){
            srv_senddone(lpServerProc, (SRV_DONE_COUNT | SRV_DONE_MORE), 0, dwCount);
			break;
	    }		
    }
	return (XP_SUCCESS);
}
